Package: Moonlight2R
Type: Package
Title: Identify oncogenes and tumor suppressor genes from omics data
Version: 1.11.0
Date:
Authors@R: 
  c(person("Mona", "Nourbakhsh",
           role="aut"),
    person("Astrid", "Saksager",
           role="aut"),
    person("Nikola", "Tom",
           role="aut"),
    person("Katrine", "Meldgård",
           role="aut"),
    person("Anna", "Melidi",
           role="aut"),
    person("Xi Steven", "Chen",
           role="aut"),
    person("Antonio", "Colaprico",
           role="aut"),
    person("Catharina", "Olsen",
           role="aut"),
    person("Alessia", "Campo",
           role=c("aut")),
    person("Matteo", "Tiberti",
           role=c("cre", "aut"),
	   email="tiberti@cancer.dk"),
    person("Elena", "Papaleo",
	   role="aut",
    	   ))
Depends: R (>= 4.5), doParallel, foreach
Imports: parmigene, randomForest, gplots, circlize, RColorBrewer,
        HiveR, clusterProfiler, DOSE, Biobase, grDevices, graphics,
        GEOquery, stats, purrr, RISmed, grid, utils, ComplexHeatmap,
        GenomicRanges, dplyr, fuzzyjoin, rtracklayer, magrittr, qpdf,
        readr, seqminer, stringr, tibble, tidyHeatmap, tidyr,
        AnnotationHub, easyPubMed, org.Hs.eg.db, EpiMix, BiocGenerics,
        ggplot2, ExperimentHub, rlang, withr, data.table, fgsea
Description: The understanding of cancer mechanism requires the
        identification of genes playing a role in the development of
        the pathology and the characterization of their role (notably
        oncogenes and tumor suppressors). We present an updated version
        of the R/bioconductor package called MoonlightR, namely
        Moonlight2R, which returns a list of candidate driver genes for
        specific cancer types on the basis of omics data integration.
        The Moonlight framework contains a primary layer where gene
        expression data and information about biological processes are
        integrated to predict genes called oncogenic mediators, divided
        into putative tumor suppressors and putative oncogenes. This is
        done through functional enrichment analyses, gene regulatory
        networks and upstream regulator analyses to score the
        importance of well-known biological processes with respect to
        the studied cancer type. By evaluating the effect of the
        oncogenic mediators on biological processes or through random
        forests, the primary layer predicts two putative roles for the
        oncogenic mediators: i) tumor suppressor genes (TSGs) and ii)
        oncogenes (OCGs). As gene expression data alone is not enough
        to explain the deregulation of the genes, a second layer of
        evidence is needed. We have automated the integration of a
        secondary mutational layer through new functionalities in
        Moonlight2R. These functionalities analyze mutations in the
        cancer cohort and classifies these into driver and passenger
        mutations using the driver mutation prediction tool,
        CScape-somatic. Those oncogenic mediators with at least one
        driver mutation are retained as the driver genes. As a
        consequence, this methodology does not only identify genes
        playing a dual role (e.g. TSG in one cancer type and OCG in
        another) but also helps in elucidating the biological processes
        underlying their specific roles. In particular, Moonlight2R can
        be used to discover OCGs and TSGs in the same cancer type. This
        may for instance help in answering the question whether some
        genes change role between early stages (I, II) and late stages
        (III, IV). In the future, this analysis could be useful to
        determine the causes of different resistances to
        chemotherapeutic treatments. An additional mechanistic layer
        evaluates if there are mutations affecting the protein
        stability of the transcription factors (TFs) of the TSGs and
        OCGs, as that may have an effect on the expression of the
        genes.
License: GPL-3
biocViews: DNAMethylation, DifferentialMethylation, GeneRegulation,
        GeneExpression, MethylationArray, DifferentialExpression,
        Pathways, Network, Survival, GeneSetEnrichment,
        NetworkEnrichment
Suggests: BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0), devtools,
        roxygen2, png
SystemRequirements: CScapeSomatic
VignetteBuilder: knitr
URL: https://github.com/ELELAB/Moonlight2R
BugReports: https://github.com/ELELAB/Moonlight2R/issues
RoxygenNote: 7.3.3
LazyData: false
Encoding: UTF-8
Config/testthat/edition: 3
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev libglpk-dev libglu1-mesa-dev make texlive
        libbz2-dev libicu-dev libjpeg-dev liblzma-dev libpng-dev
        libuv1-dev libxml2-dev libgl1-mesa-dev libssl-dev perl
        libx11-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 13:01:32 UTC
RemoteUrl: https://github.com/bioc/Moonlight2R
RemoteRef: HEAD
RemoteSha: f947d16ced3766bab9e78fc0aac8a4a9e9137319
NeedsCompilation: no
Packaged: 2026-05-30 09:20:46 UTC; root
Author: Mona Nourbakhsh [aut],
  Astrid Saksager [aut],
  Nikola Tom [aut],
  Katrine Meldgård [aut],
  Anna Melidi [aut],
  Xi Steven Chen [aut],
  Antonio Colaprico [aut],
  Catharina Olsen [aut],
  Alessia Campo [aut],
  Matteo Tiberti [cre, aut],
  Elena Papaleo [aut]
Maintainer: Matteo Tiberti <tiberti@cancer.dk>
Built: R 4.6.0; ; 2026-05-30 13:03:00 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DMA)
export(FEA)
export(GLS)
export(GMA)
export(GRN)
export(GSEA)
export(LPA)
export(LiftMAF)
export(MAFtoCscape)
export(PRA)
export(PRAtoTibble)
export(RunCscape_somatic)
export(TFinfluence)
export(URA)
export(confidence)
export(getDataGEO)
export(loadMAVISp)
export(moonlight)
export(plotCircos)
export(plotDMA)
export(plotFEA)
export(plotGMA)
export(plotHeatmap)
export(plotMetExp)
export(plotMoonlight)
export(plotMoonlightMet)
export(plotNetworkHive)
export(plotURA)
import(ComplexHeatmap)
import(EpiMix)
import(ExperimentHub)
import(HiveR)
import(RISmed)
import(doParallel)
import(dplyr)
import(easyPubMed)
import(foreach)
import(org.Hs.eg.db)
import(parallel)
import(readr)
import(tibble)
import(tidyHeatmap)
import(tidyr)
importFrom(AnnotationHub,AnnotationHub)
importFrom(BiocGenerics,as.data.frame)
importFrom(DOSE,gseDO)
importFrom(DOSE,simplot)
importFrom(EpiMix,EpiMix_PlotModel)
importFrom(GEOquery,getGEO)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(RColorBrewer,brewer.pal)
importFrom(circlize,circlize)
importFrom(circlize,circos.clear)
importFrom(circlize,circos.initialize)
importFrom(circlize,circos.par)
importFrom(circlize,circos.rect)
importFrom(circlize,circos.text)
importFrom(circlize,circos.trackPlotRegion)
importFrom(circlize,colorRamp2)
importFrom(circlize,get.cell.meta.data)
importFrom(clusterProfiler,bitr)
importFrom(data.table,rbindlist)
importFrom(dplyr,filter)
importFrom(dplyr,pull)
importFrom(fgsea,fgsea)
importFrom(fuzzyjoin,genome_left_join)
importFrom(ggplot2,ggsave)
importFrom(gplots,heatmap.2)
importFrom(gplots,redblue)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,graphics.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,text)
importFrom(grid,gpar)
importFrom(magrittr,"%>%")
importFrom(parmigene,knnmi.cross)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,pmap)
importFrom(qpdf,pdf_combine)
importFrom(randomForest,randomForest)
importFrom(readr,parse_guess)
importFrom(readr,read_csv)
importFrom(rlang,set_names)
importFrom(rtracklayer,import.chain)
importFrom(seqminer,tabix.read.table)
importFrom(stats,fisher.test)
importFrom(stats,p.adjust)
importFrom(stringr,str_c)
importFrom(stringr,str_extract)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_split_i)
importFrom(stringr,str_trim)
importFrom(tibble,as_tibble_col)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,replace_na)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(utils,data)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(withr,with_options)
importMethodsFrom(Biobase,"fvarLabels<-")
importMethodsFrom(Biobase,fvarLabels)

Cancer Systems Biology, Section of Bioinformatics, Department of Health and Technology, Technical University of Denmark, 2800, Lyngby, Copenhagen

Repository associated to the publications:

Interpreting pathways to discover cancer driver genes with Moonlight. Colaprico A, Olsen C, Bailey MH, Odom GJ, Terkelsen T, Silva TC, Olsen AV, Cantini L, Zinovyev A, Barillot E, Noushmehr H, Bertoli G, Castiglioni I, Cava C, Bontempi G, Chen XS, Papaleo E. Nat Commun. 2020 Jan 3;11(1):69. doi: 10.1038/s41467-019-13803-0., PMID: 31900418

A workflow to study mechanistic indicators for driver gene prediction with Moonlight. Mona Nourbakhsh, Astrid Saksager, Nikola Tom, Xi Steven Chen, Antonio Colaprico, Catharina Olsen, Matteo Tiberti, Elena Papaleo*, Briefings in Bioinformatics, 2023, bbad274, https://doi.org/10.1093/bib/bbad274 

contacts for repository: Elena Papaleo, elpap-at-dtu.dk, elenap-at-cancer.dk; Matteo Tiberti: tiberti-at-cancer.dk

## Introduction

This repository contains a new release fo our MoonlightR R package, called
Moonlight2R, which implements the new and improved Moonlight2 workflow. Moonlight2R
contains a number of differences and improvements respect to the original MoonlightR
package. For a full overview of what the package can do, please see and cite when
appropriate:

> Mona Nourbakhsh, Astrid Saksager, Nikola Tom, Xi Steven Chen, Antonio Colaprico,
> Catharina Olsen, Matteo Tiberti, Elena Papaleo*
> A workflow to study mechanistic indicators for driver gene prediction with 
> Moonlight, Briefings in Bioinformatics, 2023, bbad274, https://doi.org/10.1093/bib/bbad274

### Documentation

please see the material included in the repository. The Moonlight2R [vignette
is publicly available on RPubs](https://rpubs.com/mtiberti/moonlight2r-vignette)

### Changes respect to MoonlightR

- The following user-facing functions were added to Moonlight2R:
    - `DMA` and `plotDMA`, for the new driver mutation analysis layer
    - `GLS`, for automated gene literature search for the most interesting
    identified driver genes
    - `plotMoonlight` function to generate a heatmap of Moonlight gene z-scores
    for selected genes
    - new helper functions: `LiftMAF`, `MAFtoCscape`, `PRAtoTibble`, `RunCscape_somatic`,
`confidence`, `plotHeatmap`, and `tabix_func`

- Deprecation and removal of the following functions, which were available in
MoonlightR: `DPA` and `getDataTCGA`. This means that users now needs to supply
their own differential expression analysis results (see vignettes) to run
predictions with Moonlight2R

- The `moonlight` function, which implements the whole Moonlight pipeline in a
single function,  was updated accordingly to the new changes in Moonlight2R

- Example data has been updated which follows changes in the functions.
Specifically, the following data files have been added to Moonlight2R:
`DEG_Mutations_Annotations.rda`, `EncodePromoters.rda`, `LOC_protein.rda`, `LOC_transcription.rda`,
`LOC_translation.rda`, `NCG.rda`, `Oncogenic_mediators_mutation_summary.rda`, `cscape_somatic_output.rda`,
`dataDMA.rda`, `dataGLS.rda`, and `dataMAF.rda`. The following data files were not included in
Moonlight2R: `GDCprojects.rda` and `geneInfo.rda` as these files were connected to the deleted functions.

- the vignette has been updated in light of changes implemented in Moonlight2R

## Installation

### Installation from BioConductor

To install this package from the BioConductor repositories, start R (version "4.3")
and enter:

```R
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("Moonlight2R")
```

### Installation from GitHub
```R
devtools::install_github(repo = "ELELAB/Moonlight2R")
```

#### Installation from GitHub with accompanying vignette

You need the BiocStyle Bioconductor package to install Moonlight2R with the vignette.
This package can be installed like:
```R
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("BiocStyle")
```

To install Moonlight2R with its vignette:
```R
devtools::install_github(repo = "ELELAB/Moonlight2R", build_vignettes = TRUE)
```

To view the vignette:
```R
library(Moonlight2R)
```

```R
vignette( "Moonlight2R", package="Moonlight2R")
```

### Installation of other requirements

To make the best out of Moonlight2R the user should download the [CSCapeSomatic
pre-computed scores](http://cscape-somatic.biocompute.org.uk/#download). More
detailed instructions are available in the [INSTALL file](./INSTALL)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(dpi = 72)
knitr::opts_chunk$set(cache=FALSE)

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("Moonlight2_pipeline_upd.png")

## ----eval = FALSE-------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("Moonlight2R")

## ----eval = FALSE-------------------------------------------------------------
# install.packages("devtools")
# library(devtools)

## ----eval = FALSE-------------------------------------------------------------
# devtools::install_github(repo = "ELELAB/Moonlight2R")

## ----eval = FALSE-------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("BiocStyle")

## ----eval = FALSE-------------------------------------------------------------
# devtools::install_github(repo = "ELELAB/Moonlight2R", build_vignettes = TRUE)

## ----eval = FALSE-------------------------------------------------------------
# vignette( "Moonlight2R", package="Moonlight2R")

## ----eval = TRUE--------------------------------------------------------------
library(Moonlight2R)
library(magrittr)
library(dplyr)

## ----eval = TRUE--------------------------------------------------------------
data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(GEO_TCGAtab)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)
data(dataMethyl)
data(DEG_Methylation_Annotations)
data(Oncogenic_mediators_methylation_summary)
data(MetEvidenceDriver)
data(LUAD_sample_anno)

## ----eval = TRUE, echo = TRUE-------------------------------------------------
knitr::kable(GEO_TCGAtab, digits = 2,
       caption = "Table with GEO data set matched to one
       of the 18 given TCGA cancer types ",
       row.names = TRUE)

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
dataFilt_GEO <- getDataGEO(GEOobject = "GSE20347", platform = "GPL571")

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
dataFilt_GEO <- getDataGEO(TCGAtumor = "ESCA")

## ----eval = TRUE, echo = TRUE, results='hide'---------------------------------
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
dataFEA <- FEA(DiffMatrix = DEGsmatrix, method="ora")

## ----eval = TRUE, echo = TRUE, message=FALSE, results='hide', warning=FALSE----
plotFEA(dataFEA = dataFEA, additionalFilename = "_exampleVignette", height = 10, width = 20)

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("FEAplot.gif")

## ----eval = TRUE--------------------------------------------------------------
data(DEGsmatrix)
data(dataFilt)
dataGRN <- GRN(DEGsmatrix = DEGsmatrix, TFs = sample(rownames(DEGsmatrix), 100), normCounts = dataFilt,
	               nGenesPerm = 5, kNearest = 3, nBoot = 5, DiffGenes = TRUE)

## ----eval = TRUE, echo = TRUE, results='hide'---------------------------------
data(dataGRN)
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)

dataFEA <- FEA(DiffMatrix = DEGsmatrix)

BPselected <- dataFEA$Diseases.or.Functions.Annotation[1:5]
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = BPselected,
               nCores=1)

## ----eval = TRUE--------------------------------------------------------------
data(dataURA)
data(tabGrowBlock)
data(knownDriverGenes)
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis","proliferation of cells"),
               thres.role = 0)

## ----eval = FALSE-------------------------------------------------------------
# data(dataPRA)
# data(dataMAF)
# data(DEGsmatrix)
# data(LOC_transcription)
# data(LOC_translation)
# data(LOC_protein)
# data(NCG)
# data(EncodePromoters)
# dataDMA <- DMA(dataMAF = dataMAF,
#                dataDEGs = DEGsmatrix,
#                dataPRA = dataPRA,
#                results_folder = "DMAresults",
#                coding_file = "css_coding.vcf.gz",
#                noncoding_file = "css_noncoding.vcf.gz")

## ----eval = TRUE--------------------------------------------------------------
data("dataMethyl")
data("dataFilt")
data("dataPRA")
data("DEGsmatrix")
data("LUAD_sample_anno")
data("NCG")
data("EncodePromoters")
data("MetEvidenceDriver")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)

## ----eval = TRUE--------------------------------------------------------------
genes_query <- "BRCA1"
dataGLS <- GLS(genes = genes_query,
	       query_string = "AND cancer AND driver")
head(dataGLS)

## ----eval = TRUE--------------------------------------------------------------

mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')

specific_protein <- loadMAVISp(mavispDB = mavisp_db_location,
                               mode = 'simple',
                               proteins_of_interest = c('RUNX1'))

all_proteins <- loadMAVISp(mavispDB = mavisp_db_location,
                           mode = 'simple')

ensemble <- loadMAVISp(mavispDB = mavisp_db_location,
                       mode = 'ensemble',
                       ensemble = 'cabsflex')

## ----eval = TRUE--------------------------------------------------------------
data(dataPRA)
data(DEGsmatrix)
data(dataTRRUST)
data(dataMAF)
data(dataMAVISp)

TFresults <- TFinfluence(dataTRRUST = dataTRRUST,
            dataMAF = dataMAF,
            dataDEGs = DEGsmatrix,
            dataPRA = dataPRA,
            dataMAVISp = dataMAVISp,
            stabClassMAVISp = 'rasp')

## -----------------------------------------------------------------------------
knitr::kable(LOC_transcription)

## -----------------------------------------------------------------------------
knitr::kable(LOC_translation)

## -----------------------------------------------------------------------------
knitr::kable(LOC_protein)

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
data(knownDriverGenes)
data(dataGRN)
plotNetworkHive(dataGRN, knownDriverGenes, 0.55)

## ----eval = TRUE, warning = FALSE, message = FALSE, include=TRUE--------------
data(dataDMA)
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
plotDMA(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        type = 'complete', additionalFilename = "")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("heatmap_complete.gif")

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
data(dataURA_plot)
plotMoonlight(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        dataURA_plot, gene_type = "drivers", n = 50)

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("moonlight_heatmap.gif")

## ----eval = TRUE, warning = FALSE, message = FALSE, include = TRUE------------
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotGMA(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
        Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
        type = "genelist", genelist = genes,
        additionalFilename = "./GMAresults/")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("heatmap_genelist_met.gif")

## ----eval = TRUE, warning = FALSE, message = FALSE, include = TRUE------------
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
data("dataURA_plot")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotMoonlightMet(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
                 Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
                 dataURA = dataURA_plot,
                 genes = genes,
                 additionalFilename = "./GMAresults/")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("moonlight_heatmap_met.gif")

## ----eval = TRUE, warning = FALSE, message = FALSE, include = TRUE------------
data("EpiMix_Results_Regular")
data("dataMethyl")
data("dataFilt")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

plotMetExp(EpiMix_results = EpiMix_Results_Regular,
           probe_name = "cg03035704",
           dataMET = dataMethyl,
           dataEXP = dataFilt,
           gene_of_interest = "ACVRL1",
           additionalFilename = "./GMAresults/")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("plotMetExp.png")

## ----eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE, results='hide'--------
data(DEGsmatrix)
data(dataFilt)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)

dataFEA <- FEA(DiffMatrix = DEGsmatrix)

dataGRN <- GRN(TFs = sample(rownames(DEGsmatrix), 100),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               nBoot = 5,
	       kNearest = 3)

dataURA <- URA(dataGRN = dataGRN,
              DEGsmatrix = DEGsmatrix,
              BPname = c("apoptosis",
                         "proliferation of cells"))

dataDual <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

oncogenic_mediators <- list("TSG"=names(dataDual$TSG), "OCG"=names(dataDual$OCG))


## ----eval = TRUE,message=FALSE,warning=FALSE, results='hide'------------------
data(dataURA)
plotURA(dataURA = dataURA, additionalFilename = "_exampleVignette")

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("URAplot.gif")

## ----eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE-----------------------
# data(dataFilt)
# data(DEGsmatrix)
# data(dataMAF)
# data(DiseaseList)
# data(EAGenes)
# data(tabGrowBlock)
# data(knownDriverGenes)
# data(LOC_transcription)
# data(LOC_translation)
# data(LOC_protein)
# data(NCG)
# data(EncodePromoters)
# 
# listMoonlight <- moonlight(dataDEGs = DEGsmatrix,
#                            dataFilt = dataFilt,
#                            nTF = 100,
#                            DiffGenes = TRUE,
# 			   nGenesPerm = 5,
#                            nBoot = 5,
#                            BPname = c("apoptosis","proliferation of cells"),
#                            dataMAF = dataMAF,
#                            path_cscape_coding = "css_coding.vcf.gz",
#                            path_cscape_noncoding = "css_noncoding.vcf.gz")
# save(listMoonlight, file = paste0("listMoonlight_ncancer4.Rdata"))
# 
# 

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
data(listMoonlight)
plotCircos(listMoonlight = listMoonlight, additionalFilename = "_ncancer5")

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("circos_ocg_tsg_ncancer5.gif")

## ----eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE-----------------------
# 
# data(DEGsmatrix)
# data(dataFilt)
# data(dataMAF)
# data(DiseaseList)
# data(EAGenes)
# data(tabGrowBlock)
# data(knownDriverGenes)
# data(LOC_transcription)
# data(LOC_translation)
# data(LOC_protein)
# data(NCG)
# data(EncodePromoters)
# 
# # Perform gene regulatory network analysis
# dataGRN <- GRN(TFs = rownames(DEGsmatrix),
# 	       DEGsmatrix = DEGsmatrix,
# 	       DiffGenes = TRUE,
#                normCounts = dataFilt,
#                nGenesPerm = 5,
#                kNearest = 3,
#                nBoot = 5)
# 
# # Perform upstream regulatory analysis
# # As example, we use apoptosis and proliferation of cells as the biological processes
# dataURA <- URA(dataGRN = dataGRN,
#                DEGsmatrix = DEGsmatrix,
#                BPname = c("apoptosis",
#                           "proliferation of cells"),
#                nCores = 1)
# 
# # Perform pattern recognition analysis
# dataPRA <- PRA(dataURA = dataURA,
#                BPname = c("apoptosis",
#                           "proliferation of cells"),
#                thres.role = 0)
# 
# # Perform driver mutation analysis
# dataDMA <- DMA(dataMAF = dataMAF,
#                dataDEGs = DEGsmatrix,
#                dataPRA = dataPRA,
#                results_folder = "DMAresults",
#                coding_file = "css_coding.vcf.gz",
#                noncoding_file = "css_noncoding.vcf.gz")
# 

## ----eval = TRUE--------------------------------------------------------------
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)

# Extract oncogenic mediators that contain at least one driver mutation
# These are the driver genes
knitr::kable(Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)))

# Extract mutation annotations of the predicted driver genes
driver_mut <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver")

# Extract driver genes with a predicted effect on the transcriptional level
transcription_mut <- Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)) %>%
  filter(Transcription_mut_sum > 0)

# Extract mutation annotations of predicted driver genes that have a driver mutation
# in its promoter region with a potential effect on the transcriptional level
promoters <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver",
         Potential_Effect_on_Transcription == 1,
         Annotation == 'Promoter')

## ----eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE,results='hide'---------

data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)
data(dataMethyl)
data(LUAD_sample_anno)
data(MetEvidenceDriver)

# Perform gene regulatory network analysis
dataGRN <- GRN(TFs = rownames(DEGsmatrix),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               kNearest = 3,
               nBoot = 5)

# Perform upstream regulatory analysis
# As example, we use apoptosis and proliferation of cells as the biological processes
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               nCores = 1)

# Perform pattern recognition analysis
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

# Perform gene methylation analysis
dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1,
               roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)


## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' plotMoonlight
#'
#' This function creates a heatmap of Moonlight gene z-scores for selected genes.
#'
#' @param DEG_Mutations_Annotations A tibble, output file from DMA.
#' @param Oncogenic_mediators_mutation_summary A tibble, output file from DMA.
#' @param dataURA Output URA function.
#' @param gene_type A character string either \code{"mediators"} or \code{"drivers"}.
#' \itemize{
#' \item If \code{NULL} defaults to \code{"drivers"}.
#' \item \code{"mediators"} will show the oncogenic mediators with the highst number of mutations regardless of driver/passenger classification.
#' \item \code{"drivers"} will show the driver genes with the highest number of driver mutations.
#' }
#' @param n Numeric. The top number of genes to be plotted. If \code{NULL} defaults to 50.
#' @param genelist A vector of strings containing Hugo Symbols of genes.
#' Overwrites \code{gene_type} argument.
#' @param BPlist A vector of strings. Selection of the biological processes to visualise.
#'  If left empty defaults to every BP provided in the URA file.
#' @param additionalFilename A character string. Adds prefix or filepath to the filename of the pdf.
#'
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @import ComplexHeatmap
#' @import tidyHeatmap
#' @importFrom circlize colorRamp2
#' @importFrom tidyr drop_na pivot_longer replace_na
#' @importFrom tibble rownames_to_column
#'
#' @return No return value. Moonlight scores are plotted for selected genes.
#' @export
#'
#' @examples
#' data(DEG_Mutations_Annotations)
#' data(Oncogenic_mediators_mutation_summary)
#' data(dataURA_plot)
#' plotMoonlight(DEG_Mutations_Annotations,
#'               Oncogenic_mediators_mutation_summary,
#'               dataURA_plot, genelist = c("AFAP1L1", "ABCG2"),
#'               additionalFilename = "myplot_")
plotMoonlight <- function(DEG_Mutations_Annotations,
                          Oncogenic_mediators_mutation_summary,
                          dataURA,
                          gene_type = "drivers",
                          n = 50,
                          genelist = c(),
                          BPlist = c(),
                          additionalFilename = "") {

  # Check user input

  if (all(c("Hugo_Symbol", "Moonlight_gene_z_score", "logFC") %in% colnames(DEG_Mutations_Annotations)) == FALSE) {
    stop("DEG_Mutations_Annotations must contain Hugo_Symbol,
Moonlight_gene_z_score, and logFC as column names")
  }

  if (c("Moonlight_Oncogenic_Mediator") %in% colnames(Oncogenic_mediators_mutation_summary) == FALSE) {
    stop("Oncogenic_mediators_mutation_summary must contain Moonlight_Oncogenic_Mediator
as column name")
  }

  if (is(dim(dataURA), "NULL")) {
    stop("The URA data must be non-empty with genes in rows and BPs in columns")
  }

  if (!is(gene_type, "NULL") && (gene_type %in% c("mediators", "drivers")) == FALSE) {
    stop("Gene type must either be NULL, mediators or drivers")
  }

  if (!is(genelist, "NULL") & !is(genelist, "character")) {
    stop("Genelist must be either NULL or a character vector containing gene names")
  }

  if (!is(BPlist, "NULL") & !is(BPlist, "character")) {
    stop("BPlist must be either NULL or a character vector containing BP names")
  }

  if (!is(additionalFilename, "character")) {
    stop("additionalFilename must be a character vector adding a prefix or
filepath to the filename of the pdf")
  }

  # The differentially expressed genes that are annotated as TSG/OCG
  DEGs <- DEG_Mutations_Annotations %>%
    select(Hugo_Symbol, Moonlight_gene_z_score, logFC) %>%
    unique() %>%
    drop_na(Moonlight_gene_z_score)

  # Restructure URA to tibble
  ura <- as_tibble(dataURA, rownames = NA) %>%
    rownames_to_column(var = "Genes")

  ura_wrangled <- ura %>%
    pivot_longer(cols = !c("Genes"),
                 names_to = "Biological_Process",
                 values_to = "Moonlight_score") %>%
    right_join(Oncogenic_mediators_mutation_summary,
               by = c("Genes" = "Hugo_Symbol")) %>%
    right_join(DEGs, by = c("Genes" = "Hugo_Symbol")) %>%
    replace_na(list(CScape_Driver = 0,
                    CScape_Passenger = 0,
                    CScape_Unclassified = 0))

  # Biological processes
  n_BP <- ura_wrangled %>%
    group_by(Biological_Process) %>%
    summarise() %>%
    count() %>%
    pull()

  if (length(BPlist) > 0) {
    ura_wrangled <- ura_wrangled %>%
      filter(Biological_Process %in% BPlist)
    n_BP <- length(BPlist)
  }

  n <- n * n_BP

  # Type of plot:
  if (length(genelist) > 0) {
    ura_wrangled <- ura_wrangled %>%
      filter(Genes %in% genelist)
  } else if (gene_type == "mediators") {
    ura_wrangled <- ura_wrangled %>%
      slice_max(Total_Mutations, n = n, with_ties = FALSE)
  } else {
    ura_wrangled <- ura_wrangled %>%
      slice_max(CScape_Driver, n = n, with_ties = FALSE)
  }

  n_types <- ura_wrangled %>%
    group_by(Moonlight_Oncogenic_Mediator) %>%
    summarise() %>%
    count() %>%
    pull()

  if (n_types <= 1) {
    stop("Must contain at least one OCG and one TSG. Increase n or add gene to genelist")
  }

  # Variable for color scaling in legend
  max_driver <- ura_wrangled %>%
    arrange(desc(CScape_Driver)) %>%
    select(CScape_Driver) %>%
    head(1) %>%
    pull

  # Plot Heatmap
  bp_heatmap <- heatmap(ura_wrangled,
                        .row = Biological_Process,
                        .column = Genes,
                        .value = Moonlight_score,
                        scale = "none",
                        clustering_distance_columns = "euclidean",
                        clustering_method_columns = "complete",
                        cluster_rows = FALSE) %>%
    annotation_tile(Moonlight_Oncogenic_Mediator,
             palette = c("goldenrod2", "dodgerblue3")) %>%
    annotation_tile(logFC, palette = c("chartreuse4", "firebrick3")) %>%
    annotation_tile(CScape_Driver, palette = colorRamp2(c(0, max_driver), c("white", "dodgerblue3"))) %>%
    annotation_bar(Total_Mutations)

  save_pdf(bp_heatmap, height = 15, width = 35, units = "cm",
           filename = paste(additionalFilename, "moonlight_heatmap.pdf", sep = ""))
}

utils::globalVariables(c("Hugo_Symbol", "Moonlight_gene_z_score", "logFC",
                         "Biological_Process", "Genes", "Total_Mutations",
                         "CScape_Driver", "Moonlight_Oncogenic_Mediator",
                         "head", "Moonlight_score"))

#' plotMoonlightMet
#' This function visualizes the effect of genes on biological processes
#' and total number of hypo/hyper/dual methylated CpG sites in genes.
#' @param DEG_Methylation_Annotations A tibble which is outputted
#' from the GMA function.
#' @param Oncogenic_mediators_methylation_summary A tibble which
#' is outputted from the GMA function.
#' @param dataURA Output of the URA function: a table containing
#' the effect of oncogenic mediators on biological processes. This
#' effect is quantified through Moonlight Gene Z-scores.
#' @param genes A character string containing the genes to be
#' visualized.
#' @param additionalFilename A character string that can be used
#' to add a prefix or filepath to the filename of the pdf
#' visualizing the heatmap. Default is an empty string.
#' @import dplyr
#' @import tibble
#' @import tidyr
#' @import ComplexHeatmap
#' @import tidyHeatmap
#' @importFrom magrittr "%>%"
#' @return No value is returned. Visualization in form of a heatmap
#' is saved.
#' @export
#' @examples
#' data("DEG_Methylation_Annotations")
#' data("Oncogenic_mediators_methylation_summary")
#' data("dataURA_plot")
#' genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
#' plotMoonlightMet(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
#'                  Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
#'                  dataURA = dataURA_plot,
#'                  genes = genes,
#'                  additionalFilename = "./GMAresults/")
plotMoonlightMet <- function(DEG_Methylation_Annotations,
                             Oncogenic_mediators_methylation_summary,
                             dataURA,
                             genes,
                             additionalFilename = "") {

  # Get logFC of DEGs
  DEGs_logFC <- DEG_Methylation_Annotations %>%
    dplyr::select(Hugo_Symbol,
                  logFC) %>%
    distinct()

  # Convert URA data to long format needed for visualization
  ura <- as_tibble(dataURA,
                   rownames = NA) %>%
    rownames_to_column(var = "Genes") %>%
    pivot_longer(cols = !c("Genes"),
                 names_to = "Biological_process",
                 values_to = "Moonlight_gene_z_score")

  # Count total number of hypo/hyper/dual methylated CpG sites
  # for each oncogenic mediator
  combined_data <- Oncogenic_mediators_methylation_summary %>%
    dplyr::select(-c(contains("NCG"),
                     No_methyl_num)) %>%
    mutate(Total_methyl_num = rowSums(dplyr::select(.,
                                                    contains("methyl_num")),
                                      na.rm = TRUE)) %>%
    #join with ura data
    left_join(x = .,
              y = ura,
              by = c("Hugo_Symbol" = "Genes")) %>%
    #join with DEA data
    left_join(x = .,
              y = DEGs_logFC,
              by = "Hugo_Symbol") %>%
    #filter data to contain only genes supplied in input
    filter(Hugo_Symbol %in% genes)

  # Get number of Moonlight_Oncogenic_Mediator groups in subsetted data
  driver_groups <- combined_data$Moonlight_Oncogenic_Mediator %>%
    unique %>%
    length

  met_heatmap <- combined_data

  # If data contains both oncogenic mediator gene types
  if (driver_groups == 2) {
    met_heatmap <- met_heatmap %>%
      group_by(Moonlight_Oncogenic_Mediator)
  }

  # Create heatmap of genes
  met_heatmap <- met_heatmap %>%
    heatmap(.data = .,
            .row = Biological_process,
            .column = Hugo_Symbol,
            .value = Moonlight_gene_z_score,
            scale = "none",
            clustering_distance_columns = "euclidean",
            clustering_method_columns = "complete",
            cluster_rows = FALSE) %>%
    annotation_tile(logFC,
             palette = c("chartreuse4",  "firebrick3")) %>%
    annotation_bar(Total_methyl_num)

  # If data contains both oncogenic mediator gene types
  # add gene type annotation to heatmap
  if (driver_groups == 2) {
    met_heatmap <- met_heatmap %>%
      annotation_tile(Moonlight_Oncogenic_Mediator,
               palette = c("goldenrod2", "dodgerblue3"))
  }

  # Save heatmap
  save_pdf(met_heatmap,
           height = 15,
           width = 35,
           units = "cm",
           filename = paste(additionalFilename,
                            "moonlight_heatmap_met.pdf",
                            sep = ""))
}

utils::globalVariables(c("No_methyl_num", "Biological_process", "Total_methyl_num"))

#' @title URA Upstream Regulator Analysis
#' @description
#' This function carries out the upstream regulator analysis
#' @param dataGRN output GNR function
#' @param DEGsmatrix output DPA function
#' @param BPname biological processes
#' @param nCores number of cores to use
#' @importFrom stats fisher.test
#' @import parallel
#' @import doParallel
#' @import foreach
#' @export
#' @return an adjacent matrix
#' @examples
#' data(DEGsmatrix)
#' dataDEGs <- DEGsmatrix
#' data(dataGRN)
#' data(DiseaseList)
#' data(EAGenes)
#' dataURA <- URA(dataGRN = dataGRN,
#' DEGsmatrix = dataDEGs,
#' BPname = c("apoptosis",
#' "proliferation of cells"))
URA <- function(dataGRN,
                DEGsmatrix,
                BPname,
                nCores = 1) {

  # List of variable names
  variables_to_check <- c("DiseaseList", "EAGenes")

  # Check and load variables if they do not exist
  for (variable_name in variables_to_check) {
    if (! variable_name %in% names(.GlobalEnv)) {
      data(list=c(variable_name))
    }
  }

  DiseaseList <- get("DiseaseList")

  # Check user input
  if (.row_names_info(DEGsmatrix) < 0) {
    stop("Row names were generated automatically. The input DEG table needs to
have the gene names as rownames. Double check that genes are rownames.")
  }

  if (!is(BPname, "NULL") && all(BPname %in% names(DiseaseList)) == FALSE) {
    stop("BPname should be NULL or a character vector containing one or more
BP(s) among possible BPs stored in the DiseaseList object.")
  }

  if (nCores > 1) {
    cl <- parallel::makeCluster(nCores)
    doParallel::registerDoParallel(cl)
  }

  if (is(BPname, "NULL")) {
    BPname <- names(DiseaseList)
  }

  tRlist <- rownames(dataGRN$miTFGenes)
  pb <- txtProgressBar(min = 0, max = length(tRlist), style = 3)
  j <- NULL

  TableDiseases <- foreach(j = seq.int(tRlist), .combine = "rbind", .packages="foreach") %dopar% {

    currentTF <- as.character(tRlist[j])
    currentTF_regulon <- names(which(dataGRN$miTFGenes[currentTF, ] > as.numeric(dataGRN$maxmi[currentTF])))
    currentTF_regulon <- as.matrix(currentTF_regulon)
    DEGsregulon <- intersect(rownames(DEGsmatrix), currentTF_regulon)

    if (length(DEGsregulon) > 2) {
      tabFEA <- FEA(BPname = BPname, DEGsmatrix = DEGsmatrix[DEGsregulon, ])
      return(tabFEA$Moonlight.Z.score)
    } else {
      return(rep(0, length(BPname)))
    }
  }

  dimnames(TableDiseases) <- list(tRlist, BPname)

  if (nCores > 1 ) {
    doParallel::stopImplicitCluster()
  }

  close(pb)

  return(TableDiseases)

}

# Test URA function

# Run example of URA
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
dataDEGs <- DEGsmatrix
data(dataGRN)

# Load example data of URA serving as reference point
data(dataURA)

dataURA_test <- URA(dataGRN = dataGRN,
                    DEGsmatrix = dataDEGs,
                    BPname = c("apoptosis",
                               "proliferation of cells"))

# Test that output of URA is as expected
test_that("output of URA is a double with 2 columns", {
  expect_type(dataURA_test, "double")
  expect_equal(ncol(dataURA_test), 2)
  expect_equal(colnames(dataURA_test), c("apoptosis",
                                    "proliferation of cells"))
  expect_true(all(sapply(dataURA_test, is.numeric)))
})

# Test that output of URA is as expected compared to reference / example data
test_that("URA output is identical to reference point", {
  expect_equal(dataURA_test, dataURA)
})

dataURA_test <- URA(dataGRN = dataGRN,
                    DEGsmatrix = dataDEGs,
                    BPname = c("apoptosis",
                               "proliferation of cells"),
                    nCores = 2)

# Test that output of URA is as expected
test_that("output of URA is a double with 2 columns", {
  expect_type(dataURA_test, "double")
  expect_equal(ncol(dataURA_test), 2)
  expect_equal(colnames(dataURA_test), c("apoptosis",
                                    "proliferation of cells"))
  expect_true(all(sapply(dataURA_test, is.numeric)))
})

# Test that output of URA is as expected compared to reference / example data
test_that("URA output is identical to reference point", {
  expect_equal(dataURA_test, dataURA)
})

#' Differentially expressed genes
#'
#'@description  A matrix containing differentially expressed genes between lung cancer and normal
#' samples found using TCGA-LUAD data and TCGAbiolinks. 
#'@details The matrix contains the differentially expressed genes in rows and log2 fold change
#' and FDR values in columns.
#'
#'@docType data
#'@usage data(DEGsmatrix)
#'@name DEGsmatrix
#'@aliases DEGsmatrix
#'@return A 3390x5 matrix
#'
#'@format A 3390x5 matrix
#'
"DEGsmatrix"


#' Cancer-related biological processes
#'
#'@description  A dataset containing information about 101 cancer-related biological processes. 
#'@details The dataset contains a list of the 101 biological processes which includes genes
#' playing a role in each biological processes including literature findings of the genes' function
#' in the biological processes.
#'
#'@docType data
#'@usage data(DiseaseList)
#'@name DiseaseList
#'@aliases DiseaseList
#'@return A list of 101 elements
#'
#'@format A list of 101 elements
#'
"DiseaseList"


#' Information about genes
#'
#'@description  A matrix containing information about 20038 genes 
#' including their gene description, location and family
#'@details The matrix contains the genes in rows and description, 
#' location and family in columns.
#'
#'@docType data
#'@usage data(EAGenes)
#'@name EAGenes
#'@aliases EAGenes
#'@return A 20038x5 matrix
#'
#'@format A 20038x5 matrix
#'
"EAGenes"

#' Information on GEO and TCGA data
#'
#'@description  A matrix that provides the GEO dataset
#'   matched to one of 18 TCGA cancer types
#'@details The matrix contains the cancer types in rows and 
#' information about sample type from both TCGA and GEO in columns.
#'
#'@docType data
#'@usage data(GEO_TCGAtab)
#'@name GEO_TCGAtab
#'@aliases GEO_TCGAtab
#'@return A 18x12 matrix
#'
#'@format A 18x12 matrix
#'
"GEO_TCGAtab"

#' Gene expression data from TCGA-LUAD
#'
#'@description  A matrix that provides processed gene expression 
#' data (obtained from RNA seq) from the TCGA-LUAD project
#'@details The matrix contains the genes in rows and samples in 
#' columns. The data has been downloaded and processed using 
#' TCGAbiolinks.
#'
#'@docType data
#'@usage data(dataFilt)
#'@name dataFilt
#'@aliases dataFilt
#'@return A 3000x20 matrix
#'
#'@format A 3000x20 matrix
#'
"dataFilt"

#' Functional enrichment analysis
#'
#'@description  The output of the FEA function which does enrichment analysis
#'@details The input to the FEA is the differentially expressed genes.
#'
#'@docType data
#'@usage data(dataFEA)
#'@name dataFEA
#'@aliases dataFEA
#'@return A dataframe of dimension 101x7
#'
#'@format A dataframe of dimension 101x7
#'
"dataFEA"

#' Gene regulatory network
#'
#'@description  The output of the GRN function which finds connections 
#' between genes.
#'@details The input to the GRN is the differentially expressed genes and the gene expression data.
#'
#'@docType data
#'@usage data(dataGRN)
#'@name dataGRN
#'@aliases dataGRN
#'@return A list of 2 elements where the first element is a 23x613 matrix and the second element is a vector of length 23
#'
#'@format A list of 2 elements where the first element is a 23x613 matrix 
#' and the second element is a vector of length 23
#'
"dataGRN"


#' Gene regulatory network
#'
#'@description  The output of the GRN function which finds connections 
#' between genes where the noise is set to 0 for testing reproducibility purposes.
#'@details The input to the GRN is the differentially expressed genes and the gene expression data.
#'
#'@docType data
#'@usage data(dataGRN_no_noise)
#'@name dataGRN_no_noise
#'@aliases dataGRN_no_noise
#'@return A list of 2 elements where the first element is a 23x613 matrix and the second element is a vector of length 23
#'
#'@format A list of 2 elements where the first element is a 23x613 matrix 
#' and the second element is a vector of length 23
#'
"dataGRN_no_noise"

#' Upstream regulator analysis
#'
#'@description  The output of the URA function which carries out the 
#' upstream regulator analysis
#'@details The input to URA is the output of GRN and a list of biological 
#' processes and the differentially expressed genes
#'
#'@docType data
#'@usage data(dataURA)
#'@name dataURA
#'@aliases dataURA
#'@return A 23x2 matrix
#'
#'@format A 23x2 matrix
#'
"dataURA"

#' Upstream regulator analysis
#'
#'@description  The output of the URA function which carries out the 
#' upstream regulator analysis
#'@details This URA data is used to showcase some of the visualization
#' functions
#'
#'@docType data
#'@usage data(dataURA_plot)
#'@name dataURA_plot
#'@aliases dataURA_plot
#'@return A 12x2 matrix
#'
#'@format A 12x2 matrix
#'
"dataURA_plot"

#' Information of known cancer driver genes from COSMIC
#'
#'@description  A list of known cancer driver genes from COSMIC
#'@details The list contains two elements: a vector of known tumor #' suppressors and a vector of known oncogenes 
#'
#'@docType data
#'@usage data(knownDriverGenes)
#'@name knownDriverGenes
#'@aliases knownDriverGenes
#'@return A list containing two elements where the first element is a 
#' character vector of 55 and the second element is a character vector of #' 84
#'
#'@format A list containing two elements where the first element is a 
#' character vector of 55 and the second element is a character vector of #' 84
#'
"knownDriverGenes"

#' List of oncogenic mediators of 5 TCGA cancer types
#'
#'@description  A list of oncogenic mediators of 5 TCGA cancer types: 
#' BLCA, BRCA, LUAD, READ and STAD
#'@details Each element in the list contains differentially expressed 
#' genes and output from the URA and PRA functions
#'
#'@docType data
#'@usage data(listMoonlight)
#'@name listMoonlight
#'@aliases listMoonlight
#'@return A list containing 5 elements where each element contains 
#' differentially expressed genes and output from the URA and PRA 
#' functions of 5 TCGA cancer types
#'
#'@format A list containing 5 elements where each element contains 
#' differentially expressed genes and output from the URA and PRA 
#' functions of 5 TCGA cancer types
#'
"listMoonlight"

#' Information of growing/blocking characteristics of 101 biological
#' processes
#'
#'@description  A matrix with biological processes in rows and the cancer #' growing or blocking effect of the process in columns
#'@details For each biological processes the cancer growing/blocking 
#' effect is indicated 
#'
#'@docType data
#'@usage data(tabGrowBlock)
#'@name tabGrowBlock
#'@aliases tabGrowBlock
#'@return A 101x3 matrix
#'
#'@format A 101x3 matrix
#'
"tabGrowBlock"


#' Promoters
#'
#'@description Experimentially verified promoter sites by J. Michael Cherry, Stanford.
#' Downloaded from the ENCODE identifier ENCSR294YNI.
#' It contains chromosome, start and end sites of promoters.
#'
#'@docType data
#'@usage data(EncodePromoters)
#'@name EncodePromoters
#'@aliases EncodePromoters
#'@return A 84738x6 table
#'@format A tibble with no columnnames or rownames.
#'\enumerate{
#'\item The first column is chromosome eg. chr1
#'\item The second column is start position eg. 10451
#'\item The third column is end position eg. 10563
#'}
#'
#'@source \url{https://www.encodeproject.org/}
#'@references  ENCODE identifier: ENCSR294YNI
#'
#'Luo Y, Hitz BC, Gabdank I, Hilton JA, Kagda MS, Lam B, Myers Z, Sud P, Jou J,
#'Lin K, Baymuradov UK, Graham K, Litton C, Miyasato SR, Strattan JS, Jolanki O,
#'Lee JW, Tanaka FY, Adenekan P, O'Neill E, Cherry JM.
#'New developments on the Encyclopedia of DNA Elements (ENCODE) data portal.
#'Nucleic Acids Res. 2020 Jan 8;48(D1):D882-D889. doi: 10.1093/nar/gkz1062.
#'PMID: 31713622; PMCID: PMC7061942.
#'
"EncodePromoters"

#' Level of Consequence: Protein
#'
#'@description  A dataset binary dataset describing if a mutation of a certain class and
#' type possibly have an effect on protein structure or function.
#'@details The values are binary: 0 no effect is possible, 1 an effect is possible.
#'
#'See supplementary material for details.
#'
#'@docType data
#'@usage data(LOC_protein)
#'@name LOC_protein
#'@aliases LOC_protein
#'@return A 18x7 table
#'
#'@format A 18x7 table
#'@references  paper
#'
"LOC_protein"

#' Level of Consequence: Translation
#'
#'@description  A dataset describing if a mutation of a certain class and
#' type possibly have an effect on peptide level.
#'@details The values are binary: 0 no effect is possible, 1 an effect is possible.
#'
#'See supplementary material for details.
#'
#'@docType data
#'@usage data(LOC_translation)
#'@name LOC_translation
#'@aliases LOC_translation
#'@return A 18x7 table
#'
#'@format A 18x7 table
#'@references  paper
#'
"LOC_translation"


#' Level of Consequence: Transcription
#'
#'@description  A dataset describing if a mutation of a certain class and
#' type possibly have an effect on transcript level.
#'@details The values are binary: 0 no effect is possible, 1 an effect is possible.
#'
#'See supplementary material for details.
#'
#'@docType data
#'@usage data(LOC_transcription)
#'@name LOC_transcription
#'@aliases LOC_transcription
#'@return A 18x7 table
#'
#'@format A 18x7 table
#'@references paper
#'
"LOC_transcription"

#' Network of Cancer Genes 7.0
#'
#'@description  A dataset retrived from Network of Cancer Genes 7.0
#'@details The NCG_driver is reported as a OCG or TSG when at least one of three
#' three databases have documented it. These are cosmic gene census (cgc),
#' vogelstein et al. 2013 or saito et al. 2020. The NCG_driver is reported as a
#' candidate, when literature support the gene as a cancer driver.
#'@docType data
#'@usage data(NCG)
#'@name NCG
#'@aliases NCG
#'@return A 3347x7 table
#'
#'@format The format have been rearranged from the original.
#' <symbold>|<NCG_driver>|<NCG_cgc_annotation>|<NCG_vogelstein_annotation>|
#' <NCG_saito_annotation>|<NCG_pubmed_id>
#'@source \url{http://ncg.kcl.ac.uk/}
#'@references  Comparative assessment of genes driving cancer and somatic
#'evolution in non-cancer tissues: an update of the Network of Cancer Genes (NCG)
#'resource.
#'Dressler L., Bortolomeazzi M., Keddar M.R., Misetic H., Sartini G.,
#'Acha-Sagredo A., Montorsi L., Wijewardhane N., Repana D., Nulsen J.,
#'Goldman J., Pollit M., Davis P., Strange A., Ambrose K. and Ciccarelli F.D.
#'
"NCG"

#' Output example from function Pattern Recognition Analysis
#'
#'@description The predicted TSGs and OCGs and their moonlight gene z-score based on the small sample TCGA-LUAD data.
#'The PRA() were run with expert-based approach with apoptosis and proliferation of cells. 
#'
#'@docType data
#'@usage data(dataPRA)
#'@name dataPRA
#'@aliases dataPRA
#'@return A list of two.
#'@format A list of two. 
#'

#'
"dataPRA"

#' Mutation data from TCGA LUAD
#'
#'@description An examplary MAF file from TCGA on lung cancer LUAD. It contains 500 randomly selected mutations. 
#'
#'@docType data
#'@usage data(dataMAF)
#'@name dataMAF
#'@aliases dataMAF
#'@return A 500x141 matrix.
#'@format A 500x141 matrix. 
#'
#'
"dataMAF"

#' Output example from the function Driver Mutation Analysis
#'
#'@description The predicted driver genes, which have at least one driver mutation. 
#'
#'@docType data
#'@usage data(dataDMA)
#'@name dataDMA
#'@aliases dataDMA
#'@return A list of two, containing 0 tumor-suppressor and 1 oncogene.
#'@format A list of two. 
#'
#'
"dataDMA"

#' Cscape-somatic annotations of TCGA-LUAD 
#'@description Output from DMA. 
#'This contains the cscape-somatic annotations for all differentially expressed genes 
#'
#'@docType data
#'@usage data(cscape_somatic_output)
#'@name cscape_somatic_output
#'@aliases cscape_somatic_output
#'@return A 645x7 matrix.
#'@format A 645x7 matrix. 
#'
#'
"cscape_somatic_output"

#' Differentially expressed genes's Mutations 
#'@description Output from DMA. 
#'This contains the differentially expressed genes's mutations and all annotations
#'generated in DMA() on the TCGA-LUAD project. 
#'
#'@docType data
#'@usage data(DEG_Mutations_Annotations)
#'@name DEG_Mutations_Annotations
#'@aliases DEG_Mutations_Annotations
#'@return A 3561x173 matrix.
#'@format A 3561x173 matrix. 
#'
#'
"DEG_Mutations_Annotations"

#' Oncogenic Mediators Mutation Summary  
#'@description Output from DMA. 
#'This contains the oncogenic mediator from the TCGA-LUAD project, and their mutation
#'assesments summarised based on CSCape-somatic and Level of Consequence. 
#'
#'@docType data
#'@usage data(Oncogenic_mediators_mutation_summary)
#'@name Oncogenic_mediators_mutation_summary
#'@aliases Oncogenic_mediators_mutation_summary
#'@return A 12x15 matrix.
#'@format A 12x15 matrix. 
#'
#'
"Oncogenic_mediators_mutation_summary"

#' Literature search of driver genes
#'
#'@description  A tibble containing results of literature search where predicted
#' driver genes stored in dataDMA were queried for their role as drivers in
#' PubMed 
#'@details The tibble contains PubMed IDs, doi, title, abstract, year of publication,
#' keywords, and total number of publications for the genes.
#'
#'@docType data
#'@usage data(dataGLS)
#'@name dataGLS
#'@aliases dataGLS
#'@return A 13x8 tibble.
#'
#'@format A 13x8 tibble.
#'
"dataGLS"


#' Methylation evidence table to define driver genes
#'
#'@description  A tibble containing combinations of methylation states
#' of probes used to define driver genes 
#'@details The tibble contains a value of 1 if a probe is found that
#' is either hypo-, hyper-, dualmethylated or not methylated. This is
#' compared with Moonlight's predictions of role of oncogenic mediators
#' to define driver genes based on methylation evidence. 
#'
#'@docType data
#'@usage data(MetEvidenceDriver)
#'@name MetEvidenceDriver
#'@aliases MetEvidenceDriver
#'@return A 30x6 tibble.
#'
#'@format A 30x6 tibble.
#'
"MetEvidenceDriver"


#' Methylation data matrix from TCGA-LUAD project
#'
#'@description  A data matrix containing methylation data from TCGA-LUAD
#' where CpG probes are in rows and samples are in columns. 
#'@details The CpG probes are in rows and samples are in columns.
#'
#'@docType data
#'@usage data(dataMethyl)
#'@name dataMethyl
#'@aliases dataMethyl
#'@return A 73x27 matrix.
#'
#'@format A 73x27 matrix.
#'
"dataMethyl"


#' Sample annotations of TCGA-LUAD project 
#'
#'@description  A matrix that annotates LUAD samples as either
#' cancer or normal 
#'@details The matrix contains two columns: "primary" which
#' contains patient barcodes of TCGA-LUAD and "sample.type"
#' which denotes if the sample is either a "Cancer" or 
#' "Normal" sample
#'
#'@docType data
#'@usage data(LUAD_sample_anno)
#'@name LUAD_sample_anno
#'@aliases LUAD_sample_anno
#'@return A 23x2 matrix
#'
#'@format A 23x2 matrix
#'
"LUAD_sample_anno"


#' Output example from GMA function 
#'
#'@description  The predicted driver genes based on methylation
#' evidence 
#'@details The data is a list of two elements where each 
#' element represents predicted oncogenes and tumor suppressors
#'
#'@docType data
#'@usage data(dataGMA)
#'@name dataGMA
#'@aliases dataGMA
#'@return A list of length two
#'
#'@format A list of length two
#'
"dataGMA"


#' Output example from GMA function 
#'
#'@description  The object, a list, that was returned from
#' running the EpiMix function and is one of the outputs from the 
#' GMA function. 
#'@details The data is a list of nine elements which is outputted
#' from the EpiMix function  
#'
#'@docType data
#'@usage data(EpiMix_Results_Regular)
#'@name EpiMix_Results_Regular
#'@aliases EpiMix_Results_Regular
#'@return A list of length nine
#'
#'@format A list of length nine
#'
"EpiMix_Results_Regular"


#' Output example from GMA function 
#'
#'@description  Output file from running the GMA function which
#' is a summary of the oncogenic mediators and their sum of
#' methylated CpG probes together with the evidence level of
#' their role as driver gene. 
#'@details The data is a table where each row is an oncogenic
#' mediator and the columns represent the predicted driver role
#' and the sum of hypo-, hyper-, and dualmethylated CpG sites. 
#'
#'@docType data
#'@usage data(Oncogenic_mediators_methylation_summary)
#'@name Oncogenic_mediators_methylation_summary
#'@aliases Oncogenic_mediators_methylation_summary
#'@return A 25x7 tibble
#'
#'@format A 25x7 tibble
#'
"Oncogenic_mediators_methylation_summary"


#' Output example from GMA function 
#'
#'@description  Output file from running GMA function which
#' is a summary of DEGs and associated CpG probes 
#'@details The data is a table where each row is a CpG probe
#' in a DEG. Various annotations such as start/end site of
#' CpG probe, promoter/enhancer annotations, NCG annotations
#' are included in the table.
#'
#'@docType data
#'@usage data(DEG_Methylation_Annotations)
#'@name DEG_Methylation_Annotations
#'@aliases DEG_Methylation_Annotations
#'@return A 3435x35 tibble
#'
#'@format A 3435x35 tibble
#'
"DEG_Methylation_Annotations"


#' MAVISp dataset example
#'
#'@description MAVISp results for proteins TP53 and RUNX1, simple mode. They are used
#' for testing and vignettes. They are stored as a list of tibbles.
#'@details The dataframes contain the full available MAVISp data for simple mode at
#' present time.
#'
#'@docType data
#'@usage data(dataMAVISp)
#'@name dataMAVISp
#'@aliases dataMAVISp
#'@return A list of tibbles (183 x 43 and 1180 x 46).
#'
#'@format A list of data frames (183 x 43 and 1180 x 46).
#'
"dataMAVISp"

#' GSEA
#'
#' This function carries out the GSEA enrichment analysis.
#' @param DEGsmatrix DEGsmatrix output from DEA such as dataDEGs
#' @param top is the number of top BP to plot
#' @param plot if TRUE return a GSEA's plot
#' @import org.Hs.eg.db
#' @importFrom grDevices dev.list
#' @importFrom grDevices graphics.off
#' @importFrom clusterProfiler bitr
#' @importFrom DOSE gseDO
#' @importFrom DOSE simplot
#' @return return GSEA result
#' @export
#' @examples
#' data("DEGsmatrix")
#' DEGsmatrix_example <- DEGsmatrix[1:2,]
#' dataFEA <- GSEA(DEGsmatrix = DEGsmatrix_example)
GSEA <- function(DEGsmatrix,
                 top,
                 plot = FALSE) {

  # Check user input

  if (.row_names_info(DEGsmatrix) < 0) {
    stop("Row names were generated automatically. The input DEG table needs to
         have the gene names as rownames. Double check that genes are rownames.")
  }

  dataDEGsnew <- cbind(mRNA = rownames(DEGsmatrix), DEGsmatrix)

  eg <- as.data.frame(bitr(dataDEGsnew$mRNA,
                           fromType = "SYMBOL",
                           toType = "ENTREZID",
                           OrgDb = "org.Hs.eg.db"))
  eg <- eg[!duplicated(eg$SYMBOL), ]

  dataDEGsFiltLevel <- dataDEGsnew
  dataDEGsFiltLevel <- dataDEGsFiltLevel[dataDEGsFiltLevel$mRNA %in% eg$SYMBOL, ]
  dataDEGsFiltLevel <- dataDEGsFiltLevel[order(dataDEGsFiltLevel$mRNA, decreasing = FALSE), ]
  eg <- eg[order(eg$SYMBOL, decreasing = FALSE), ]

  dataDEGsFiltLevel$GeneID <- eg$ENTREZID

  dataDEGsFiltLevel_sub <- subset(dataDEGsFiltLevel, select = c("GeneID", "logFC"))

  genelistDEGs <- as.numeric(dataDEGsFiltLevel_sub$logFC)
  names(genelistDEGs) <- dataDEGsFiltLevel_sub$GeneID

  genelistDEGs_sort <- sort(genelistDEGs, decreasing = TRUE)

  # temporary workaround for issue 144 - will change once the problem is fixed upstream
  # gseDO will crash without it
  # see https://github.com/YuLab-SMU/DOSE/issues/86#issuecomment-3361787836
  base_url <- 'https://yulab-smu.top/DOSE'

  y <- gseDO(genelistDEGs_sort,
             nPerm = 100,
             minGSSize = 120,
             pvalueCutoff = 0.2,
             pAdjustMethod = "BH",
             verbose = FALSE)

  res <- as.matrix(summary(y))

  if (plot == TRUE) {
    topID <- res[1, 1]
    pdf("GSEAplot.pdf")
    DOSE::simplot(y, geneSetID = topID)

    if (!(is.null(dev.list()["RStudioGD"]))) {
      graphics.off()
    }
  }

  return(res)

}

#' EpiMix_getInfiniumAnnotation
#' This function gets Infinium probe annotation from the
#' sesameData library. This function is from the EpiMix package
#' https://bioconductor.org/packages/release/bioc/html/EpiMix.html.
#' Zheng Y, Jun J, Gevaert O (2023). EpiMix: 
#' EpiMix: an integrative tool for the population-level analysis of 
#' DNA methylation. R package version 1.1.2.
#' @param plat A character string representing the methylation platform 
#' which can either be HM27, HM450 or EPIC
#' @param genome A character string representing the genome build version
#' which can either be hg19 or hg38
#' @import ExperimentHub
#' @return Probe annotations
#' @keywords internal
EpiMix_getInfiniumAnnotation <- function(plat = "EPIC", genome = "hg38") {
  hubID <- NULL
  if (tolower(genome) == "hg19" & toupper(plat) == "HM27")
    hubID <- "EH3672"
  if (tolower(genome) == "hg38" & toupper(plat) == "HM27")
    hubID <- "EH3673"
  if (tolower(genome) == "hg19" & toupper(plat) == "HM450")
    hubID <- "EH3674"
  if (tolower(genome) == "hg38" & toupper(plat) == "HM450")
    hubID <- "EH3675"
  if (tolower(genome) == "hg19" & toupper(plat) == "EPIC")
    hubID <- "EH3670"
  if (tolower(genome) == "hg38" & toupper(plat) == "EPIC")
    hubID <- "EH3671"
  ProbeAnnotation <- ExperimentHub::ExperimentHub()[[hubID]]
  return(ProbeAnnotation)
}

#' @title plotFEA
#' @description
#' This function visualize the functional enrichment analysis (FEA)'s barplot
#' @param dataFEA dataFEA
#' @param topBP topBP
#' @param additionalFilename additionalFilename
#' @param height Figure height
#' @param width Figure width
#' @param offsetValue offsetValue
#' @param angle angle
#' @param xleg xleg
#' @param yleg yleg
#' @param titleMain title of the plot
#' @param minY minY
#' @param maxY maxY
#' @param mycols colors to use for the plot
#' @importFrom graphics barplot
#' @importFrom graphics legend
#' @importFrom graphics par
#' @importFrom graphics text
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @export
#' @return no return value, FEA result is plotted
#' @examples
#' data(DEGsmatrix)
#' data(DiseaseList)
#' data(EAGenes)
#' data(dataFEA)
#' plotFEA(dataFEA = dataFEA[1:10,], additionalFilename = "_example",height = 20,width = 10)
plotFEA <- function(dataFEA,
                    topBP = 10,
                    additionalFilename = NULL,
                    height,
                    width,
                    offsetValue = 5,
                    angle = 90,
                    xleg = 35,
                    yleg = 5,
                    titleMain = "",
                    minY = -5,
                    maxY = 10,
                    mycols = c("#8DD3C7",
                               "#FFFFB3",
                               "#BEBADA")) {

  # Check user input

  if (all(c("Moonlight.Z.score", "FDR", "commonNg", "Diseases.or.Functions.Annotation") %in% colnames(dataFEA)) == FALSE) {
    stop("Moonlight.Z.score, FDR, commonNg, and Diseases.or.Functions.Annotation
must be column names in dataFEA")
  }

  if (!is(additionalFilename, "NULL") & !is(additionalFilename, "character")) {
    stop("additionalFilename must be either NULL or a character vector adding
a prefix or filepath to the filename of the pdf")
  }

  if (!is(additionalFilename, "NULL")) {
    pdf(additionalFilename, width, height)
  }

  par(mar = c(12, 5, 5, 1))

  dataFEA <- dataFEA[order(abs(dataFEA$Moonlight.Z.score), decreasing = TRUE), ]

  tmp <- dataFEA[seq.int(topBP), ]
  tmp <- as.data.frame(tmp)
  tmp$FDR <- as.numeric(tmp$FDR)
  tmp$Moonlight.Z.score <- as.numeric(tmp$Moonlight.Z.score)
  tmp$commonNg <- as.numeric(tmp$commonNg)
  tmp$FDR <- -log2(tmp$FDR) / 10
  tmp <- tmp[order(tmp[, 4], decreasing = TRUE), ]

  toPlot <- matrix(0, nrow = 3, ncol = nrow(tmp))
  toPlot[1, ] <- tmp$FDR
  toPlot <- toPlot[, order(toPlot[1, ], decreasing = TRUE)]
  toPlot[2, as.numeric(tmp$Moonlight.Z.score) > 0] <- as.numeric(tmp$Moonlight.Z.score[as.numeric(tmp$Moonlight.Z.score) > 0])
  toPlot[3, as.numeric(tmp$Moonlight.Z.score) < 0] <- as.numeric(tmp$Moonlight.Z.score[as.numeric(tmp$Moonlight.Z.score) < 0])
  toPlot[1, which(toPlot[1, ] > 10)] <- 10
  toPlot <- as.matrix(toPlot)

  xAxis <- barplot(toPlot,
                   beside = TRUE,
                   col = mycols,
                   ylab = "-log2FDR/10 and Moonlight z-score",
                   names = NULL,
                   main = paste0("FEA - Enriched BioFunctions", titleMain, sep = " "),
                   ylim = c(minY, maxY))

  legend("topright",
         bty = "n",
         xleg,
         yleg,
         legend = c("-log2FDR/10",
                    "Moonlight Z-score Increased",
                    "Moonlight Z-score Decreased"),
         fill = mycols, cex = 0.5)

  text(xAxis[2, ],
       par("usr")[3] + offsetValue,
       srt = angle,
       adj = 1,
       xpd = TRUE,
       labels = paste(tmp$Diseases.or.Functions.Annotation,
                      " (n=",
                      tmp$commonNg,
                      ")",
                      sep = ""),
       cex = 1)

  if (!is(additionalFilename, "NULL")) {
    dev.off()
  }
}

#' @title moonlight pipeline
#' @description moonlight is a tool for identification of cancer driver genes.
#' This function wraps the different steps of the complete analysis workflow.
#' @param BPname biological processes to use, if NULL: all processes will be used in analysis, RF for candidate; if not NULL the candidates for these processes will be determined (no learning)
#' @param dataDEGs table of differentially expressed genes
#' @param dataFilt matrix of gene expression data with genes in rows and samples in columns
#' @param Genelist Genelist
#' @param kNearest kNearest
#' @param nTF nTF
#' @param nGenesPerm nGenesPerm
#' @param nBoot nBoot
#' @param DiffGenes DiffGenes
#' @param thres.role thres.role
#' @param dataMAF A MAF file rda object for DMA
#' @param path_cscape_coding A character string to path of CScape-somatic coding file
#' @param path_cscape_noncoding A character string to path of CScape-somatic non-coding file
#' @export
#' @return table with cancer driver genes TSG and OCG.
#' @examples
#' \donttest{
#' drivers <- moonlight(dataDEGs = DEGsmatrix,
#' dataFilt = dataFilt,
#' BPname = c("apoptosis", "proliferation of cells"),
#' dataMAF = dataMAF,
#' path_cscape_coding = "css_coding.vcf.gz",
#' path_cscape_noncoding = "css_noncoding.vcf.gz")
#' }
moonlight <- function(dataDEGs,
                      dataFilt,
                      BPname = NULL,
                      Genelist= NULL,
                      kNearest = 3,
                      nGenesPerm = 2000,
                      DiffGenes = FALSE,
                      nBoot = 400,
                      nTF = NULL,
                      thres.role = 0,
                      dataMAF,
                      path_cscape_coding,
                      path_cscape_noncoding) {

  # Check user input

  # List of variable names
  variables_to_check <- c("LOC_transcription", "LOC_translation", "LOC_protein",
                          "EncodePromoters", "NCG", "DiseaseList", "EAGenes",
                          "tabGrowBlock", "knownDriverGenes")

  # Check and load variables if they do not exist
  for (variable_name in variables_to_check) {
    if (! variable_name %in% names(.GlobalEnv)) {
      data(list=c(variable_name))
    }
  }

  if (.row_names_info(dataDEGs) < 0) {
    stop("Row names were generated automatically. The input DEG table needs to
         have the gene names as rownames. Double check that genes are rownames.")
  }

  if (is(dim(dataFilt), "NULL")) {
    stop("The expression data must be non-empty with genes in rows and samples in columns")
  }

  if (!is(BPname, "NULL") && all(BPname %in% names(DiseaseList)) == FALSE) {
    stop("BPname should be NULL or a character vector containing one or more BP(s)
			   among possible BPs stored in the DiseaseList object.")
  }

  if (!is(Genelist, "NULL") && !is(Genelist, "character")) {
    stop("Genelist must be NULL or a character vector containing gene names")
  }

  if (!is(kNearest, "numeric") | !is(nGenesPerm, "numeric") | !is(nBoot, "numeric") | !is(thres.role, "numeric")) {
    stop("kNearest, nGenesPerm, nBoot, and thres.role must be numeric values")
  }

  if (!is(DiffGenes, "logical")) {
    stop("DiffGenes must be either TRUE or FALSE")
  }

  if (is(dim(dataMAF), "NULL")) {
    stop("The mutation data must be a non-empty table")
  }

  if (!is(path_cscape_coding, "character") | !is(path_cscape_noncoding, "character")) {
    stop("Paths to cscape coding and non-coding files must be character vectors")
  }

  res <- NULL

  ### Gene regulatory network
  print("-----------------------------------------")
  print("Gene regulatory network")
  print("-----------------------------------------")

  #### Parameter nTF for testing purposes
  if (is(nTF, "NULL")) {
    nTF <- nrow(dataDEGs)
  }

  if (is(Genelist, "NULL")) {
    Genelist <- rownames(dataDEGs)[seq.int(nTF)]
  }

  dataGRN <- GRN(TFs = Genelist,
                 normCounts = dataFilt,
                 DEGsmatrix = dataDEGs,
                 DiffGenes = FALSE,
                 nGenesPerm = nGenesPerm,
                 kNearest = kNearest,
                 nBoot = nBoot)

  ### Upstream regulator analysis
  print("-----------------------------------------")
  print("Upstream regulator analysis")
  print("-----------------------------------------")
  dataURA <- URA(dataGRN = dataGRN,
                 DEGsmatrix = dataDEGs,
                 BPname = BPname)

  ### Get TSG/OCG candidates using PRA
  print("-----------------------------------------")
  print("Get candidates")
  print("-----------------------------------------")
  listCandidates <- PRA(dataURA = dataURA,
                        BPname = BPname,
                        thres.role = thres.role)

  ### Get TSG/OCG driver genes using DMA
  print("-----------------------------------------")
  print("Get driver genes")
  print("-----------------------------------------")
  driverGenes <- DMA(dataMAF = dataMAF,
                     dataDEGs = dataDEGs,
                     dataPRA = listCandidates,
                     runCscape = TRUE,
                     coding_file = path_cscape_coding,
                     noncoding_file = path_cscape_noncoding,
                     results_folder = "./DMAresults")

  res <- list(dataDEGs = dataDEGs,
              dataURA = dataURA,
              listCandidates = listCandidates,
              driverGenes = driverGenes)

  return(res)

}

utils::globalVariables(c("DiseaseList"))

#' MAFtoCscape
#'
#' This function extracts columns from a MAF tibble to fit CScape input format
#' @param MAF tibble of MAF
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @importFrom tidyr separate
#' @return tibble of cscape-somatic input
#' @export
#' @examples
#' data(dataMAF)
#' MAFtoCscape(dataMAF[seq.int(2),])
MAFtoCscape <- function(MAF) {

  cscape <- MAF %>%
    filter(Variant_Type == 'SNP') %>%
    dplyr::select(Chromosome,
                  Start_Position,
                  Reference_Allele,
                  Tumor_Seq_Allele1,
                  Tumor_Seq_Allele2) %>%
    mutate(Mutant = case_when(Reference_Allele == Tumor_Seq_Allele1 ~ Tumor_Seq_Allele2,
                              Reference_Allele == Tumor_Seq_Allele2 ~ Tumor_Seq_Allele1)) %>%
    separate(Chromosome, into = c(NA, "Chr"), sep = 3) %>%
    dplyr::select(Chr, Start_Position, Reference_Allele, Mutant)

  return(cscape)
}

utils::globalVariables(c("Variant_Type", "Chromosome", "Start_Position",
                         "Reference_Allele", "Tumor_Seq_Allele1",
                         "Tumor_Seq_Allele2", "Chr", "Mutant"))

#' LiftMAF
#'
#' This function lifts a MAF file to a different genomic build.
#' @param Infile A tibble of MAF.
#' @param Current_Build A charcter string, either \code{GRCh38} or \code{GRCh37}
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @importFrom GenomicRanges makeGRangesFromDataFrame
#' @importFrom rtracklayer import.chain
#' @importFrom AnnotationHub AnnotationHub
#' @return MAF tibble with positions lifted to another build
#' @export
#' @examples
#' data(dataMAF)
#' dataMAF_example <- dataMAF[1,]
#' LiftMAF(dataMAF_example, Current_Build = 'GRCh38')
LiftMAF <- function(Infile,
                    Current_Build) {

  # The input file is assumed to be maf_tibble, this file is then lifted to
  # either 38 or 37 and return as a tibble

  ah <- AnnotationHub()

  if (Current_Build == "GRCh38") {
    chain <- ah[['AH14108']]   #UCSC  hg38ToHg19.over.chain.gz
  } else if (Current_Build == "GRCh37") {
    chain <- ah[['AH14150']]   #UCSC  hg19ToHg38.over.chain.gz
  } else {
    print("Error: Build must be either GRCh38 or GRCh37")
    return
  }

  # Change to Grange format
  infile_GRange <- makeGRangesFromDataFrame(Infile,
                                            start.field = "Start_Position",
                                            end.field = "End_Position",
                                            seqnames.field = "Chromosome",
                                            keep.extra.columns = TRUE)

  # Do liftover
  infile_GRange_lifted <- liftOver(x = infile_GRange, chain = chain)

  # Recreate maf column names and tibble format
  outfile_tibble_lifted <- as_tibble(infile_GRange_lifted) %>%
    dplyr::rename(Start_Position = start,
                  End_Position = end,
                  Chromosome = seqnames,
                  Strand = strand) %>%
    dplyr::select(-c(group, group_name, width))

  return(outfile_tibble_lifted)
}

utils::globalVariables(c("liftOver", "start", "end", "seqnames", "strand",
                         "group", "group_name", "width"))

#' GMA
#' This function carries out Gene Methylation Analysis
#' @param dataMET A data matrix containing the methylation data
#' where the CpG probes are in the rows and samples are in the
#' columns
#' @param dataEXP A data matrix containing the gene expression
#' data where the genes are in the rows and the samples are in
#' the columns
#' @param dataPRA A table containing the output of the PRA
#' function
#' @param dataDEGs A table containing the output of a DEA
#' where gene names are rownames
#' @param sample_info A table containing information on the
#' samples. This table needs to contain two columns called
#' primary and sample.type. The primary column contains
#' sample names which should be the same as the column names
#' in dataMET. The sample.type column indicates for each
#' sample if it is a Cancer or Normal sample.
#' @param met_platform A character string representing the
#' microarray type that was used to collect the methylation
#' data. This can either be HM27, HM450 or EPIC. Default
#' is HM450.
#' @param prevalence_filter A float or NULL representing if
#' a prevalence filter should be applied or not. Default is NULL,
#' meaning a prevalence filter will not be applied. If a float
#' is specified, a prevalence filter will be applied where
#' methylation states of probes will be altered depending on the
#' threshold of prevalence supplied as prevalence_filter. For
#' example, if prevalence_filter = 20, it means that if the prevalence
#' of the hyper- or hypomethylated CpG probe exceeds 20, the methylation
#' state will be unchanged but if the prevalence is lower than 20 the
#' methylation state will be changed to NA, meaning no methylation state
#' was detected. In case of dual methylated probes, the methylation
#' state will stay dual if both the prevalence of hyper- and
#' hypomethylations exceed 20, but if only one of the prevalences exceed
#' 20 the dual state will be changed to the state exceeding 20. If none
#' of the prevalences exceed 20, the dual state will be changed to NA.
#' @param output_dir Path to where the results will be stored.
#' If this directory does not exist, it will be created by the
#' function. Default is ./GMAresults.
#' @param cores Number of cores to be used. Default is 1.
#' @param roadmap.epigenome.ids A character string representing
#' the epigenome ID that will be used to select enhancers.
#' Since enhancers are tissue-specific, the tissue type needs
#' to be specified in EpiMix. The enhancers are found from the
#' RoadmapEpigenome project and the IDs can be found from
#' Figure 2 in the publication with doi: 10.1038/nature14248.
#' Default is NULL.
#' @param roadmap.epigenome.groups A character string representing
#' the epigenome group that will be used to select enhancers.
#' Details are provided above with the roadmap.epigenome.ids
#' parameter. Default is NULL.
#' @import EpiMix
#' @import tibble
#' @import dplyr
#' @import tidyr
#' @import readr
#' @importFrom fuzzyjoin genome_left_join
#' @importFrom stringr str_c
#' @importFrom magrittr "%>%"
#' @importFrom BiocGenerics as.data.frame
#' @return List of two elements, containing predicted oncogenes
#' and tumor suppressors. Additionally, various output files are
#' saved in the specified output directory:
#' DEG_Methylation_Annotations.rda, Oncogenic_mediators_methylation_summary.rda,
#' EpiMix_Results_Enhancer.rds, EpiMix_Results_Regular.rds,
#' FunctionalPairs_Enhancer.csv, FunctionalPairs_Regular.csv,
#' FunctionalProbes_Regular.rds
#' @export
#' @examples
#' data("dataMethyl")
#' data("dataFilt")
#' data("dataPRA")
#' data("DEGsmatrix")
#' data("LUAD_sample_anno")
#' data("NCG")
#' data("EncodePromoters")
#' data("MetEvidenceDriver")
#' pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
#' dataFilt_subset <- dataFilt[,1:10]
#' colnames(dataFilt_subset) <- sub(pattern, "\\1", colnames(dataFilt_subset))
#' dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt_subset,
#' dataPRA = dataPRA, dataDEGs = DEGsmatrix,
#' sample_info = LUAD_sample_anno, met_platform = "HM450",
#' prevalence_filter = NULL,
#' output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
#' roadmap.epigenome.groups = NULL)
GMA <- function(dataMET,
                dataEXP,
                dataPRA,
                dataDEGs,
                sample_info,
                met_platform = "HM450",
                prevalence_filter = NULL,
                output_dir = "./GMAresults",
                cores = 1,
                roadmap.epigenome.ids = NULL,
                roadmap.epigenome.groups = NULL) {

  # Check met_platform input
  if (!met_platform %in% c("HM27", "HM450", "EPIC")) {
    stop("met_platform must be a character string of either HM27, HM450 or EPIC")
  }

  if (dir.exists(output_dir)) {
    print("Output folder already exits")
  }
  else {
    dir.create(path = output_dir,
               showWarnings = TRUE,
               recursive = TRUE)
  }


  ## EpiMix ---------------------------------------------------------------

  ## Run EpiMix regular mode
  EpiMix_regular <- EpiMix(methylation.data = dataMET,
                           gene.expression.data = dataEXP,
                           sample.info = sample_info,
                           mode = "Regular",
                           group.1 = "Cancer",
                           group.2 = "Normal",
                           met.platform = met_platform,
                           correlation = "negative",
                           OutputRoot = output_dir,
                           cores = cores)

  ## Run EpiMix enhancer mode
  EpiMix_enhancer <- EpiMix(methylation.data = dataMET,
                            gene.expression.data = dataEXP,
                            sample.info = sample_info,
                            mode = "Enhancer",
                            roadmap.epigenome.ids = roadmap.epigenome.ids,
                            roadmap.epigenome.groups = roadmap.epigenome.groups,
                            group.1 = "Cancer",
                            group.2 = "Normal",
                            met.platform = met_platform,
                            correlation = "negative",
                            OutputRoot = output_dir,
                            cores = cores)

  # Convert functional pairs results from EpiMix to tibble
  EpiMix_regular_FP <- as_tibble(EpiMix_regular$FunctionalPairs) %>%
    dplyr::rename("Hugo_Symbol" = "Gene")
  EpiMix_enhancer_FP <- as_tibble(EpiMix_enhancer$FunctionalPairs) %>%
    dplyr::rename("Hugo_Symbol" = "Gene")

  # Add enhancer prefix to columns in EpiMix_enhancer_FP
  # Create column that specifies that identified probes are in enhancers
  EpiMix_enhancer_FP <- EpiMix_enhancer_FP %>%
    dplyr::rename_with(.cols = -1,
                       .fn = ~str_c(., "_enhancer")) %>%
    dplyr::mutate(Probe_in_enhancer = "Enhancer") %>%
    dplyr::relocate(Probe_in_enhancer,
                    .after = Probe_enhancer)


  ## Initialize data --------------------------------------------------------

  # Convert PRA data to tibble
  PRA_tbl <- PRAtoTibble(dataPRA = dataPRA)

  # Get NCG data
  NCG <- get("NCG")


  ## DEG_Methylation_Annotations --------------------------------------------

  ## Intersect DEA results with EpiMix results
  ## Create DEG_Methylation_Annotations

  # Convert DEA data to tibble
  dataDEGs <- dataDEGs %>%
    tibble::rownames_to_column(var = "Hugo_Symbol") %>%
    as_tibble()

  # Bind EpiMix regular and enhancer functional-pairs results
  # Join with DEA data and with PRA data containing information
  # about oncogenic mediators
  # Add NCG annotations
  DEG_Methylation_Annotations <- EpiMix_regular_FP %>%
    bind_rows(EpiMix_enhancer_FP) %>%
    right_join(x = .,
               y = dataDEGs,
               by = "Hugo_Symbol") %>%
    left_join(x = .,
              y = PRA_tbl,
              by = "Hugo_Symbol") %>%
    left_join(x = .,
              y = NCG,
              by = c("Hugo_Symbol" = "symbol")) %>%
    dplyr::select(-c("Fold change of gene expression",
                     "Fold change of gene expression_enhancer"))

  ## Get chromosome, start and end locations of probes

  # Retrieve annotations of probes and select only chromosome, start and
  # end positions
  probe_annotation <- EpiMix_getInfiniumAnnotation(plat = met_platform,
						   genome = "hg38") %>%
    BiocGenerics::as.data.frame() %>%
    as_tibble(rownames = "Probe") %>%
    dplyr::select(Probe,
                  seqnames,
                  start,
                  end) %>%
    dplyr::rename(Chromosome = seqnames)

  # Add probe annotations (chromosome, start and end positions)
  # to DEG_Methylation_Annotations
  DEG_Methylation_Annotations <- DEG_Methylation_Annotations %>%
    left_join(x = .,
              y = probe_annotation,
              by = "Probe") %>%
    left_join(x = .,
              y = probe_annotation,
              by = c("Probe_enhancer" = "Probe")) %>%
    unite(start,
          start.x,
          start.y,
          na.rm = TRUE) %>%
    unite(end,
          end.x,
          end.y,
          na.rm = TRUE) %>%
    unite(chromosome,
          Chromosome.x,
          Chromosome.y,
          na.rm = TRUE) %>%
    mutate(start = na_if(start,
                         ""),
           end = na_if(end,
                       ""),
           chromosome = na_if(chromosome,
                              "")) %>%
    dplyr::relocate(chromosome,
                    start,
                    end,
                    .after = "Hugo_Symbol") %>%
    mutate(start = as.numeric(start),
           end = as.numeric(end))

  ## Add Encode promoter annotations to DEG_Methylation_Annotations

  # Get promoter annotations and join with DEG_Methylation_Annotations
  promoters <- get("EncodePromoters") %>%
    mutate(Annotation = "Promoter") %>%
    dplyr::select(X1,
                  X2,
                  X3,
                  Annotation) %>%
    dplyr::rename(chromosome_annot = "X1",
                  promoter_start = "X2",
                  promoter_end = "X3")
  DEG_Methylation_Annotations <- genome_left_join(x = DEG_Methylation_Annotations,
                                                  y = promoters,
                                                  by = c(chromosome = "chromosome_annot",
                                                         start = "promoter_start",
                                                         end = "promoter_end")) %>%
    dplyr::select(-c("chromosome_annot")) %>%
    dplyr::rename(Probe_in_promoter = Annotation) %>%
    dplyr::relocate(contains("promoter"),
                    .before = NCG_driver)


  ## Oncogenic_mediators_methylation_summary --------------------------------

  # Get MetEvidenceDriver
  MetEvidenceDriver <- get("MetEvidenceDriver")

  ## Intersect oncogenic mediator results with EpiMix functional pairs results
  ## Create Oncogenic_mediators_methylation_summary

  # Left join PRA data and functional pairs results from EpiMix
  Oncogenic_mediators_methylation_summary <- PRA_tbl %>%
    left_join(x = .,
              y = EpiMix_regular_FP,
              by = "Hugo_Symbol")

  # Update methylation state of probes based on prevalence
  if (is.null(prevalence_filter)) {
    state_status <- "State"
  } else {

    Oncogenic_mediators_methylation_summary <- Oncogenic_mediators_methylation_summary %>%
      dplyr::select(Hugo_Symbol, Moonlight_Oncogenic_Mediator,
                    Probe, State, contains("Prevalence")) %>%
      # Update methylation state of probes based on prevalence of
      # methylation in patients
      dplyr::mutate(State_upd = case_when(State == "Hyper" &
                                            `Prevalence of hyper (%)` > prevalence_filter ~ "Hyper",
                                          State == "Hypo" &
                                            `Prevalence of hypo (%)` > prevalence_filter ~ "Hypo",
                                          State == "Dual" &
                                            `Prevalence of hyper (%)` > prevalence_filter &
                                            `Prevalence of hypo (%)` > prevalence_filter ~ "Dual",
                                          State == "Dual" &
                                            `Prevalence of hyper (%)` < prevalence_filter &
                                            `Prevalence of hypo (%)` > prevalence_filter ~ "Hypo",
                                          State == "Dual" &
                                            `Prevalence of hyper (%)` > prevalence_filter &
                                            `Prevalence of hypo (%)` < prevalence_filter ~ "Hyper"))

    state_status <- "State_upd"

  }

  # Count number of CpG probes that are hyper/hypo/dual methylated for
  # each oncogenic mediator
  # Add oncogenic mediator role (putative TSG/OCG) for each oncogenic mediator
  Oncogenic_mediators_methylation_summary <- Oncogenic_mediators_methylation_summary %>%
    group_by(Hugo_Symbol,
             !!sym(state_status)) %>%
    dplyr::count() %>%
    pivot_wider(.,
                id_cols = "Hugo_Symbol",
                names_from = !!sym(state_status),
                values_from = "n") %>%
    dplyr::rename("No" = "NA") %>%
    full_join(x = .,
              y = PRA_tbl,
              by = "Hugo_Symbol") %>%
    dplyr::select(-"Moonlight_gene_z_score") %>%
    dplyr::relocate(.data = .,
                    "Moonlight_Oncogenic_Mediator",
                    .after = "Hugo_Symbol") %>%
    replace(is.na(.), 0) %>%
    ungroup

  # If Hypo, Hyper, Dual or No columns do not exist, create them containing all 0s
  columns_to_check <- c("Hypo", "Hyper", "Dual", "No")

  # Iterate over each column
  for (col_name in columns_to_check) {
    # If the column does not exist, create it and fill it with 0s
    if (!(col_name %in% colnames(Oncogenic_mediators_methylation_summary))) {
      Oncogenic_mediators_methylation_summary[[col_name]] <- 0
    }
  }

  # Temporary copy of Oncogenic_mediators_methylation_summary where all values
  # greater than 1 are set to 1 to allow for overlap with MetEvidenceDriver
  # table
  Oncogenic_mediators_methylation_summary_tmp <- Oncogenic_mediators_methylation_summary %>%
    mutate(across(where(is.integer), ~ if_else(. > 0, 1, .))) %>%
    replace(is.na(.), 0)

  # Left join Oncogenic_mediators_methylation_summary_tmp with MetEvidenceDriver
  Oncogenic_mediators_methylation_summary_tmp <- Oncogenic_mediators_methylation_summary_tmp %>%
    left_join(MetEvidenceDriver,
              by = c("Hypo", "Hyper", "No", "Dual", "Moonlight_Oncogenic_Mediator"))

  # Add Evidence column to original Oncogenic_mediators_methylation_summary
  Oncogenic_mediators_methylation_summary <- Oncogenic_mediators_methylation_summary %>%
    mutate(Evidence = Oncogenic_mediators_methylation_summary_tmp$Evidence)

  # Rename methylation state columns
  Oncogenic_mediators_methylation_summary <- Oncogenic_mediators_methylation_summary %>%
    dplyr::rename("No_methyl_num" = "No",
                  "Hypo_methyl_num" = "Hypo",
                  "Hyper_methyl_num" = "Hyper",
                  "Dual_methyl_num" = "Dual")

  # Print number of oncogenic mediators with different level of evidence
  count_evidence <- map(Oncogenic_mediators_methylation_summary$Evidence %>%
                          unique,
                        function(x) {
                          num_genes <- Oncogenic_mediators_methylation_summary %>%
                            dplyr::filter(Evidence == x) %>%
                            distinct(Hugo_Symbol) %>%
                            pull %>%
                            length
                          print(paste(num_genes,
                                      "oncogenic mediator(s) out of",
                                      Oncogenic_mediators_methylation_summary %>%
                                        distinct(Hugo_Symbol) %>%
                                        pull %>%
                                        length,
                                      "were found in the",
                                      x,
                                      "evidence category")) })


  ## Get driver genes --------------------------------

  # Get driver genes which are those oncogenic mediators that have a
  # significant association between expression and methylation and
  # where pattern of presumed driver role matches with methylation
  # pattern
  TSG <- Oncogenic_mediators_methylation_summary %>%
    dplyr::filter(Evidence == "Agreement",
                  Moonlight_Oncogenic_Mediator == "TSG") %>%
    pull(Hugo_Symbol)
  OCG <- Oncogenic_mediators_methylation_summary %>%
    dplyr::filter(Evidence == "Agreement",
                  Moonlight_Oncogenic_Mediator == "OCG") %>%
    pull(Hugo_Symbol)


  ## Save data ------------------------------------------------------------

  # Save DEG_Methylation_Annotations as rda file
  save(DEG_Methylation_Annotations,
       file = paste(output_dir,
                    "DEG_Methylation_Annotations.rda",
                    sep = "/"))

  # Save Oncogenic_mediators_methylation_summary as rda file
  save(Oncogenic_mediators_methylation_summary,
       file = paste(output_dir,
                    "Oncogenic_mediators_methylation_summary.rda",
                    sep = "/"))

  return(list(TSG = TSG,
              OCG = OCG))


}

utils::globalVariables(c("Probe_in_enhancer", "Probe_enhancer", "Probe",
			 "start.x", "start.y", "end.x", "end.y", "chromosome",
			 "Chromosome.x", "Chromosome.y", "NCG_driver", "State",
			 "Evidence"))

# Test GMA function

# Run example of GMA function
data("dataMethyl")
data("dataFilt")
data("dataPRA")
data("DEGsmatrix")
data("LUAD_sample_anno")
data("NCG")
data("EncodePromoters")
data("MetEvidenceDriver")
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))
dataGMA_test <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
                    dataPRA = dataPRA, dataDEGs = DEGsmatrix,
                    sample_info = LUAD_sample_anno, met_platform = "HM450",
                    prevalence_filter = NULL,
                    output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
                    roadmap.epigenome.groups = NULL)

# Load example data of GMA serving as reference point
data(dataGMA)

# Test that output of GMA is as expected
test_that("output of GMA is a list with 2 elements", {
  expect_type(dataGMA_test, "list")
  expect_named(dataGMA_test, c("TSG", "OCG"))
  expect_true(all(sapply(dataGMA_test, is.character)))
})

# Test that output of GMA is as expected compared to reference / example data
test_that("GMA output is identical to reference point", {
  expect_equal(dataGMA_test, dataGMA)
})

#' @title Pattern Recognition Analysis (PRA)
#' @description
#' This function carries out the pattern recognition analysis
#' @param dataURA output URA function
#' @param BPname BPname
#' @param thres.role thres.role
#' @importFrom randomForest randomForest
#' @importFrom utils data
#' @return returns list of TSGs and OCGs when biological processes are provided, otherwise a randomForest based classifier that can be used on new data
#' @export
#' @examples
#' data(dataURA)
#' data(DiseaseList)
#' data(tabGrowBlock)
#' data(knownDriverGenes)
#' dataPRA <- PRA(dataURA = dataURA[seq.int(2),],
#' BPname = c("apoptosis","proliferation of cells"),
#' thres.role = 0)
PRA <- function(dataURA,
                BPname,
                thres.role = 0) {

  # List of variable names
  variables_to_check <- c("DiseaseList", "tabGrowBlock", "knownDriverGenes")

  # Check and load variables if they do not exist
  for (variable_name in variables_to_check) {
    if (! variable_name %in% names(.GlobalEnv)) {
      data(list=c(variable_name))
    }
  }

  # Check user input

  DiseaseList <- get("DiseaseList")

  if (!is(BPname, "NULL") && all(BPname %in% names(DiseaseList)) == FALSE) {
    stop("BPname should be NULL or a character vector containing one or more BP(s)
among possible BPs stored in the DiseaseList object.")
  }

  if (is(dim(dataURA), "NULL")) {
    stop("The URA data must be non-empty with genes in rows and BPs in columns")
  }

  if (all(colnames(dataURA) %in% names(DiseaseList)) == FALSE) {
    stop("The columns in the URA data must be BPs among possible BPs stored in the DiseaseList")
  }

  if (!is(thres.role, "numeric")) {
    stop("Thres.role must be numeric")
  }

  tabGrowBlock <- get("tabGrowBlock")
  knownDriverGenes <- get("knownDriverGenes")

  names.blocking <- tabGrowBlock[which(tabGrowBlock$Cancer.blocking == "Increasing"), "Disease"]
  names.growing <- tabGrowBlock[which(tabGrowBlock$Cancer.growing == "Increasing"), "Disease"]

  if (is(BPname, "NULL")) {

    common.genes.tsg <- intersect(knownDriverGenes$TSG, rownames(dataURA))
    common.genes.ocg <- intersect(knownDriverGenes$OCG, rownames(dataURA))

    dataTrain <- data.frame(dataURA[c(common.genes.tsg, common.genes.ocg), ],
                            "labels" = as.factor(c(rep(1, length(common.genes.tsg)), 
                                                   rep(0, length(common.genes.ocg)))))

    fit.rf <- randomForest((labels) ~ ., data = dataTrain, importance = TRUE)

    return(fit.rf)

  } else {

    ind.proc.growing  <- which(BPname %in% names.growing)
    ind.proc.blocking <- which(BPname %in% names.blocking)

    names.genes.tsg <- names(which(dataURA[, BPname[ind.proc.growing]] < -thres.role & dataURA[, BPname[ind.proc.blocking]] >  thres.role))
    names.genes.ocg <- names(which(dataURA[, BPname[ind.proc.growing]] >  thres.role & dataURA[, BPname[ind.proc.blocking]] < -thres.role))

    if (length(names.genes.tsg) > 0) {
      scores.tsg <- apply(abs(dataURA[names.genes.tsg, , drop=FALSE]), 1, mean)
    } else {
      scores.tsg <- NULL
    }

    if (length(names.genes.ocg) > 0) {
      scores.ocg <- apply(abs(dataURA[names.genes.ocg, , drop=FALSE]), 1, mean)
    } else {
      scores.ocg <- 0
    }

    return(list("TSG" = scores.tsg, "OCG" = scores.ocg))

  }
}

utils::globalVariables(c("DiseaseList"))

# Test PRA function

# Run example of PRA
data(dataURA)
data(DiseaseList)
data(tabGrowBlock)
data(knownDriverGenes)
dataPRA_test <- PRA(dataURA = dataURA,
                    BPname = c("apoptosis", "proliferation of cells"),
                    thres.role = 0)

# Load example data of PRA serving as reference point
data(dataPRA)

# Test that output of PRA is as expected
test_that("output of PRA is a list with 2 elements", {
  expect_type(dataPRA_test, "list")
  expect_named(dataPRA_test, c("TSG", "OCG"))
  expect_named(dataPRA_test$TSG)
  expect_named(dataPRA_test$OCG)
  expect_true(all(sapply(dataPRA_test, is.numeric)))
})

# Test that output of PRA is as expected compared to reference / example data
test_that("PRA output is identical to reference point", {
  expect_equal(dataPRA_test, dataPRA)
})

#' @title plotCircos
#' @description
#' This function visualize the plotCircos
#' @param listMoonlight output Moonlight function
#' @param additionalFilename additionalFilename
#' @param intensityColOCG intensityColOCG
#' @param intensityColTSG intensityColTSG
#' @param intensityColDual intensityColDual
#' @param fontSize fontSize
#' @param listMutation listMutation
#' @importFrom RColorBrewer brewer.pal
#' @importFrom circlize circos.par
#' @importFrom circlize circlize
#' @importFrom circlize circos.clear
#' @importFrom circlize circos.initialize
#' @importFrom circlize circos.trackPlotRegion get.cell.meta.data circos.text circos.rect
#' @importFrom grDevices dev.off rainbow rgb
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @export
#' @return no return value, plot is saved
#' @examples
#' data('listMoonlight')
#' plotCircos(listMoonlight = listMoonlight, additionalFilename = "_ncancer5")
plotCircos <- function(listMoonlight,
                       listMutation = NULL,
                       additionalFilename = NULL,
                       intensityColOCG = 0.5,
                       intensityColTSG = 0.5,
                       intensityColDual = 0.5,
                       fontSize = 1) {

  # Check user input

  if (is(listMoonlight, "list") == FALSE) {
    stop("listMoonlight must be a list")
  }

  if (!is(listMutation, "list") & !is(listMutation, "NULL")) {
    stop("listMutation must be either NULL or a list")
  }

  if (!is(intensityColOCG, "numeric") | !is(intensityColTSG, "numeric") | !is(intensityColDual, "numeric") | !is(fontSize, "numeric")) {
    stop("intensityColOCG, intensityColTSG, intensityColDual, and fontSize must be numeric")
  }

  if (!is(additionalFilename, "NULL") & !is(additionalFilename, "character")) {
    stop("additionalFilename must be either NULL or a character vector containing
filename of plot")
  }

  ### Prepare data
  n <- length(listMoonlight)
  mycancertypes <- names(listMoonlight)
  mytsg <- myocg <- NULL

  for (i in seq.int(n)) {
    mytsg <- c(mytsg, list(names(listMoonlight[[i]]$listCandidates$TSG)))
    myocg <- c(myocg, list(names(listMoonlight[[i]]$listCandidates$OCG)))
  }
  names(mytsg) <- names(myocg) <- mycancertypes

  n.mygenes <- vapply(mytsg, length, integer(1)) + vapply(myocg, length, integer(1))
  mynames <- mycancertypes
  ind.rm <- which(n.mygenes == 0)

  if (length(ind.rm) > 0) {
    mytsg <- mytsg[-ind.rm]
    myocg <-myocg[-ind.rm]

    n.mygenes <- vapply(mytsg, length, integer(1)) + vapply(myocg, length, integer(1))
    mynames <- mycancertypes[-ind.rm]
    listMutation <- listMutation[-ind.rm]
  }

  n <- n - length(ind.rm)

  ntsg <- vapply(mytsg, length, integer(1))
  nocg <- vapply(myocg, length, integer(1))

  if (!is(additionalFilename, "NULL")) {
    pdf(paste0("circos_ocg_tsg", additionalFilename, ".pdf"), width = 16, height = 16)
  }

  df1 <- data.frame("order" = seq.int(n),
                    "cancertype" = mynames,
                    "xmin" = rep(0, n),
                    "xmax" = n.mygenes)

  ### Plot sectors (outer part)
  par(mar = c(1, 1, 6, 6))
  circlize::circos.clear()

  ### Basic circos graphic parameters
  circlize::circos.par(cell.padding = c(0, 0, 0, 0),
                       track.margin = c(0, 0.15),
                       start.degree = 90,
                       gap.degree = 4)

  if (n > 12) {
    mycols <- rainbow(n + 4)[seq.int(n)]
  } else {
    mycols <- RColorBrewer::brewer.pal(n, "Set3")
  }

  ### Sector details
  circlize::circos.initialize(factors = df1$cancertype,
                              xlim = cbind(df1$xmin,
                                           df1$xmax))

  plot_settings <- function(x, y) {

    # Select details of current sector
    name <- get.cell.meta.data("sector.index")
    i <- get.cell.meta.data("sector.numeric.index")
    xlim <- get.cell.meta.data("xlim")
    ylim <- get.cell.meta.data("ylim")

    # Text direction (dd) and adjusmtents (aa)
    theta <- circlize(mean(xlim), 1.3)[1, 1] %% 360
    dd <- ifelse(theta < 90 || theta > 270, "clockwise", "reverse.clockwise")
    aa <- c(1, 0.5)
    if (theta < 90 || theta > 270) {
      aa <- c(0, 0.5)
    }

    circlize::circos.text(x = mean(xlim),
                          y = 1.7,
                          labels = paste0(name,
                                          "\n(",
                                          vapply(myocg, length, integer(1))[i],
                                          ", ",
                                          vapply(mytsg, length, integer(1))[i],")"),
                          facing = dd,
                          cex = fontSize,
                          adj = aa)

    # Plot main sector
    circlize::circos.rect(xleft = xlim[1],
                          xright = xlim[2],
                          ybottom = ylim[1],
                          ytop = ylim[2],
                          col = mycols[i], border = mycols[i])

    circlize::circos.rect(xleft = xlim[1],
                          ybottom = ylim[1],
                          xright = xlim[2] - vapply(mytsg, length, integer(1))[i],
                          ytop = ylim[1] + 0.3,
                          col = "darkgreen", border = "darkgreen")

    circlize::circos.rect(xleft = vapply(myocg, length, integer(1))[i],
                          ybottom = ylim[1],
                          xright = xlim[2],
                          ytop = ylim[1] + 0.3,
                          col = "goldenrod", border = "goldenrod")

    # White line all the way around
    circlize::circos.rect(xleft = xlim[1],
                          ybottom = 0.3,
                          xright = xlim[2],
                          ytop = 0.32,
                          col = "white", border = "white")
  }

  ### Plot sectors
  circlize::circos.trackPlotRegion(ylim = c(0, 1),
                                   factors = df1$cancertype,
                                   track.height = 0.1,
                                   # panel.fun for each sector
                                   panel.fun = plot_settings)

  if (!is(listMutation, "NULL")) {

    mycols.mut <- c("white", "darkviolet", "tomato3")

    for (i in seq.int(length(listMutation))) {

      if (length(myocg[[i]]) > 0) {

        myoverlap  <-  myocg[[i]] %in% unique(as.matrix(listMutation[[i]][which(listMutation[[i]][, "Consequence"]=="inframe_deletion"), 1]))
        myoverlap2 <-  myocg[[i]] %in% unique(as.matrix(listMutation[[i]][which(listMutation[[i]][, "Consequence"]=="inframe_insertion"), 1]))
        myoverlap3 <-  myocg[[i]] %in% unique(as.matrix(listMutation[[i]][which(listMutation[[i]][, "Consequence"]=="missense_variant"), 1]))

        cnt <- 0

        for (j in seq.int(length(myocg[[i]]))) {

          if (myoverlap[j]) {

            circlize::circos.rect(sector.index = df1$cancertype[i],
                                  xleft = cnt,
                                  ybottom = -1/2,
                                  xright = cnt + 1,
                                  ytop = -0.33,
                                  col = mycols.mut[2],
                                  border = mycols.mut[2])

          }

          if (myoverlap2[j]) {

            circlize::circos.rect(sector.index = df1$cancertype[i],
                                  xleft = cnt,
                                  ybottom = -0.77,
                                  xright = cnt + 1,
                                  ytop = -0.6,
                                  col = mycols.mut[2],
                                  border = mycols.mut[2])


          }

          if (myoverlap3[j]) {

            circlize::circos.rect(sector.index = df1$cancertype[i],
                                  xleft = cnt,
                                  ybottom = -1.04,
                                  xright = cnt + 1,
                                  ytop = -0.87,
                                  col = mycols.mut[2],
                                  border = mycols.mut[2])

          }

          cnt <- cnt+1

        }
      }

      if (length(mytsg[[i]]) > 0) {

        myoverlap  <-  mytsg[[i]] %in% unique(as.matrix(listMutation[[i]][which(listMutation[[i]][, "Consequence"]=="inframe_deletion"), 1]))
        myoverlap2 <-  mytsg[[i]] %in% unique(as.matrix(listMutation[[i]][which(listMutation[[i]][, "Consequence"]=="inframe_insertion"), 1]))
        myoverlap3 <-  mytsg[[i]] %in% unique(as.matrix(listMutation[[i]][which(listMutation[[i]][, "Consequence"]=="missense_variant"), 1]))

        cnt <- 0

        for (j in seq.int(length(mytsg[[i]]))) {

          if (myoverlap[j]) {

            circlize::circos.rect(sector.index = df1$cancertype[i],
                                  xleft = nocg[i] + cnt,
                                  ybottom = -1/2,
                                  xright = nocg[i] + cnt + 1,
                                  ytop = -0.33,
                                  col = mycols.mut[3],
                                  border = mycols.mut[3])

          }

          if (myoverlap2[j]) {

            circlize::circos.rect(sector.index = df1$cancertype[i],
                                  xleft = nocg[i] + cnt,
                                  ybottom = -0.77,
                                  xright = nocg[i] + cnt + 1,
                                  ytop = -0.6,
                                  col = mycols.mut[3],
                                  border = mycols.mut[3])

          }

          if (myoverlap3[j]) {

            circlize::circos.rect(sector.index = df1$cancertype[i],
                                  xleft=nocg[i] + cnt,
                                  ybottom = -1.04,
                                  xright = nocg[i] + cnt + 1,
                                  ytop = -0.87,
                                  col = mycols.mut[3],
                                  border = mycols.mut[3])

          }

          cnt <- cnt + 1

        }
      }
    }
  }

  ## plot inner part
  ### OCG - OCG

  mycol.ocg <- rgb(10/255, 99/255, 12/255, intensityColOCG)

  for (i in seq.int(n-1)) {

    print(paste("cancertype", i, "out of", n))

    for (j in seq.int(i+1, n)) {

      if (i != j) {

        ind <- which(myocg[[i]] %in% myocg[[j]])

        for (k in ind) {

          ind2 <- which(myocg[[j]] == myocg[[i]][k])
          circlize::circos.link(sector.index1 = df1$cancertype[i],
                                point1 = c(k-1, k),
                                sector.index2 = df1$cancertype[j],
                                point2 = c(ind2-1, ind2),
                                col = mycol.ocg)

        }
      }
    }
  }

  ### TSG - TSG

  mycol.tsg <- rgb(217/255, 164/255, 50/255, intensityColTSG)

  for (i in seq.int(n-1)) {

    print(paste("cancertype", i, "out of", n))

    for (j in seq.int(i+1, n)) {

      if (i != j) {

        ind <- which(mytsg[[i]] %in% mytsg[[j]])

        for (k in ind) {

          ind2 <- which(mytsg[[j]] == mytsg[[i]][k])
          circlize::circos.link(sector.index1 = df1$cancertype[i],
                                point1 = c(vapply(myocg, length, integer(1))[i] + k - 1,
                                           vapply(myocg, length, integer(1))[i] + k),
                                sector.index2 = df1$cancertype[j],
                                point2 = c(vapply(myocg, length, integer(1))[j] + ind2 - 1,
                                           vapply(myocg, length, integer(1))[j] + ind2),
                                col = mycol.tsg)

        }
      }
    }
  }

  # ### TSG - OSG

  mycol.tsg.osg <- rgb(252/255, 51/255, 57/255, intensityColDual)

  for (i in seq.int(n)) {

    print(paste("cancertype", i, "out of", n))

    for (j in seq.int(n)) {

      ind <- which(myocg[[i]] %in% mytsg[[j]])

      for (k in ind) {

        ind2 <- which(mytsg[[j]] == myocg[[i]][k])
        circlize::circos.link(sector.index1 = df1$cancertype[i],
                              point1 = c(k-1, k),
                              sector.index2 = df1$cancertype[j],
                              point2 = c(vapply(myocg, length, integer(1))[j] + ind2 - 1, vapply(myocg, length, integer(1))[j] + ind2),
                              col = mycol.tsg.osg)

      }
    }
  }

  if (!is(additionalFilename, "NULL")) {
    dev.off()
  }
}

utils::globalVariables(c("is"))

#' RunCscape_somatic
#'
#' This function retrive cscape-scores to SNPs
#' @param input Input matching cscape input
#' @param coding_file cscape_table with coding scores
#' @param noncoding_file cscape_table with noncoding scores
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @importFrom tidyr unite nest unnest
#' @importFrom purrr pmap
#' @importFrom readr parse_guess
#'
#' @return returns a tibble with a score and remark for each SNP
#' @export
#' @examples
#' \donttest{
#' cscape_out <- RunCscape_somatic(input, coding_file, noncoding_file)
#' }
RunCscape_somatic <- function(input,
                              coding_file,
                              noncoding_file) {

  cscape_in <- input %>%
    unite(col = Ranges, c("Chr", "Start_Position"), sep = ":", remove = FALSE) %>%
    unite(col = Ranges, c("Ranges", "Start_Position"), sep = "-", remove = FALSE)

  # When using pmap important to call the list and varibales the same name
  cscape_annot <- cscape_in %>%
    mutate(file_coding = coding_file, file_noncoding = noncoding_file) %>%
    dplyr::select(-Start_Position) %>%
    mutate(Mydata = pmap(.l = list(Ranges,
                                   Reference_Allele,
                                   Mutant,
                                   file_coding,
                                   file_noncoding),
                         .f = tabix_func)) %>%
    unnest(cols = Mydata)

  # Make standard CScape output format
  cscape_out <- cscape_annot %>%
    separate(col = Ranges, into = c("Chr", "Position", NA)) %>%
    dplyr::rename(Reference = Reference_Allele) %>%
    dplyr::select(-c(file_coding, file_noncoding)) %>%
    relocate(Remark, .after = last_col()) %>%
    mutate(across(.fns = parse_guess))

  return(cscape_out)

}

utils::globalVariables(c("Ranges", "Start_Position", "Reference_Allele", "Mutant",
                         "file_coding", "file_noncoding", "Mydata", "Remark"))

#' GLS
#' This function carries out gene literature search.
#' @param genes A character string containing the genes
#' to search in PubMed database
#' @param query_string A character string containing words
#' in query to follow the gene of interest. Default is
#' "AND cancer AND driver" resulting in a final query of
#' "Gene AND cancer AND driver". Standard PubMed syntax
#' can be used in the query. For example Boolean operators
#' AND, OR, NOT can be applied and tags such as [AU],
#' [TITLE/ABSTRACT], [Affiliation] can be used.
#' @import easyPubMed
#' @import tibble
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @importFrom purrr map
#' @return A tibble containing results of literature search
#' where PubMed was queried for information of input genes.
#' Each row in the tibble contains a PubMed ID matching the
#' query, doi, title, abstract, year of publication, mesh_terms,
#' and total number of PubMed publications, resulting in a
#' total of eight columns.
#' @export
#' @examples
#' genes_query <- "ABCG2"
#' dataGLS <- GLS(genes = genes_query,
#'               query_string = "AND cancer AND driver AND
#'               '2020/01/01'[Date - Publication] : '2025/01/01'[Date - Publication]")
GLS <- function(genes,
                query_string = "AND cancer AND driver") {

  # Check user input

  if (!is(genes, "character")) {
    stop("Genes must be a character vector containing gene names to search
in PubMed")
  }

  if (!is(query_string, "character")) {
    stop("The query string must be a character vector")
  }

  # Initialize empty tibble to store results
  pubmed_mining <- tibble()

  # For each gene x in input, search PubMed based on specified
  # query

  search_pubmed <- function(x) {

    pubmed_query <- paste(x, query_string)

    # Search and retrieve results from PubMed
    gene_pubmed <- epm_query(pubmed_query)

    # Retrieve number of publications
    count_pubmed <- gene_pubmed@meta$exp_count %>%
      as.numeric()

    # If query matches any pubmed records
    if (count_pubmed > 0) {

      # Fetch data of PubMed records searched via above query
      top_results <- epm_fetch(gene_pubmed)

      # Extract information from PubMed records into a an easyPubMed object
      record_info <- epm_parse(top_results, max_authors = 1)
      # Extract information from PubMed records into a table
      record_info_data <- get_epm_data(record_info)

      # Select only PubMed id, doi, title, abstract, year, and mesh terms of
      # PubMed records
      record_info_wrangled <- record_info_data %>%
        as_tibble() %>%
        dplyr::select(c(pmid, doi, title, abstract, year, mesh_terms)) %>%
        mutate(gene = x, pubmed_count = count_pubmed) %>%
        dplyr::relocate(gene, .after = pmid)

      # Bind table to table containing results from previous gene(s)
      pubmed_mining <- pubmed_mining %>%
        bind_rows(record_info_wrangled)

      # If no records of query is found in PubMed
    } else {

      # Create tibble of one row of gene that did not have any PubMed results
      no_results_tbl <- tibble(pmid = NA, gene = x, doi = NA,
                               title = NA, abstract = NA, year = NA,
                               mesh_terms = NA, pubmed_count = count_pubmed)

      # Bind tibble of gene without PubMed information to table containing
      # results of previous gene(s)
      pubmed_mining <- pubmed_mining %>%
        bind_rows(no_results_tbl)

    }

  }

  literature_search <- map(genes, search_pubmed) %>%
    bind_rows()

  return(literature_search)

}

utils::globalVariables(c("pmid", "doi", "title", "abstract", "year",
                         "mesh_terms", "gene"))

# Test GLS function

# Run example of GLS
data(dataDMA)
genes_query <- Reduce(c, dataDMA)
dataGLS_test <- GLS(genes = genes_query,
                    query_string = "AND cancer AND driver AND '1980/01/01'[Date - Publication] : '2023/01/01'[Date - Publication]")

# Load example data of GLS serving as reference point
data(dataGLS)

# Test that output of GLS is as expected

# Test number of columns
test_that("Number of columns in GLS output are equal to 8", {
  expect_equal(ncol(dataGLS_test), 8)
})

dataGLS_colnames <- c("pmid", "gene", "doi", "title", "abstract", "year",
                      "mesh_terms", "pubmed_count")

# Test correct column names
test_that("Column names in GLS output are correct", {
  expect_equal(colnames(dataGLS_test), dataGLS_colnames)
})

# Test expected class of values in output
test_that("pmid, gene, doi, title, abstract, year, and mesh_terms are characters", {
  expect_type(dataGLS_test$pmid, "character")
  expect_type(dataGLS_test$gene, "character")
  expect_type(dataGLS_test$doi, "character")
  expect_type(dataGLS_test$title, "character")
  expect_type(dataGLS_test$abstract, "character")
  expect_type(dataGLS_test$year, "double")
})

test_that("pubmed_count is double", {
  expect_type(dataGLS_test$pubmed_count, "double")
})

# Test that output of GLS is as expected compared to reference / example data
test_that("GLS output is identical to reference point", {
  expect_equal(dataGLS_test, dataGLS)
})

#' @title Generate network
#' @description This function carries out the gene regulatory network inference using parmigene
#' @param TFs a vector of genes.
#' @param DEGsmatrix DEGsmatrix output from DEA such as dataDEGs
#' @param DiffGenes if TRUE consider only diff.expr genes in GRN
#' @param normCounts is a matrix of gene expression with genes in rows and samples in columns.
#' @param kNearest the number of nearest neighbors to consider to estimate the mutual information.
#' Must be less than the number of columns of normCounts.
#' @param nGenesPerm nGenesPerm
#' @param nBoot nBoot
#' @param noise_mi noise in knnmi.cross function. Default is 1e-12.
#' @importFrom parmigene knnmi.cross
#' @export
#' @return an adjacent matrix
#' @examples
#' data('DEGsmatrix')
#' data('dataFilt')
#' dataGRN <- GRN(TFs = sample(rownames(DEGsmatrix), 30),
#' DEGsmatrix = DEGsmatrix,
#' DiffGenes = TRUE,
#' normCounts = dataFilt,
#' nGenesPerm = 2,
#' nBoot = 2)
GRN <- function(TFs,
                DEGsmatrix,
                DiffGenes = FALSE,
                normCounts,
                kNearest = 3,
                nGenesPerm = 2000,
                nBoot = 400,
                noise_mi = 1e-12) {

  # Check user input

  if (!is(TFs, "character") | length(TFs) == 0) {
    stop("TFs must be a non-empty character vector containing gene names")
  }

  if (.row_names_info(DEGsmatrix) < 0) {
    stop("Row names were generated automatically. The input DEG table needs to
have the gene names as rownames. Double check that genes are rowname")
  }

  if (!is(DiffGenes, "logical")) {
    stop("DiffGenes must be either TRUE or FALSE")
  }

  if (is(dim(normCounts), "NULL")) {
    stop("The expression data must be non-empty with genes in rows and samples 
in columns")
  }

  if (!is(kNearest, "numeric") | !is(nGenesPerm, "numeric") | !is(nBoot, "numeric") | !is(noise_mi, "numeric")) {
    stop("kNearest, nGenesPerm, nBoot and noise_mi must be numeric values")
  }

  normCountsA <- normCounts
  normCountsB <- normCounts

  if (DiffGenes == TRUE) {
    commonGenes <- intersect(rownames(DEGsmatrix), rownames(normCountsB))
    normCountsB <- normCountsB[commonGenes,]

  } else {
    normCountsB <- normCountsA
  }

  MRcandidates <- intersect(rownames(normCountsA), TFs)

  # Mutual information between TF and genes
  sampleNames <- colnames(normCounts)
  geneNames <- rownames(normCounts)

  miTFGenes <- knnmi.cross(normCountsA[MRcandidates, ],
                           normCountsB,
                           k = kNearest,
                           noise = noise_mi)

  # Threshold with bootstrap
  tfListCancer <- TFs
  tfListCancer <- intersect(tfListCancer, rownames(normCountsA))

  maxmi <- rep(0, length(tfListCancer))

  Cancer_null_distr <- matrix(0, length(tfListCancer), nBoot)
  rownames(Cancer_null_distr) <- tfListCancer

  for (i in seq.int(nBoot)) {

    SampleS <- sample(seq.int(ncol(normCountsA)))
    g <- sample(seq.int(nrow(normCountsA)), nGenesPerm)
    mi <- knnmi.cross(normCountsA[tfListCancer, ],
                      normCountsA[g, SampleS],
                      k = kNearest,
                      noise = noise_mi)

    maxmiCurr <- apply(mi, 1, max)
    Cancer_null_distr[,i] <- maxmiCurr
    index <- maxmi < maxmiCurr
    maxmi[index] <- maxmiCurr[index]

  }

  names(maxmi) <- rownames(Cancer_null_distr)

  return(list(miTFGenes = miTFGenes, maxmi = maxmi))

}

# Test GRN function

# Run example of GRN
data('DEGsmatrix')
data('dataFilt')
dataDEGs <- DEGsmatrix
set.seed(317)
dataGRN_test <- GRN(TFs = sample(rownames(dataDEGs), 100),
                    DEGsmatrix = dataDEGs,
                    DiffGenes = TRUE,
                    normCounts = dataFilt,
                    nGenesPerm = 5,
                    nBoot = 5,
		    noise_mi = 0)

# Load example data of GRN serving as reference point
data(dataGRN_no_noise)

# Test that output of GRN is as expected

# Test that output of GRN is a list of length 2
test_that("output of GRN is a list", {
  expect_type(dataGRN_test, "list")
  expect_type(dataGRN_test$miTFGenes, "double")
  expect_type(dataGRN_test$maxmi, "double")
  expect_length(dataGRN_test, 2)
})

# Test that names of elements in GRN are correct
test_that("names of elements in GRN output are correct", {
  expect_equal(names(dataGRN_test), c("miTFGenes", "maxmi"))
})

# Test that output of GRN is as expected compared to reference / example data
if (R.version[["arch"]] == "aarch64") {
  test_that("GRN output is identical to reference point", {
    expect_equal(dataGRN_test, dataGRN_no_noise, tolerance = 0.1)
  })
} else {
  test_that("GRN output is identical to reference point", {
    expect_equal(dataGRN_test, dataGRN_no_noise)
  })
}

#' plotHeatmap
#'
#' This function creates a unclustered heatmap from the inputted data tibble
#' and saves it
#'
#' @param df a tibble
#'
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @import ComplexHeatmap
#' @import tidyHeatmap
#'
#' @return The name of the alphabeatically first gene in the tibble
#' @export
#'
plotHeatmap <- function(df) {

  # Check user input

  if (c("Moonlight_Oncogenic_Mediator") %in% colnames(Oncogenic_mediators_mutation_summary) == FALSE) {
    stop("Moonlight_Oncogenic_Mediator must be a column name in Oncogenic_mediators_mutation_summary")
  }

  first_gene <- pull(df[1, 1])

  driver_mut_heatmap <- df %>%
    group_by(Moonlight_Oncogenic_Mediator) %>%
    heatmap(.row = Mutation_type,
            .column = Hugo_Symbol,
            .value = Count,
            scale = "none",
            cluster_rows = FALSE,
            cluster_columns = FALSE,
            show_column_dend = FALSE,
            show_column_names = TRUE,
            palette_value = c("white", "blue", "darkblue"),
            column_title = paste("Heatmap Driver annotation by CScape-Somatic, from gene:",
                                first_gene,
                                "\n Hugo_Symbol")) %>%
    add_tile(Moonlight_Oncogenic_Mediator, palette = c("goldenrod2", "dodgerblue3")) %>%
    add_bar(Total_Mutations) %>%
    add_tile(logFC, palette = c("chartreuse4", "firebrick3"))

  # Save plot
  save_pdf(driver_mut_heatmap, height = 15, width = 35, units = "cm",
           filename = paste("heatmaps/heatmap_", first_gene, ".pdf", sep=""))

  return(first_gene)

}

utils::globalVariables(c("Mutation_type", "Hugo_Symbol", "Count", "Total_Mutations",
                         "logFC", "Oncogenic_mediators_mutation_summary"))

#' plotDMA
#'
#' This function creates one or more heatmaps on the output from DMA.
#' It visualises the CScape-Somatic annotations per oncogenic mediator either
#' in a single heatmap or split into several different ones.
#' It is also possible to provide a personalised genelist to visualise.
#'
#' @param DEG_Mutations_Annotations A tibble, output file from DMA.
#' @param Oncogenic_mediators_mutation_summary A tibble, output file from DMA.
#' @param type A character string. It can take the values \code{"split"} or \code{"complete"}.
#' If both type and genelist are \code{NULL}, the function will default to \code{"split"}.
#' \itemize{
#' \item \code{"split"} will split the entire dataset into sections of 40 genes and create individual plots.
#' These plots will be merged into one pdf. The genes will be sorted alphabeatically.
#' \item \code{"complete"} will create one plot, though it will not be possible to see the individual gene names.
#' The heatmap will be clustered hierarchically.
#' }
#' @param genelist A character vector containing HUGO symbols.
#' A single heatmap will be created with only these genes.
#' The heatmap will be hierarchically clustered. This will overwrite \code{type}.
#'
#' @param additionalFilename A character string. Adds prefix or filepath to the filename of the pdf.
#'
#' @import dplyr
#' @import tidyr
#' @importFrom magrittr "%>%"
#' @import ComplexHeatmap
#' @import tidyHeatmap
#' @importFrom qpdf pdf_combine
#' @importFrom purrr map
#'
#' @return No return value. DMA results are plotted.
#' @export
#'
#' @examples
#' data('DEG_Mutations_Annotations')
#' data('Oncogenic_mediators_mutation_summary')
#' plotDMA(DEG_Mutations_Annotations,
#'         Oncogenic_mediators_mutation_summary,
#'         genelist = c("ACSS2", "AFAP1L1"), additionalFilename = "myplots_")
plotDMA <- function(DEG_Mutations_Annotations,
                    Oncogenic_mediators_mutation_summary,
                    type = "split",
                    genelist = c(),
                    additionalFilename = "") {

  # Check user input

  if (all(c("Hugo_Symbol", "logFC") %in% colnames(DEG_Mutations_Annotations)) == FALSE) {
    stop("DEG_Mutations_Annotations must contain Hugo_Symbol and logFC as column names")
  }

  if (all(c("CScape_Driver", "CScape_Passenger", "CScape_Unclassified", "Moonlight_Oncogenic_Mediator") %in% colnames(Oncogenic_mediators_mutation_summary)) == FALSE) {
    stop("Oncogenic_mediators_mutation_summary must contain CScape_Driver,
CScape_Passenger, CScape_Unclassified, and Moonlight_Oncogenic_Mediator as column names")
  }

  if (!is(type, "NULL") && (type %in% c("split", "complete")) == FALSE) {
    stop("Type must either be NULL, split or complete")
  }

  if (!is(genelist, "NULL") & !is(genelist, "character")) {
    stop("Genelist must be either NULL or a character vector containing gene names")
  }

  if (!is(additionalFilename, "character")) {
    stop("additionalFilename must be a character vector adding a prefix or filepath to the filename of the pdf")
  }

  # Modify input
  DEGs <- DEG_Mutations_Annotations %>%
    dplyr::select(Hugo_Symbol, logFC) %>%
    distinct()

  Summary_wrangled <- Oncogenic_mediators_mutation_summary %>%
    pivot_longer(cols = c(CScape_Driver, CScape_Passenger, CScape_Unclassified),
                 names_to = "Mutation_type",
                 values_to = "Count") %>%
    replace_na(list(Count = 0)) %>%
    left_join(DEGs)

  if (length(genelist) > 0) {

    Summary_wrangled <- Summary_wrangled %>%
      filter(Hugo_Symbol %in% genelist) %>%
      group_by(Moonlight_Oncogenic_Mediator)

    # Check that there are both OCGs and TGS
    n <- Summary_wrangled %>%
      summarise() %>%
      count() %>%
      pull()

    if (n > 1) {

      driver_mut_heatmap <- Summary_wrangled %>%
        heatmap(.row = Mutation_type,
                .column = Hugo_Symbol,
                .value = Count,
                scale = "none",
                cluster_rows = FALSE,
                cluster_columns = TRUE,
                show_column_dend = FALSE,
                show_column_names = TRUE,
                palette_value = c("white", "blue", "darkblue"),
                column_title = paste("Heatmap Driver annotation by CScape-Somatic",
                                     "\n Hugo_Symbol")) %>%
        annotation_tile(Moonlight_Oncogenic_Mediator, palette = c("goldenrod2", "dodgerblue3")) %>%
        annotation_bar(Total_Mutations) %>%
        annotation_tile(logFC, palette = c("chartreuse4", "firebrick3"))

      # Save plot
      save_pdf(driver_mut_heatmap, height = 15, width = 35, units = "cm",
               filename = paste(additionalFilename, "heatmap_genelist.pdf",
               sep = ""))
    } else {
      stop("The genelist must contain at least one OCG and one TSG")
    }
  } else if (type == "complete") {

    driver_mut_heatmap <- Summary_wrangled %>%
      group_by(Moonlight_Oncogenic_Mediator) %>%
      heatmap(.row = Mutation_type,
              .column = Hugo_Symbol,
              .value = Count,
              scale = "none",
              cluster_rows = FALSE,
              cluster_columns = TRUE,
              show_column_dend = FALSE,
              show_column_names = TRUE,
              palette_value = c("white", "blue", "darkblue"),
              column_title = paste("Heatmap Driver annotation by CScape-Somatic",
                                   "\n Hugo_Symbol")) %>%
      annotation_tile(Moonlight_Oncogenic_Mediator, palette = c("goldenrod2", "dodgerblue3")) %>%
      annotation_bar(Total_Mutations) %>%
      annotation_tile(logFC, palette = c("chartreuse4", "firebrick3"))

    # Save plot
    save_pdf(driver_mut_heatmap, height = 15, width = 35, units = "cm",
             filename = paste(additionalFilename, "heatmap_complete.pdf", sep = ""))

  } else if (type == "split") {

    # Create temporary heatmap folder
    if (dir.exists("./heatmaps")) {
      stop("'heatmaps' folder already exits. Please remove it.")
    } else {
      print("A temporary heatmaps folder is created. It will be removed before finalising.")
      dir.create(path = "./heatmaps", showWarnings = TRUE, recursive = TRUE)
    }

    # Make vector with groups (40 genes in each plot x 3 (driver, pas, unclas) per gene) = 120
    split_vector <- rep(seq(1, ceiling(nrow(Summary_wrangled) / 120), by = 1), each = 120)
    split_vector <- split_vector[seq.int(nrow(Summary_wrangled))]

    grouped_data <- Summary_wrangled %>%
      arrange(Hugo_Symbol) %>%
      ungroup() %>%
      mutate(gr = split_vector) %>%
      group_by(gr) %>%
      tidyr::nest(data = -gr)

    print("Start creating indivual heatmaps, this can take a while")
    test<- grouped_data %>%
      ungroup %>%
      mutate(First_gene = map(grouped_data$data, ~plotHeatmap(.))) %>%
      mutate(heatmaps = "heatmaps/heatmap_", pdf = ".pdf") %>%
      unite(heatmaps, c(heatmaps, First_gene, pdf), sep = "")
    print("Indivial heatmaps finished, start combining")

    # Stable pdf into one pdf
    pdfs <- test %>%
      pull(heatmaps)
    pdf_combine(input = pdfs, output = paste(additionalFilename, "heatmaps_split.pdf", sep = ""))

    # Remove heatmap-folder with redudant pdfs
    unlink("heatmaps", recursive = TRUE)

  }
}

utils::globalVariables(c("Hugo_Symbol", "logFC", "CScape_Driver", "CScape_Passenger",
                         "CScape_Unclassified", "Moonlight_Oncogenic_Mediator",
                         "Mutation_type", "Count", "Total_Mutations", "gr",
                         "heatmaps", "First_gene"))

#' FEA
#'
#' This function carries out the functional enrichment analysis (FEA).
#' The analysis can be performed using as input a data matrix from Differential Expression Analysis of genes or Differential Abundance Analysis of proteins.
#' The analysis can be performed using the ORA (Over Representation Analysis)-based method which implements Fisher's Exact Test or the pre-ranked FCS(Functional Class Scoring)-based method
#' which implements fgsea (Fast Gene Set Enrichment Analysis).
#' @param BPname BPname biological process such as "proliferation of cells", "ALL" (default) if FEA should be carried out for all 101 biological processes
#' @param DiffMatrix output from DEA such as dataDEGs or from DAA such as differential abundace matrix
#' @param method method to perform Functional Enirchment analysis. Must be either "ora" (Over-Representation Analysis) or "fgsea" (Fast Gene Set Enrichment Analysis)
#' @param seed internal seed before the 'fgsea' function is called to ensure reproducibility of the permutation-based p-values. Default is NULL
#' @importFrom stats p.adjust
#' @importFrom fgsea fgsea
#' @return matrix from FEA
#' @export
#' @examples
#' data(DEGsmatrix)
#' data(DiseaseList)
#' data(EAGenes)
#' DEGsmatrix <- DEGsmatrix[seq.int(2), ]
#' dataFEA <- FEA(DiffMatrix = DEGsmatrix, BPname = "apoptosis")
FEA <- function(BPname = NULL,
                DiffMatrix,
                method = "ora",
                seed = NULL) {

  # List of variable names
  variables_to_check <- c("DiseaseList", "EAGenes")

  # Check and load variables if they do not exist
  for (variable_name in variables_to_check) {
    if (! variable_name %in% names(.GlobalEnv)) {
      data(list=c(variable_name))
    }
  }

  DiseaseList <- get("DiseaseList")
  EAGenes <- get("EAGenes")

  # Check user input

  if ("logFC" %in% colnames(DiffMatrix) == FALSE) {
    stop("The input data table must contain a column called logFC.")
  }

  if (.row_names_info(DiffMatrix) < 0) {
    stop("Row names were generated automatically. The input data table needs to
    have the gene names as rownames. Double check that genes are rownames.")
  }

  if (!is(BPname, "NULL") && all(BPname %in% names(DiseaseList)) == FALSE) {
    stop("BPname should be NULL or a character vector containing one or more
    BP(s) among possible BPs stored in the DiseaseList object.")
  }
  if (!method %in% c("ora", "fgsea")) {
    stop("Invalid method name. Must be either ora or fgsea")

  }

  if (is(BPname, "NULL")) {
    lf2 <- names(DiseaseList)
  } else {
    lf2 <- BPname
  }
  TableDiseasesNew <- NULL

  pb <- txtProgressBar(min = 0, max = length(DiseaseList), style = 3)
 
   
  if (method == "fgsea") {

    pvals <- pmax(DiffMatrix$PVal, .Machine$double.xmin)
    rankings <- sign(DiffMatrix$logFC)*(-log10(pvals))
    names(rankings)  <- rownames(DiffMatrix)
    # sorting strategy to avoid different sorting in case of tie ranks
    rankings <- rankings[order(-rankings, names(rankings))]
    
    
    pathwayNamesList <- list()
    for (k in seq_along(lf2)) {

      selected_diseases <- as.data.frame(DiseaseList[[which(names(DiseaseList) == lf2[k])]])
      name <- lf2[k]
      values <- selected_diseases$ID
      pathwayNamesList[[name]] <- values
    }

    pathway_sizes <- sapply(DiseaseList, function(pathway) length(unique(pathway$ID)))
    max_pathway_size <- max(pathway_sizes)
    
    if (!is(seed, "NULL")){
      set.seed(seed)
    }
    
    fgseaRes <- fgsea(pathways = pathwayNamesList, stats = rankings, scoreType = 'std', minSize = 1, maxSize = max_pathway_size, nproc = 1)
    
    bp_score_collection <- list()
    for (k in seq_along(lf2)) {
      
      setTxtProgressBar(pb, k)

      res <- as.data.frame(matrix(0, nrow = 1, ncol = 2,
                                  dimnames = list(1, c("pathway",
                                                     "Moonlight.Z.score"))))
      
      selected_diseases <- as.data.frame(DiseaseList[[which(names(DiseaseList) == lf2[k])]])
      
      res$pathway <- lf2[k]
      
      Zscore <- .compute_moonlight_zscore(selected_diseases)
      
      res$Moonlight.Z.score <- Zscore
      
      bp_score_collection[[k]] <- res
    
    }

    close(pb)

    bp_score_collection_merged <- bind_rows(bp_score_collection)

    TableDiseasesNew <- fgseaRes %>% left_join(bp_score_collection_merged, by = "pathway")
    TableDiseasesNew <- TableDiseasesNew %>% select(-log2err)
    TableDiseasesNew <- TableDiseasesNew %>% 
      rename("Diseases.or.Functions.Annotation" = pathway, "p.value" = pval) %>% 
      select("Diseases.or.Functions.Annotation", "Moonlight.Z.score","p.value", "padj", "ES", "NES", "size", "leadingEdge")

  
  }
  else if (method == "ora") {

    for (k in seq.int(lf2)) {

      setTxtProgressBar(pb, k)

      res <- as.data.frame(matrix(0, nrow = 1, ncol = 7,
                                  dimnames = list(1, c("Diseases.or.Functions.Annotation",
                                                      "p.Value", "Moonlight.Z.score",
                                                      "commonNg",
                                                      "FunctionNg",
                                                      "Delta",
                                                      "Molecules"))))

      GeneList <- data.frame(PROBE_ID = rownames(DiffMatrix),
                            logFC = DiffMatrix$logFC,
                            stringsAsFactors = FALSE)


      rownames(GeneList) <- GeneList$PROBE_ID

      selected_diseases <- as.data.frame(DiseaseList[[which(names(DiseaseList) == lf2[k])]])

      selected_diseases$ID <- selected_diseases$Genes.in.dataset

      res$commonNg <- length(intersect(GeneList$PROBE_ID, selected_diseases$ID))

      res$FunctionNg <- nrow(selected_diseases)

      res$Diseases.or.Functions.Annotation <- lf2[k]

      allgene <- unique(c(as.character(unique(EAGenes[, "ID"])),
                        GeneList$PROBE_ID,
                        selected_diseases$ID))

      seta <- allgene %in% GeneList$PROBE_ID
      setb <- allgene %in% selected_diseases$ID

      if (res$commonNg > 1) {
        ft <- fisher.test(seta, setb)
        FisherpvalueTF <- ft$p.value
        res$p.Value <- FisherpvalueTF
      } else {
        res$p.Value <- 1
      }

      GeneList <- GeneList[GeneList$PROBE_ID %in% selected_diseases$ID, ]

      selected_diseases <- selected_diseases[selected_diseases$ID %in% GeneList[, "PROBE_ID"], ]
      selected_diseases[, "Exp.Log.Ratio"] <- gsub(",", ".", selected_diseases[, "Exp.Log.Ratio"])
      selected_diseases[, "Exp.Log.Ratio"] <- as.numeric(selected_diseases[, "Exp.Log.Ratio"])

      rownames(selected_diseases) <- selected_diseases$ID

      res$Molecules <- paste0(GeneList$PROBE_ID, collapse = ",")

      for (idx in seq.int(nrow(selected_diseases))) {

        currTR <- selected_diseases$ID[idx]

        if (length(grep("Increases", selected_diseases[currTR, "Findings"])) == 1) {

          if (sign(GeneList[currTR, "logFC"]) > 0) {
            selected_diseases[currTR, "Prediction..based.on.expression.direction."] <- "Increased"
          } else if (sign(GeneList[currTR, "logFC"]) < 0) {
            selected_diseases[currTR, "Prediction..based.on.expression.direction."] <- "Decreased"
          }
        }

        if (length(grep("Decreases", selected_diseases[currTR, "Findings"])) == 1) {
          if (sign(GeneList[currTR, "logFC"]) < 0) {
            selected_diseases[currTR, "Prediction..based.on.expression.direction."] <- "Increased"
          } else if (sign(GeneList[currTR, "logFC"]) > 0) {
            selected_diseases[currTR, "Prediction..based.on.expression.direction."] <- "Decreased"
          }
        }
      }

      Zscore <- .compute_moonlight_zscore(selected_diseases)

      res$Moonlight.Z.score <- Zscore
      TableDiseasesNew <- rbind(TableDiseasesNew, res)
    }

    close(pb)

    TableDiseasesNew <- cbind(TableDiseasesNew, FDR = p.adjust(TableDiseasesNew$p.Value,
                                                             method = "fdr"))
    TableDiseasesNew <- subset(TableDiseasesNew, select = c("Diseases.or.Functions.Annotation",
                                                          "Moonlight.Z.score",
                                                          "p.Value",
                                                          "FDR",
                                                          "commonNg",
                                                          "FunctionNg",
                                                          "Molecules"))
    
  }

  return(TableDiseasesNew)
}

#' compute_moonlight_zscore
#' 
#' This internal function compute the Moonlight Z-score for each enriched pathway 
#' extracted from the Functional Enrichment Analysis
#' @keywords internal
#' @param selected_diseases A dataframe containing pathway's genes expression direction and changes 
#' @return Moonlight Z-score A numeric score that measures how well the direction of change agrees between observed expression changes and literature findings
#' @noRd
.compute_moonlight_zscore <- function(selected_diseases) {
  
  Correlation <- matrix(0, nrow(selected_diseases), 1)

  selected_diseases <- cbind(selected_diseases, Correlation)

  if (length(grep("Decreases", selected_diseases$Findings)) != 0) {

        selected_diseases[grep("Decreases", selected_diseases$Findings), "Findings"] <- -1
        selected_diseases[grep("Increases", selected_diseases$Findings), "Findings"] <- 1
        selected_diseases[grep("Affects", selected_diseases$Findings), "Findings"] <- 0
        selected_diseases[, "Findings"] <- as.numeric(selected_diseases[, "Findings"])

        selected_diseases[, "Exp.Log.Ratio"] <- gsub(",", ".", selected_diseases[, "Exp.Log.Ratio"])
        selected_diseases[, "Exp.Log.Ratio"] <- as.numeric(selected_diseases[, "Exp.Log.Ratio"])
        
        PredictionIncreased <- which(sign(selected_diseases$Exp.Log.Ratio) == selected_diseases$Findings)
        PredictionDecreased <- which(sign(selected_diseases$Exp.Log.Ratio) != selected_diseases$Findings)
        PredictionAffected <- which(sign(selected_diseases$Findings) == 0)

        selected_diseases[PredictionIncreased, "Correlation"] <- 1
        selected_diseases[PredictionDecreased, "Correlation"] <- -1
        selected_diseases[PredictionAffected, "Correlation"] <- 0

        Zscore <- sum(selected_diseases$Correlation) / sqrt( length(PredictionIncreased) + length(PredictionDecreased))
      } else {
        Zscore <- 0
      }

      return(Zscore)

}

# Test FEA function

# Run example of FEA
data("DEGsmatrix")
data("DiseaseList")
data("EAGenes")
DEGsmatrix_ora <- DEGsmatrix[1:10, ]
dataFEA_test_ora <- FEA(DiffMatrix = DEGsmatrix_ora, method = "ora")
dataFEA_test_fgsea <- FEA(DiffMatrix = DEGsmatrix, method = "fgsea", seed = 123)
# Load example data of FEA serving as reference points
data(dataFEA)
data(dataFEA_fgsea)

# Test that output of FEA using ORA-based method is as expected 

# Test number of columns
test_that("Number of columns in FEA output are equal to 7", {
  expect_equal(dim(dataFEA_test_ora), c(101, 7))
})

dataFEA_colnames <- c("Diseases.or.Functions.Annotation", "Moonlight.Z.score",
                      "p.Value", "FDR", "commonNg", "FunctionNg", "Molecules")

# Test correct column names
test_that("Column names in FEA output are correct", {
  expect_equal(colnames(dataFEA_test_ora), dataFEA_colnames)
})

# Test expected class of values in output
test_that("Moonlight scores, p-values and FDR values are numeric", {
  expect_type(dataFEA_test_ora$Diseases.or.Functions.Annotation, "character")
  expect_type(dataFEA_test_ora$Moonlight.Z.score, "double")
  expect_type(dataFEA_test_ora$p.Value, "double")
  expect_type(dataFEA_test_ora$FDR, "double")
  expect_type(dataFEA_test_ora$commonNg, "integer")
  expect_type(dataFEA_test_ora$FunctionNg, "integer")
  expect_type(dataFEA_test_ora$Molecules, "character")
})

# Test that output of FEA is as expected compared to reference 
test_that("FEA output is identical to reference point", {
  expect_equal(dataFEA_test_ora, dataFEA)
})


# Test that output of FEA using fgsea method is as expected

# Test number of columns
test_that("Number of columns in FEA output are equal to 8", {
  expect_equal(dim(dataFEA_test_fgsea), c(101, 8))
})

dataFEA_colnames <- c("Diseases.or.Functions.Annotation","Moonlight.Z.score", "p.value",
                      "padj", "ES", "NES", "size", "leadingEdge")

# Test correct column names
test_that("Column names in FEA output are correct", {
  expect_equal(colnames(dataFEA_test_fgsea), dataFEA_colnames)
})

# Test expected class of values in output
test_that("Moonlight scores, p-values and FDR values are numeric", {
  expect_type(dataFEA_test_fgsea$Diseases.or.Functions.Annotation, "character")
  expect_type(dataFEA_test_fgsea$p.value, "double")
  expect_type(dataFEA_test_fgsea$padj, "double")
  expect_type(dataFEA_test_fgsea$ES, "double")
  expect_type(dataFEA_test_fgsea$NES, "double")
  expect_type(dataFEA_test_fgsea$size, "integer")
  expect_type(dataFEA_test_fgsea$leadingEdge, "list")
  expect_true(all(sapply(dataFEA_test_fgsea$leadingEdge, is.character)))

})

# Test that output of FEA is as expected compared to reference 
test_that("FEA output is similar enough to reference point", {
  expect_equal(dataFEA_fgsea$NES, dataFEA_test_fgsea$NES, tolerance = 1e-6)
  expect_equal(dataFEA_fgsea$ES, dataFEA_test_fgsea$ES, tolerance = 1e-6)
})

# Check using tolerance of half order of magnitude
test_that("FEA p-values are qualitatively similar", {
expect_equal(
  -log10(pmax(dataFEA_test_fgsea$p.value, .Machine$double.xmin)),
  -log10(pmax(dataFEA_fgsea$p.value, .Machine$double.xmin)),
  tolerance = 0.5
)})

#' loadMAVISp
#' 
#' This function loads the MAVISp database from the directory specified by the user. 
#' 
#' @param mavispDB path to the MAVISp database as a string. The database can be downloaded from an OSF repository (https://osf.io/ufpzm/) and has the following structure.
#' @param proteins_of_interest vector containing specific proteins of interest in HUGO format
#' @param mode string determining whether to use simple or ensemble mode of mavisp. Default is simple mode.
#' Takes values: 
#' \itemize{
#' \item simple
#' \item ensemble
#'} 
#' @param ensemble string specifying the ensemble to use. The available ensembles  can be found in the MAVISp database. This is ignored for simple mode.
#'
#' @importFrom stringr str_c str_split_i
#' @importFrom purrr map
#' @importFrom readr read_csv
#' @importFrom tibble as_tibble_col
#' @importFrom dplyr pull filter
#' @importFrom rlang set_names
#' @importFrom withr with_options
#' @return returns a list of tibbles each containing the MAVISp entry of one protein. Each entry will contain an extra column specifying what data
#' the stability classification is based on.
#' @export
#' @examples
#' 
#' mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')
#'
#' mavisp_data <- loadMAVISp(mavispDB = mavisp_db_location,
#'                           proteins_of_interest = c('TP53'),
#'                           mode = 'ensemble')
#' 

loadMAVISp <- function(mavispDB = NULL,
                       proteins_of_interest = NULL,
                       mode = 'simple',
                       ensemble = 'md'){
    # Look in simple mode index.csv if the protein is in the database
    if (file.exists(str_c(mavispDB,'/dataset_info.csv')) == FALSE){
        stop('MAVISp database file not found at the provided path, or the database_info.csv file is missing.')
    } else if (mode == 'simple'){
        table_location <- str_c(mavispDB,'/simple_mode/dataset_tables/')
    } else if (mode == 'ensemble'){
        if (length(ensemble) > 1){
            stop('Only one ensemble can be specified for ensemble mode.')
        }
        table_location <- str_c(mavispDB,'/ensemble_mode/dataset_tables/')
    } else {
        stop('Mode not specified correctly. Accepts strings "simple" or "ensemble"')
    }

    # Load data for proteins of interest or all proteins
    if (is.null(proteins_of_interest)){
        rawFiles <- list.files(table_location,
                               full.names = TRUE)

    } else {
        proteins_of_interest <- str_c(proteins_of_interest,'-')
        rawFiles <- list.files(table_location,
                               full.names = TRUE) |>
                               as_tibble_col(column_name = 'filepath') |>
                    filter(grepl(paste(proteins_of_interest, 
                                        collapse = '|'),
                                        filepath)) |>
                    pull(filepath)
    }

    # Double check there are files
    if (length(rawFiles) == 0){
        stop('No MAVISp files matching the criteria were found.')
    }

    mavispData <- rawFiles |>
        set_names(str_split_i(basename(rawFiles), '-', 1)) |>
        # Supress non-fatal warnings
        map(function(x) withr::with_options(
                        list(rlib_name_repair_verbosity = 'quiet'),
                        suppressWarnings(
                        classes = 'vroom_parse_issue',
                        read_csv(file = x,
                                progress = FALSE,
                                show_col_types = FALSE))))
    
    # Combine stability results based on user specification
    if (mode == 'ensemble'){
        mavispData <- mavispData |>
            map(function(x) rename(x, 'Stability classification, (Rosetta, FoldX)' = matches(paste0('Stability classification, [A-Za-z0-9, ]*\\(Rosetta, FoldX\\)( \\[',ensemble,'\\])?')),
                                    'Stability classification, (RaSP, FoldX)' = matches(paste0('Stability classification, [A-Za-z0-9, ]*\\(RaSP, FoldX\\)( \\[',ensemble,'\\])?'))) |>
                            mutate(stab_class_ros_source = paste0('ensemble_mode_',ensemble),
                                   stab_class_rasp_source = paste0('ensemble_mode_',ensemble)))
    } else if (mode == 'simple'){
        mavispData <- mavispData |>
            map(function(x) mutate(x, stab_class_data_type = 'simple_mode'))
    }


    return(mavispData)
}

# Test loadMAVISp function

# reference datasets

specific_protein_ref <- as_tibble(data.frame(
  Mutation = c("R49C", "R49H", "R49P", "M51I", "M51L"),
  "Stability (FoldX5, alphafold, kcal/mol)" = c(0.49477, 0.60357, 1.84988, 0.64349, 0.01443),
  "Stability (Rosetta Cartddg2020, alphafold, kcal/mol)" = c(0.32483, 0.35782, 0.03844, 0.26667, 0.36531),
  "Stability (RaSP, alphafold, kcal/mol)" = c(6.24453672193644, 6.22116346712404, 6.24439788955118, 4.34939570830432, 4.19197330424889),
  "Stability classification, alphafold, (Rosetta, FoldX)" = c("Neutral", "Neutral", "Neutral", "Neutral", "Neutral"),
  "Stability classification, alphafold, (RaSP, FoldX)" = c("Uncertain", "Uncertain", "Uncertain", "Uncertain", "Uncertain"),
  "Local Int. (Binding with CBFB_AFmulti, heterodimer, FoldX5, kcal/mol)" = c(0.12688, -0.13046, -0.28584, 0.04618, 0.1536),
  "Local Int. (Binding with CBFB_AFmulti, heterodimer, Rosetta Talaris 2014, kcal/mol)" = c(-0.02707, 0.11159, -0.02728, -0.09484, -0.00663),
  "Local Int. classification (CBFB_AFmulti)" = c("Neutral", "Neutral", "Neutral", "Neutral", "Neutral"),
  "Relative Side Chain Solvent Accessibility in wild-type" = c(58.2, 58.2, 58.2, 21.8, 21.8),
  "gnomAD genome allele frequency" = as.numeric(c(NA, NA, NA, NA, NA)),
  "gnomAD exome allele frequency" = as.numeric(c(NA, NA, NA, NA, NA)),
  "REVEL score" = as.numeric(c(NA, NA, NA, NA, NA)),
  "Mutation sources" = c("mutations_clinvar", "mutations_clinvar", "mutations_clinvar", "mutations_clinvar", "mutations_clinvar"),
  PTMs = as.character(c(NA, NA, NA, NA, NA)),
  "is site part of phospho-SLiM" = c(FALSE, FALSE, FALSE, TRUE, TRUE),
  "PTM residue SASA (%)" = c(58.2, 58.2, 58.2, 21.8, 21.8),
  "Change in stability with PTM (FoldX5, kcal/mol)" = as.numeric(c(NA, NA, NA, NA, NA)),
  "Change in binding with PTM (FoldX5, kcal/mol)" = as.logical(c(NA, NA, NA, NA, NA)),
  "Change in binding with mutation (FoldX5, kcal/mol)" = as.logical(c(NA, NA, NA, NA, NA)),
  "PTM effect in regulation" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTM effect in stability" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTM effect in function" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "AlloSigma2 mutation type" = c("DOWN", "DOWN", "DOWN", "-", "-"),
  "AlloSigma2 predicted consequence - active sites" = c("neutral", "neutral", "neutral", "uncertain", "uncertain"),
  "AlloSigma2 predicted consequence - cofactor sites" = c("neutral", "neutral", "neutral", "uncertain", "uncertain"),
  "AlloSigma2 predicted consequence - pockets and interfaces" = c("stabilizing", "stabilizing", "stabilizing", "uncertain", "uncertain"),
  "ClinVar Variation ID" = as.character(c(576717, 1421576, 2422003, 1061802, 239046)),
  "ClinVar Interpretation" = c("Uncertain significance", "Uncertain significance", "Uncertain significance", "Uncertain significance", "Uncertain significance"),
  "ClinVar Review Status" = as.character(c(1, 1, 1, 1, 1)),
  "AlphaFold2 model pLDDT score" = c(73.63, 73.63, 73.63, 83.75, 83.75),
  "AlphaFold2 model secondary structure" = c("-", "-", "-", "H", "H"),
  "DeMaSk delta fitness" = c(-0.3905, -0.2693, -0.4559, -0.1432, -0.0892),
  "DeMaSk Shannon entropy" = c(0.2854, 0.2854, 0.2854, 1.0975, 1.0975),
  "DeMaSk log2 variant frequency" = c(-21.8538, -5.7587, -8.5659, -7.5924, -3.4632),
  "DeMaSk predicted consequence" = c("loss_of_function", "loss_of_function", "loss_of_function", "loss_of_function", "loss_of_function"),
  "GEMME Score" = c(-4.59051627158367, -2.0166488770451, -5.21762245227481, -0.430978418892934, -1.08466102827133),
  "EVE score" = c(2.56798164871032e-05, 1.09938059686617e-05, 0.0053544417229454, 2.01767420794855e-09, 2.28711094549718e-14),
  "EVE classification (25% Uncertain)" = c("Benign", "Benign", "Benign", "Benign", "Benign"),
  "AlphaMissense pathogenicity score" = c(0.5407, 0.239, 0.7822, 0.6019, 0.2257),
  "AlphaMissense classification" = c("ambiguous", "benign", "pathogenic", "pathogenic", "benign"),
  References = c("https://doi.org/10.1101/2022.10.22.513328", "https://doi.org/10.1101/2022.10.22.513328", "https://doi.org/10.1101/2022.10.22.513328", "https://doi.org/10.1101/2022.10.22.513328", "https://doi.org/10.1101/2022.10.22.513328"),
  stab_class_data_type = c("simple_mode", "simple_mode", "simple_mode", "simple_mode", "simple_mode"),
  stringsAsFactors = FALSE, check.names=FALSE
), .name_repair=function(x) {return(x)})

ensemble_ref <- as_tibble(data <- data.frame(
  "Mutation" = c("W91C", "W91R", "W91S", "P92A", "P92H"),
  "Stability (FoldX5, kcal/mol) [md]" = c(1.09645, 0.95517, 1.32043, 1.18621, 1.46241),
  "Stability (Rosetta Cartddg2020, kcal/mol) [md]" = c(1.36905, 1.37777666666667, 1.11122333333333, 0.227326666666667, 0.140363333333333),
  "Stability (RaSP, kcal/mol) [md]" = c(0.948174598045667, 0.270774913774021, 0.298933573850963, 0.233867585586445, -0.028912863696222),
  "Stability classification, (Rosetta, FoldX)1" = c("Neutral", "Neutral", "Neutral", "Neutral", "Neutral"),
  "Stability classification, (RaSP, FoldX)1" = c("Neutral", "Neutral", "Neutral", "Neutral", "Neutral"),
  "Stability (FoldX5, kcal/mol) [cabsflex]" = c(-0.00274, -0.12478, 0.01862, 1.39309, 3.21532),
  "Stability (Rosetta Cartddg2020, kcal/mol) [cabsflex]" = c(1.76043333333333, 1.43288, 1.66825666666667, 0.268706666666667, 1.03208666666667),
  "Stability (RaSP, kcal/mol) [cabsflex]" = c(1.0890734956992, 0.308181035152004, 0.509681924125949, 0.426519134963358, -0.0911043495475189),
  "Stability classification, (Rosetta, FoldX)2" = c("Neutral", "Neutral", "Neutral", "Neutral", "Uncertain"),
  "Stability classification, (RaSP, FoldX)2" = c("Neutral", "Neutral", "Neutral", "Neutral", "Uncertain"),
  "Relative Side Chain Solvent Accessibility in wild-type (average) [md]" = c(70.277, 70.277, 70.277, 50.692, 50.692),
  "Relative Side Chain Solvent Accessibility in wild-type (standard deviation) [md]" = c(22.26, 22.26, 22.26, 19.319, 19.319),
  "Relative Side Chain Solvent Accessibility in wild-type (average) [cabsflex]" = c(66.47, 66.47, 66.47, 37.285, 37.285),
  "Relative Side Chain Solvent Accessibility in wild-type (standard deviation) [cabsflex]" = c(25.088, 25.088, 25.088, 20.61, 20.61),
  "gnomAD genome allele frequency" = as.numeric(c(NA, NA, NA, NA, NA)),
  "gnomAD exome allele frequency" = as.numeric(c(NA, NA, NA, NA, NA)),
  "REVEL score" = c(NA, NA, "0.485", "0.509", NA),
  "Mutation sources" = c("somatic_muts", "clinvar", "cBioPortal", "COSMIC,somatic_muts", "COSMIC"),
  "PTMs [md]" = as.character(c(NA, NA, NA, NA, NA)),
  "is site part of phospho-SLiM [md]" = c(TRUE, TRUE, TRUE, TRUE, TRUE),
  "PTM residue SASA (%) [md]" = c(70.277, 70.277, 70.277, 50.692, 50.692),
  "Change in stability with PTM (FoldX5, kcal/mol) [md]" = as.numeric(c(NA, NA, NA, NA, NA)),
  "Change in binding with PTM (FoldX5, kcal/mol) [md]" = c(NA, NA, NA, NA, NA),
  "Change in binding with mutation (FoldX5, kcal/mol) [md]" = c(NA, NA, NA, NA, NA),
  "PTM effect in regulation [md]" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTM effect in stability [md]" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTM effect in function [md]" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTMs [cabsflex]" = as.character(c(NA, NA, NA, NA, NA)),
  "is site part of phospho-SLiM [cabsflex]" = c(TRUE, TRUE, TRUE, TRUE, TRUE),
  "PTM residue SASA (%) [cabsflex]" = c(66.47, 66.47, 66.47, 37.285, 37.285),
  "Change in stability with PTM (FoldX5, kcal/mol) [cabsflex]" = as.numeric(c(NA, NA, NA, NA, NA)),
  "Change in binding with PTM (FoldX5, kcal/mol) [cabsflex]" = c(NA, NA, NA, NA, NA),
  "Change in binding with mutation (FoldX5, kcal/mol) [cabsflex]" = c(NA, NA, NA, NA, NA),
  "PTM effect in regulation [cabsflex]" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTM effect in stability [cabsflex]" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "PTM effect in function [cabsflex]" = c("neutral", "neutral", "neutral", "neutral", "neutral"),
  "ClinVar Variation ID" = c(NA, "845798", NA, NA, NA),
  "ClinVar Interpretation" = c(NA, "Uncertain significance", NA, NA, NA),
  "ClinVar Review Status" = c(NA, "1", NA, NA, NA),
  "DeMaSk delta fitness" = c(-0.2748, -0.3228, -0.248, -0.1124, -0.2307),
  "DeMaSk Shannon entropy" = c(2.4818, 2.4818, 2.4818, 1.3832, 1.3832),
  "DeMaSk log2 variant frequency" = c(-20.417, -20.417, -2.9594, -4.9818, -20.5915),
  "DeMaSk predicted consequence" = c("loss_of_function", "loss_of_function", "loss_of_function", "loss_of_function", "loss_of_function"),
  "GEMME Score" = c(-0.527962776148948, -3.32436611299892, -1.25226234139329, -1.1402449252468, -3.34022868100625),
  "AlphaMissense pathogenicity score" = c(0.7778, 0.9346, 0.447, 0.1419, 0.3065),
  "AlphaMissense classification" = c("pathogenic", "pathogenic", "ambiguous", "benign", "benign"),
  "References" = c("10.1101/2022.10.22.513328", "10.1101/2022.10.22.513328; 37085483", "10.1101/2022.10.22.513328; 37085483", "10.1101/2022.10.22.513328; 37085483", "10.1101/2022.10.22.513328; 37085483"),
  "stab_class_ros_source" = c("ensemble_mode_cabsflex", "ensemble_mode_cabsflex", "ensemble_mode_cabsflex", "ensemble_mode_cabsflex", "ensemble_mode_cabsflex"),
  "stab_class_rasp_source" = c("ensemble_mode_cabsflex", "ensemble_mode_cabsflex", "ensemble_mode_cabsflex", "ensemble_mode_cabsflex", "ensemble_mode_cabsflex"),
  stringsAsFactors = FALSE, check.names=FALSE
), .name_repair=function(x) {return(x)})

# gathering test data
mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')

specific_protein <- loadMAVISp(mavispDB = mavisp_db_location,
                               mode = 'simple',
                               proteins_of_interest = c('RUNX1'))

all_proteins <- loadMAVISp(mavispDB = mavisp_db_location,
                           mode = 'simple')

ensemble <- loadMAVISp(mavispDB = mavisp_db_location,
                       mode = 'ensemble',
                       ensemble = 'cabsflex')

# Test that output is as expected
test_that('Output of loadMAVISp simple mode with one protein specified is a list of one tibble', {
    expect_type(specific_protein, 'list')
    expect_length(specific_protein, 1)
    expect_true(all(sapply(specific_protein, is_tibble)))
})

test_that('Output of loadMAVISp on RUNX1 (single protein) is as expected', {
    expect_equal(specific_protein$RUNX1[1:5, ],
                 specific_protein_ref,
                 ignore_attr=TRUE)
})

test_that('Output of loadMAVISp simple mode is a list of tibbles', {
    expect_type(specific_protein, 'list')
    expect_length(all_proteins, 2)
    expect_true(all(sapply(specific_protein, is_tibble)))
})

test_that('Output of loadMAVISP on RUNX1 (all proteins) is as expected', {
    expect_equal(all_proteins$RUNX1[1:5, ],
                 specific_protein_ref,
                 ignore_attr=TRUE)
})

test_that('Output of loadMAVISp ensemble mode is a list of tibbles', {
    expect_type(ensemble, 'list')
    expect_true(all(sapply(ensemble, is_tibble)))
})

test_that('Output of loadMAVISP on TP53 (ensemble mode) is as expected', {
    expect_equal(ensemble$TP53[1:5, ],
                 ensemble_ref,
                 ignore_attr=TRUE)
})

#' getDataGEO
#'
#' This function retrieves and prepares GEO data
#' @param GEOobject GEOobject
#' @param platform platform
#' @param TCGAtumor tumor name
#' @importFrom GEOquery getGEO
#' @importMethodsFrom Biobase fvarLabels fvarLabels<-
#' @export
#' @return return GEO gset
#' @examples
#' data(GEO_TCGAtab)
#' dataGEO <-  getDataGEO(GEOobject = "GSE15641", platform = "GPL96")
getDataGEO <- function(GEOobject = "GSE39004",
                       platform = "GPL6244",
                       TCGAtumor = NULL) {

  if (! "GEO_TCGAtab" %in% names(.GlobalEnv)) {
    data(list=c("GEO_TCGAtab"))
  }

  GEO_TCGAtab <- get("GEO_TCGAtab")

  # Check user input

  if (!is(GEOobject, "character")) {
    stop("GEOobject must be a character vector")
  }

  if (!is(platform, "character")) {
    stop("Platform must be a character vector")
  }

  if (!is(TCGAtumor, "NULL") & !is(TCGAtumor, "character")) {
    stop("TCGA tumor must either be NULL or a character vector")
  }

  GEO_TCGAtab <- get("GEO_TCGAtab")

  if (length(TCGAtumor) != 0) {
    GEOobject <- GEO_TCGAtab[GEO_TCGAtab$Cancer ==  TCGAtumor,"Dataset"]
    platform <- GEO_TCGAtab[GEO_TCGAtab$Cancer ==  TCGAtumor,"Platform"]
  }

  gset <- getGEO(GEOobject, GSEMatrix = TRUE, AnnotGPL = TRUE)

  if (length(gset) > 1) {
    idx <- grep(platform, attr(gset, "names"))
  } else {
    idx <- 1
  }

  gset <- gset[[idx]]

  fvarLabels(gset) <- make.names(fvarLabels(gset))

  return(gset)

}

#' @title plotNetworkHive: Hive network plot
#' @description
#' This function visualizes the GRN as a hive plot
#' @param dataGRN output GRN function
#' @param namesGenes list TSG and OCG to define axes
#' @param thres threshold of edges to be included
#' @param additionalFilename additionalFilename
#' @import HiveR
#' @importFrom graphics plot.new
#' @importFrom grid gpar
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @export
#' @return no results Hive plot is executed
#' @examples
#' data(knownDriverGenes)
#' data(dataGRN)
#' plotNetworkHive(dataGRN = dataGRN, namesGenes = knownDriverGenes, thres = 0.55)
plotNetworkHive <- function(dataGRN,
                            namesGenes,
                            thres,
                            additionalFilename = NULL) {

  # Check user input

  if (!is(namesGenes, "list")) {
    stop("namesGenes must be a list containing genes")
  }

  if (!is(thres, "numeric")) {
    stop("thres must be numeric defining threshold of edges to be included")
  }

  if (!is(additionalFilename, "NULL") & !is(additionalFilename, "character")) {
    stop("additionalFilename must be either NULL or a character vector
containing part of the filename of plot")
  }

  if (!is(dataGRN, "list")) {
    stop("dataGRN must be a list")
  }

  names.genes.all <- intersect(as.character(unique(c(unlist(namesGenes),
                                                     rownames(dataGRN[[1]])))),
                               colnames(dataGRN[[1]]))

  tmp <- dataGRN[[1]][, (names.genes.all)]
  tmp[tmp < thres] <- 0

  genes.missing <- setdiff(names.genes.all, rownames(dataGRN[[1]]))
  tmp <- rbind(tmp, matrix(0,
                           nrow = length(genes.missing),
                           ncol = length(names.genes.all)))
  rownames(tmp) <- c(rownames(dataGRN[[1]]), genes.missing)

  tmp <- tmp[names.genes.all, names.genes.all]
  diag(tmp) <- 0

  myadj <- adj2HPD(M = tmp, axis.cols = "lightgray")
  myadj$nodes$axis <- as.integer(rep(1, length(names.genes.all)) +
                                   c(2 * as.numeric(names.genes.all %in% namesGenes$TSG) +
                                     as.numeric(names.genes.all %in% namesGenes$OCG)))

  n.axis <- table(myadj$nodes$axis)
  names(n.axis) <- paste0("a.", names(n.axis))

  mycols <- c("darkgrey", "darkgreen", "goldenrod")

  myadj$nodes$color <- mycols[myadj$nodes$axis]
  myadj$nodes$size <- 0.1

  for (i in seq.int(nrow(myadj$nodes))) {
    myadj$nodes$radius[i] <- n.axis[paste0("a.", myadj$nodes$axis[i])]
    n.axis[paste0("a.", myadj$nodes$axis[i])] <- n.axis[paste0("a.", myadj$nodes$axis[i])] - 1
  }

  ind.ocg <- which(rownames(tmp)[myadj$edges$id1] %in% namesGenes$OCG)
  myadj$edges$color[ind.ocg] <- "darkgreen"

  ind.ocg <- which(rownames(tmp)[myadj$edges$id2] %in% namesGenes$OCG)
  myadj$edges$color[ind.ocg] <- "darkgreen"

  ind.tsg <- which(rownames(tmp)[myadj$edges$id1] %in% namesGenes$TSG)
  myadj$edges$color[ind.tsg] <- "goldenrod"

  ind.tsg <- which(rownames(tmp)[myadj$edges$id2] %in% namesGenes$TSG)
  myadj$edges$color[ind.tsg] <- "goldenrod"

  if (!is(additionalFilename, "NULL")) {
    pdf(paste0("networkHive", additionalFilename, ".pdf"))
  }

  HiveR::plotHive(myadj,
                  axLabs = c("remaining TFs", "OCG", "TSG"),
                  bkgnd = "white",
                  anNode.gpar = gpar(fontsize = 10, col = "black", lwd = 0.5))

  if (!is(additionalFilename, "NULL")) {
    dev.off()
  }
}

utils::globalVariables(c("is"))

#' tabix_func
#'
#' This function retrives the individial score for a SNP
#' @param Ranges The position
#' @param Reference_Allele The reference nucleotide
#' @param Mutant The mutant nucleotide
#' @param file_coding cscape_table with coding scores
#' @param file_noncoding cscape_table with noncoding scores
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @importFrom tidyr unite nest unnest
#' @importFrom stringr str_split str_replace_all
#' @importFrom seqminer tabix.read.table
#' @importFrom tibble tibble
#'
#' @return returns the score
#' @keywords internal
#' @examples
#' \donttest{
#' data <- tabix_func(Ranges, Reference_Allele, Mutant, file_coding, file_noncoding)
#'}
tabix_func <- function(Ranges,
                       Reference_Allele,
                       Mutant,
                       file_coding,
                       file_noncoding) {

  flag <- FALSE
  bases <- c("A", "T","G", "C")
  chromosomes <- c("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11",
                   "12", "13", "14", "15", "16", "17", "18", "19", "20", "21",
                   "22") # X and Y not available

  chromosome <- str_split(Ranges, pattern = ':', simplify = TRUE) %>% .[[1]]
  remark <- "std"
  type <- ""

  reference <- as.character(Reference_Allele)
  mutant <- as.character(Mutant)

  # Check reference, mutant and chromosome are correct format and possible
  if (!(reference %in% bases) || !(mutant %in% bases)) {
    remark <- "Error: Reference and mutant nucleotides must both be ACGT"
    score <- NA
  } else if (!(chromosome %in% chromosomes)) {
    remark <- "Error: Unexpected chromosome (X and Y not available)"
    score <- NA
    # If all okay, continue:
  } else {
    # Look for score in coding region
    x <- as_tibble(tabix.read.table(tabixFile = file_coding, tabixRange = Ranges)) %>%
      mutate(across(where(is.logical), as.character)) %>%
      mutate(across(.cols = everything(),
                    .fns =~ str_replace_all(string = ., pattern = "TRUE", replacement = "T")))

    # Has the data been found
    if (dim(x)[1] != 0) {
      flag <- TRUE
      type <- "Coding"
    } else {
      # If no annotation is found in this position try the noncoding file
      x <- as_tibble(tabix.read.table(tabixFile = file_noncoding, tabixRange = Ranges)) %>%
        mutate(across(where(is.logical), as.character)) %>%
        mutate(across(.cols = everything(),
                      .fns =~ str_replace_all(string = ., pattern = "TRUE", replacement = "T")))

      if (dim(x)[1] != 0) {
        flag <- TRUE
        type <- "Noncoding"
      } else {
        # If there still is no data found at this position
        flag <- FALSE
        remark <- "Error: Unexcepted position"
        score <- NA
      }
    }

    # If the position has been found
    if (flag == TRUE) {
      if (reference == mutant) {
        remark <- "Error: Reference and mutant must be different"
        score <- NA
      } else if (reference == x$V3[1]) {
        score <- x %>%
          filter(V4 == mutant) %>%
          dplyr::select(V5) %>%
          pull()
        remark <- confidence(score, type)
      } else {
        remark <- "Error: Unexpected reference"
        score <- NA
      }
    }
  }

  if (type == "Coding") {
    data <- tibble(Coding_score = score, Remark = remark)
  } else if (type == "Noncoding") {
    data <- tibble(Noncoding_score = score, Remark = remark)
  } else {
    data <- tibble(Remark = remark)
  }

  return(data)

}

utils::globalVariables(c(".", "V4", "V5"))

#' DMA
#'
#' This function carries out the driver mutation analysis.
#' @param dataMAF A MAF file rda object.
#' The MAF file must at least contain the following columns:
#' \itemize{
#' \item Hugo_Symbol eg. BRCA1
#' \item Chromosome eg. chr1
#' \item Start_Position eg. 54402
#' \item End_Position e.g. 54443
#' \item Strand eg. +
#' \item Variant_Classification
#' \item Variant_Type
#' \item Reference_Allele
#' \item Tumor_Seq_Allele1
#' \item Tumor_Seq_Allele2
#' }
#' @param dataDEGs Output DEA function.
#' @param dataPRA Output PRA function.
#' @param runCscape Bolean. If \code{FALSE} will load CScape output file from results-folder
#' Default = \code{TRUE}.
#' @param coding_file A character string. Path to and name of CScape-somatic coding file.
#' Can be downloaded at \url{http://cscape-somatic.biocompute.org.uk/#download}. The .tbi file must be placed in the same folder.
#' @param noncoding_file A charcter string. Path to and name of CScape-somatic noncoding file.
#' Can be downloaded at \url{http://cscape-somatic.biocompute.org.uk/#download}. The .tbi file must be placed in the same folder.
#' @param results_folder A character string. Path to the results generated by this function.
#'
#' @import dplyr
#' @import tidyr
#' @import tibble
#' @importFrom fuzzyjoin genome_left_join
#' @importFrom magrittr "%>%"
#' @importFrom purrr pmap
#'
#' @return List of two, containing TSGs and OCGs with at least one driver mutation. Additionally files are saved in \code{results_folder}.
#' All output files are in compressed .rda format.
#' \describe{
#' \item{DEG_mutations_annotations.rda}{All differentially expressed genes’ mutations and their annotations. These annotations include e.g. Cscape-somatic assessment, Level of Consequence, overlab with promoter sites and information from Network of Cancer Genes (NCG 7.0). All information from MAF and DEA is contained.}
#' \item{Oncogenic_mediators_annotation_summary.rda}{All oncogenic mediators and an summarisation of their mutation based on CScape-somatic assessment, Level of Consequences and total number of mutations. If a gene as previously been assessed as a driver in Network of Cancer Genes (7.0), it is annotated in a separate column. }
#' \item{Cscape_somatic_output.rda}{The file contain the cscape-somatic assessment for every mutation found in the differentially expressed genes. It is formatted exactly as the output of cscape-somatic, as if it was run in the terminal, except it is saved as .rda instead of csv. }
#' }
#'
#' @details
#' For more information about the different annotations added to the mutations please see the documentation as followes:
#' \code{data(NCG)}, \code{data(EncodePromoters)}, \code{data(LOC_protein)}
#' \code{data(LOC_transcription)} and \code{data(LOC_translation)}.
#'
#'
#' @export
#' @examples
#' \donttest{
#' DMA(dataMAF = dataMAF,
#'     dataDEGs = DEGsmatrix,
#'     dataPRA = dataPRA,
#'     coding_file = "path/css_coding.vcf.gz",
#'     noncoding_file = "path/css_noncoding.vcf.gz",
#'     results_folder = "path/results")
#'
#' #If the cscape-somatic file have already been created
#' cscape_somatic_output <- read.csv("./results/Cscape_somatic_output.csv")
#' save(cscape_somatic_output, file = "./results/Cscape_somatic_output.rda")
#'
#' DMA(dataMAF = dataMAF,
#'     dataDEGs = DEGsmatrix,
#'     dataPRA = dataPRA,
#'     runCscape = FALSE,
#'     results_folder = "./results")
#' }

DMA <- function(dataMAF,
                dataDEGs,
                dataPRA,
                runCscape = TRUE,
                coding_file,
                noncoding_file,
                results_folder = "./DMAresults") {

  # Check user input

  if (is(dim(dataMAF), "NULL")) {
    stop("The mutation data must be a non-empty table")
  }

  if (.row_names_info(dataDEGs) < 0) {
    stop("Row names were generated automatically. The input DEG table needs to
have the gene names as rownames. Double check that genes are rownames.")
  }

  if (all(names(dataPRA) %in% c("TSG", "OCG")) == FALSE) {
    stop("The two list elements in PRA data must be named TSG and OCG")
  }


  if (!is(runCscape, "logical")) {
    stop("runCscape must be either TRUE or FALSE")
  }

  if (!is(results_folder, "character")) {
    stop("The results folder must be a character vector containing the
    path to where DMA results should be stored")
  }

  # List of variable names
  variables_to_check <- c("LOC_transcription", "LOC_translation", 
                        "LOC_protein", "EncodePromoters", "NCG")

  # Check and load variables if they do not exist
  for (variable_name in variables_to_check) {
    if (! variable_name %in% names(.GlobalEnv)) {
      data(list=c(variable_name))
    }
  }

  LOC_transcription <- get("LOC_transcription")
  LOC_translation <- get("LOC_translation")
  LOC_protein <- get("LOC_protein")

  # Create Output folder
  if (dir.exists(results_folder)) {
    print("Output folder already exists")
  } else {
    dir.create(path = results_folder,
               showWarnings = TRUE,
               recursive = TRUE)
  }

  # Load data --------------------------------
  # Read maf and add ID number to each mutation
  columns <- c("Hugo_Symbol",
               "Chromosome",
               "Start_Position",
               "End_Position",
               "Strand",
               "Variant_Classification",
               "Variant_Type",
               "Reference_Allele",
               "Tumor_Seq_Allele1",
               "Tumor_Seq_Allele2")

  if (all(columns %in% names(dataMAF)) == FALSE) {
    stop("MAF file does not contain the correct columns")
  }

  dataMAF <- dataMAF %>% tibble::rowid_to_column("ID")

  drivers_moonlight <- PRAtoTibble(dataPRA)

  dataDEGs <- dataDEGs %>% tibble::rownames_to_column(var = "Hugo_Symbol")

  # Load homemade mutations effect on transcription table
  transcription_binary <- get("LOC_transcription") %>%
    pivot_longer(cols = !Variant_Classification,
                 names_to = "Variant_Type",
                 values_to = "Potential_Effect_on_Transcription")

  translation_binary <- get("LOC_translation") %>%
    pivot_longer(cols = !Variant_Classification,
                 names_to = "Variant_Type",
                 values_to = "Potential_Effect_on_Translation")

  protein_binary <- get("LOC_protein") %>%
    pivot_longer(cols = !Variant_Classification,
                 names_to = "Variant_Type",
                 values_to = "Potential_Effect_on_Protein")

  # Load ENCODE file (promoters)
  promoters <- get("EncodePromoters") %>% mutate(Annotation = "Promoter")

  # Load NCG file
  NCG <- get("NCG")

  # Wrangle data -----------------------------
  # Keep only mutations in DEGs
  DEGs_mut <- dataDEGs %>% left_join(dataMAF, by = "Hugo_Symbol")

  # CScape-somatic -------------------------
  # Lifting from one build to another
  mut_only <- DEGs_mut %>% filter(!is.na(ID))
  DEGs_mut_hg19 <- LiftMAF(Infile = mut_only, Current_Build = "GRCh38")

  if (runCscape == TRUE) {
    cscape_in <- MAFtoCscape(DEGs_mut_hg19)

    # Check coding and noncoding files exits
    if (file.exists(coding_file) == FALSE | file.exists(noncoding_file) == FALSE) {
      stop("CScape-somatic vcf files not found at provided path.")
    } else {
      # Run CScape-somatic
      print("CScape-somatic will now run. Please wait")
      cscape_somatic_output <- RunCscape_somatic(input = cscape_in,
                                                 coding_file = coding_file,
                                                 noncoding_file = noncoding_file)

      save(cscape_somatic_output, file = paste(results_folder, "cscape_somatic_output.rda", sep = "/"))
      print("CScape_somatic is finished. Output file is saved in result folder.")
    }
  } else if (runCscape == FALSE) {
    cs_flag <- file.exists(paste(results_folder, "cscape_somatic_output.rda", sep = "/"))
    if (cs_flag == TRUE) {
      print("CScape_somatic files have been found in results-folder.")
    } else {
      stop("cscape_somatic_output.rda file is not found in results-folder.
            Please provide correct path, or set runCScape = TRUE.")
    }
  }

  # Reload CScape output and add cscape_columns in case one type is not found
  cscape_cols <- c(Coding_score = NA, Noncoding_score = NA, Remark = NA)
  load(paste(results_folder, "cscape_somatic_output.rda", sep = "/"))
  cscape_somatic_output <- cscape_somatic_output %>%
    add_column(., !!!cscape_cols[setdiff(names(cscape_cols), names(.))]) %>%
    rename_with(.cols = c("Coding_score", "Noncoding_score", "Remark"),
                .fn = ~ paste("CScape_", ., sep = "")) %>%
    mutate(Variant_Type = "SNP")

  # Merge CScape results
  DEGs_mut_annotated_19 <- DEGs_mut_hg19 %>%
    separate(Chromosome,
             into = c(NA, "Chr"),
             sep = 3,
             remove = FALSE,
             convert = TRUE) %>%
    mutate(Mutant = case_when(Reference_Allele == Tumor_Seq_Allele1 ~ Tumor_Seq_Allele2,
                              Reference_Allele == Tumor_Seq_Allele2 ~ Tumor_Seq_Allele1)) %>%
    left_join(cscape_somatic_output,
              by = c("Start_Position" = "Position",
                     "Variant_Type",
                     "Chr",
                     "Mutant")) %>%
    mutate(CScape_Mut_Class = case_when((CScape_Coding_score > 0.5 | CScape_Noncoding_score > 0.5) ~ "Driver", # Driver
                                        (CScape_Coding_score <= 0.5 | CScape_Noncoding_score <= 0.5 ~ "Passenger"), # Passenger
                                        TRUE ~ "Unclassified")) %>%  # no score is found
    unique()

  # Lift back to 38
  DEGs_mut_annotated <- LiftMAF(Infile = DEGs_mut_annotated_19,
                                Current_Build = "GRCh37")

  # Annotate file --------------------------------

  # Add level of consequence ----------
  DEGs_mut_annotated <- DEGs_mut_annotated %>%
    left_join(transcription_binary,
              by = c("Variant_Classification",
                     "Variant_Type")) %>%
    left_join(translation_binary,
              by = c("Variant_Classification",
                     "Variant_Type")) %>%
    left_join(protein_binary,
              by = c("Variant_Classification",
                     "Variant_Type"))

  # Add again the DEGs with no mutations --
  # and merge driver status from moonlight
  no_muts <- DEGs_mut %>%
    filter(is.na(ID))

  DEGs_mut_annotated <- bind_rows(DEGs_mut_annotated,
                                  no_muts) %>%
    replace_na(list(CScape_Mut_Class = "No_mutations"))

  # Add drivers from moonlight ------------
  DEGs_mut_annotated <- DEGs_mut_annotated %>%
    left_join(drivers_moonlight,
              by = "Hugo_Symbol")

  ## Promoters -------------

  # Merge Encode files into one
  encode_tibble <- promoters %>%
    dplyr::select(X1, X2, X3, Annotation) %>%
    dplyr::rename("Chromosome_annot" = "X1",
                  "Annotation_Start" = "X2",
                  "Annotation_End" = "X3")

  # Join when mutation overlap with an annotations position
  mutations_encode <- genome_left_join(x = DEGs_mut_annotated,
                                       y = encode_tibble,
                                       by = c("Chromosome" = "Chromosome_annot",
                                              "Start_Position" = "Annotation_Start",
                                              "End_Position" = "Annotation_End")) %>%
    dplyr::select(-c("Chromosome_annot"))

  # If a mutation overlap with several annotation, keep them in the same row
  DEGs_mut_annotated <- mutations_encode %>%
    group_by(ID) %>%
    mutate(Annotation = toString(Annotation),
           Annotation_Start = toString(Annotation_Start),
           Annotation_End = toString(Annotation_End)) %>%
    ungroup() %>%
    unique()

  # Make summary table --------------------------

  # Summarise CScape-somatic mutation types
  Summary_per_gene_1 <- DEGs_mut_annotated %>%
    group_by(Hugo_Symbol,
             Moonlight_Oncogenic_Mediator,
             CScape_Mut_Class) %>%
    summarise(n = n()) %>%
    pivot_wider(names_from = CScape_Mut_Class,
                values_from = n,
                names_prefix = "CScape_")

  # Summarise level of consequence
  Summary_per_gene_2 <- DEGs_mut_annotated %>%
    group_by(Hugo_Symbol,
             Moonlight_Oncogenic_Mediator) %>%
    summarise(Transcription_mut_sum = sum(Potential_Effect_on_Transcription, na.rm = TRUE),
              Translation_mut_sum = sum(Potential_Effect_on_Translation, na.rm = TRUE),
              Protein_mut_sum = sum(Potential_Effect_on_Protein, na.rm = TRUE),
              Total_Mutations = sum(!is.na(ID)))

  # Join summaries and add NCG data
  Oncogenic_mediators_mutation_summary <- full_join(Summary_per_gene_1,
                                                    Summary_per_gene_2,
                                                    by = c("Hugo_Symbol",
                                                           "Moonlight_Oncogenic_Mediator")) %>%
    filter(!is.na(Moonlight_Oncogenic_Mediator)) %>%
    arrange(dplyr::desc(CScape_Driver)) %>%
    dplyr::select(!CScape_No_mutations) %>%
    left_join(NCG,
              by = c("Hugo_Symbol" = "symbol"))

  save(Oncogenic_mediators_mutation_summary,
       file = paste(results_folder, "Oncogenic_mediators_mutation_summary.rda", sep = "/"))

  # Make mutation Table  including all annotations from this analysis --------
  # NCG is joined and the table order is restructured
  DEGs_lastCol <- colnames(dataDEGs)[ncol(dataDEGs)]

  DEG_Mutations_Annotations <- DEGs_mut_annotated %>%
    left_join(NCG, by = c("Hugo_Symbol" = "symbol")) %>%
    relocate(ID) %>%
    relocate(starts_with("NCG"),
             .after = DEGs_lastCol) %>%
    relocate(any_of(c("Moonlight_gene_z_score",
                      "Moonlight_Oncogenic_Mediator",
                      "CScape_Coding_score",
                      "CScape_Noncoding_score",
                      "CScape_Mut_Class",
                      "CScape_Remark",
                      "Potential_Effect_on_Transcription",
                      "Potential_Effect_on_Translation",
                      "Potential_Effect_on_Protein",
                      "Annotation",
                      "Annotation_Start",
                      "Annotation_End")),
             .after = DEGs_lastCol) %>%
    dplyr::select(!(Chr))

  save(DEG_Mutations_Annotations,
       file = paste(results_folder, "DEG_Mutations_Annotations.rda", sep = "/"))

  # Return final list of drivers ------
  TSG <-  DEGs_mut_annotated %>%
    filter(Moonlight_Oncogenic_Mediator == "TSG",
           CScape_Mut_Class == "Driver") %>%
    dplyr::select(Hugo_Symbol) %>%
    unique() %>%
    pull()

  OCG <- DEGs_mut_annotated %>%
    filter(Moonlight_Oncogenic_Mediator == "OCG",
           CScape_Mut_Class == "Driver") %>%
    dplyr::select(Hugo_Symbol) %>%
    unique() %>%
    pull()

  return(list("TSG" = TSG, "OCG" = OCG))

}

utils::globalVariables(c("Variant_Classification", "ID", ".", "Chromosome", "X1", "X2", "X3", "Annotation",
                         "Annotation_Start", "Annotation_End", "Hugo_Symbol", "Moonlight_Oncogenic_Mediator",
                         "CScape_Mut_Class", "Potential_Effect_on_Transcription", "Potential_Effect_on_Translation",
                         "Potential_Effect_on_Protein", "CScape_Driver", "CScape_No_mutations", "Chr"))

# Test DMA function

# Run example of DMA function
data(dataMAF)
data(DEGsmatrix)
data(dataPRA)
data(EncodePromoters)
data(NCG)
data(dataFilt)
data(cscape_somatic_output)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
dataDMA <- DMA(dataMAF = dataMAF,
               dataDEGs = DEGsmatrix,
               dataPRA = dataPRA,
               runCscape = FALSE,
               results_folder = ".")

# Test that output of DMA is as expected
test_that("output of DMA is a list with 2 elements", {
  expect_type(dataDMA, "list")
  expect_named(dataDMA, c("TSG", "OCG"))
  expect_true(all(sapply(dataDMA, is.character)))
})

#' confidence
#'
#' This function annotated a confidence level to the score
#' @param s the score
#' @param type coding or noncoding
#'
#' @return returns a confidence level or remark/error message
#' @export
#' @examples
#'
#' remark <- confidence(0.8, type='Coding')


confidence <- function(s,
                       type) {
  # Returns the confidence level for (neutral, low-confidence or
  # high-confidence) for the given p-value string using the given threshold.
  # If the string is invalid, returns "no prediction".

  # Initialization:
  # Confidence thresholds
  DEFAULT_THRESH <- 0.5
  CODING_THRESH <- 0.89
  NONCODING_THRESH <- 0.7
  if (type == "Coding") {
    thresh <- CODING_THRESH
  } else {
    thresh <- NONCODING_THRESH
  }

  # Function:
  x <- as.numeric(s)
  if (x >= thresh) {
    return("High-confidence")
  } else if (x >= DEFAULT_THRESH) {
    return("Low-confidence")
  } else {
    return("Neutral")
  }
}

#' @title plotURA: Upstream regulatory analysis heatmap plot
#' @description
#' This function visualizes the URA in a heatmap
#' @param dataURA output URA function
#' @param additionalFilename figure name
#' @importFrom gplots heatmap.2
#' @importFrom gplots redblue
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @return heatmap
#' @export
#' @examples
#' data(dataURA)
#' data(DiseaseList)
#' data(tabGrowBlock)
#' data(knownDriverGenes)
#' dataDual <- PRA(dataURA = dataURA,
#' BPname = c("apoptosis","proliferation of cells"),
#' thres.role = 0)
#' TSGs_genes <- names(dataDual$TSG)
#' OCGs_genes <- names(dataDual$OCG)
#' plotURA(dataURA = dataURA[c(TSGs_genes, OCGs_genes),],additionalFilename = "_example")
plotURA <- function(dataURA,
                    additionalFilename = "URAplot") {

  # Check user input

  if (!is(dataURA, "matrix")) {
    stop("dataURA must be a numeric matrix")
  }

  if (nrow(dataURA) < 2 | ncol(dataURA) < 2) {
    stop("dataURA must have at least two rows and two columns")
  }

  if (!is(additionalFilename, "NULL") & !is(additionalFilename, "character")) {
    stop("additionalFilename must be either NULL or a character vector 
containing part of the filename of plot")
  }

  if (nrow(dataURA) > 70) {
    cexRow <- 0.1 + 1 / (2 * nrow(dataURA)) * (log10(nrow(dataURA)))
  } else {
    cexRow <- 0.2 + 1 / (10 * log10(nrow(dataURA)))
  }

  if (!is(additionalFilename, "NULL")) {
    pdf(file = paste0("plotURA", additionalFilename, ".pdf"))
  }

  par(oma = c(6, 4, 4, 2))

  heatmap.2(dataURA,
            trace = "none",
            col = rev(gplots::redblue(128)),
            Colv = TRUE,
            dendrogram = "row",
            notecex = 10 / nrow(dataURA),
            cexCol = 0.2 + 1 / (3 * log10(ncol(dataURA))),
            cexRow = cexRow,
            cellnote = signif(dataURA, 4),
            notecol = "black",
            key = TRUE,
            keysize = 2)

  if (!is(additionalFilename, "NULL")) {
    dev.off()
  }
}

#' TFinfluence
#' 
#' This function finds mutations in the transcription factors (TF) of the DEGs that have a TF in the database.
#' 
#' @param dataTRRUST Tibble containing gene-TF pairs and type of interaction
#' Must contain the following columns:
#' \itemize{
#' \item TF (HUGO symbol of TF)
#' \item Target (HUGO symbol of target gene of TF)
#' \item InteractionType (The effect the TF has on the target gene (Activation, Repression))
#' }
#' @param dataMAF Tibble containing mutation info from MAF
#' Must at least contain the following columns:
#' \itemize{
#' \item Hugo_Symbol eg. BRCA1
#' \item HGVSp_Short eg. p.V83F
#' }
#' @param dataDEGs Output DEA function. Tibble containing differentially expressed genes.
#' Must contain the following columns
#' \itemize{
#' \item GENE (HUGO symbol of DEG)
#' \item logFC (The log fold change of DEG)
#'}
#' @param dataPRA Output PRA function. List of TSG and OCG
#' Must contain the following elements
#' \itemize{
#' \item TSG (The TSGs identified by moonlight along with the moonlight score)
#' \item OCG (The OCGs identified by moonlight along with the moonlight score)
#'}
#' @param dataMAVISp Output loadMAVISp function. List of tibbles, one for each protein.
#' The tibbles must contain at least
#' \itemize{
#' \item (First column must contain the mutation (e.g. A54W). It is assumed that the column name is empty)
#'}
#' Then the tibbles must contain columns mathing either of the following names(or they will be excluded) 
#' \itemize{
#' \item (Stability classification, [A-Za-z0-9]+, \\(Rosetta, FoldX\\)) (Values: Stabilizing, Neutral, Destabilizing, Uncertain)
#' \item (Stability classification, [A-Za-z0-9]+, \\(RaSP, FoldX\\)) (Values: Stabilizing, Neutral, Destabilizing, Uncertain)
#'}
#' @param stabClassMAVISp The protocol to use for mutation stability classification. Default is rasp.
#' Accepts one of the following strings
#' \itemize{
#' \item rosetta (uses the FoldX/Rosetta protocol)
#' \item rasp (uses the FoldX/RaSP protocol)
#' }
#' 
#' @import dplyr
#' @importFrom tibble rownames_to_column
#' @importFrom tidyr drop_na
#' @importFrom purrr map keep
#' @importFrom data.table rbindlist
#' @importFrom stringr str_extract
#' 
#' @return returns a tibble containing:
#' \itemize{
#' \item GENE
#' \item TF 
#' \item InteractionType 
#' \item PMID
#' \item tf_mutation
#' \item stab_class (the effect on stability as classified by MAVISp)
#' }
#'
#' @examples
#'
#' data('dataPRA')
#' data('DEGsmatrix')
#' data('dataTRRUST')
#' data('dataMAF')
#' data('dataMAVISp')
#'
#' TFinfluence(dataTRRUST = dataTRRUST,
#'            dataMAF = dataMAF,
#'            dataDEGs = DEGsmatrix,
#'            dataPRA = dataPRA,
#'            dataMAVISp = dataMAVISp,
#'            stabClassMAVISp = 'rasp')
#'
#' @export

TFinfluence <- function(dataPRA,
                        dataDEGs,
                        dataTRRUST,
                        dataMAF,
                        dataMAVISp,
                        stabClassMAVISp = 'rasp'){ 
    # Control user input -------------
    # dataPRA
    if (all(names(dataPRA) %in% c("TSG", "OCG")) == FALSE) {
        stop("The two list elements in PRA data must be named TSG and OCG")
    }

    # dataDEGs
    if (is.null(dim(dataDEGs)) | nrow(dataDEGs) == 0) {
        stop("The DEG data must be a non-empty table")
    }

    # dataTRRUST
    if (is.null(dim(dataTRRUST)) | nrow(dataTRRUST) == 0) {
        stop("The transcription factor data must be a non-empty table")
    }

    trrust_columns <- c('TF', 'Target', 'InteractionType')

    if (all(trrust_columns %in% names(dataTRRUST)) == FALSE) {
        stop("TRRUST dataframe does not contain the correct columns")
    }

    # dataMAF
    if (is.null(dim(dataMAF)) | nrow(dataMAF) == 0) {
        stop("The mutation data must be a non-empty table")
    }

    maf_columns <- c("Hugo_Symbol",
                "HGVSp_Short")

    if (all(maf_columns %in% names(dataMAF)) == FALSE) {
        stop("MAF file does not contain the correct columns")
    }

    # stabClassMAVISp
    if (!(stabClassMAVISp %in% c('rasp', 'rosetta'))){
        stop("The protocol for stability classification is not specified correctly. Accepts strings 'rasp' or 'rosetta'")
    } else {
        stabClassMAVISp <- paste0('stab_class_', stabClassMAVISp)
    }

    # Load data --------------------------------
    drivers <- PRAtoTibble(dataPRA)

    # Read maf and add ID number to each mutation
    dataMAFFiltered <- dataMAF |> 
        filter(Variant_Classification == "Missense_Mutation") |> 
        select(c("Hugo_Symbol",
                "HGVSp_Short")) |>
        mutate(HGVSp_Short = str_extract(HGVSp_Short, pattern = "[A-Z]\\d+[A-Z]")) |>
        rename(mutation = HGVSp_Short) |>
        drop_na()

    # Filter MAVISp data 
    # Keep only the stability classification
    dataMAVISpFiltered <- dataMAVISp |>
                    map(function(x) rename(x, 'stab_class_rosetta' = matches('Stability classification, [A-Za-z0-9, ]*\\(Rosetta, FoldX\\)'),
                                              'stab_class_rasp' = matches('Stability classification, [A-Za-z0-9, ]*\\(RaSP, FoldX\\)'))) |>
                    keep(function(x) stabClassMAVISp %in% colnames(x)) |>
                    map(function(x) rename(x, 
                                           'mutation' = 1,
                                           'stab_class' = stabClassMAVISp) |>
                                    select(mutation, stab_class)) |>
                    rbindlist(idcol = 'protein') |>
                    as_tibble()|>
                    mutate(in_MAVISp = TRUE)

    # Analysis -------------------
    # Convert rownames to column for DEGs
    dataDEGs <- dataDEGs |>
        rownames_to_column(var = "GENE") |>
        select(GENE, logFC) 

    # Join drivers with expression
    drivers_expr <- drivers |>
        left_join(dataDEGs, 
                  by = join_by(Hugo_Symbol == GENE))

    # Join drivers with TF
    drivers_expr_tf <- drivers_expr |>
        left_join(dataTRRUST,
                  by = join_by(Hugo_Symbol == Target)) |>
        rename('logFC_target' = logFC,
               'Target' = Hugo_Symbol)

    # Map mutation file to TF
    drivers_TF_mut <- drivers_expr_tf |>
        left_join(dataMAFFiltered,
                  by = join_by(TF == Hugo_Symbol),
                  relationship = "many-to-many") |>
        rename('tf_mutation' = mutation)

    # Match TF-mut with mavisp to see the effect
    drivers_mut_mavisp <- drivers_TF_mut |>
      left_join(dataMAVISpFiltered, 
                by = join_by(TF == protein, tf_mutation == mutation)) |> 
      mutate(in_MAVISp = TF %in% unique(dataMAVISpFiltered$protein)) |> 
      mutate(mutation_available = ifelse(paste(TF, tf_mutation) %in%
                                         paste(dataMAVISpFiltered$protein, dataMAVISpFiltered$mutation), TRUE, FALSE))
    # Check biological implications
    # Activation -> destabilising mutation -> decrease
    # Repression -> destabilising mutation -> increase
    filtered_effect <- drivers_mut_mavisp |>
        filter((InteractionType == 'Activation' & stab_class == 'Destabilizing' & logFC_target < 0) |
                (InteractionType == 'Repression' & stab_class == 'Destabilizing' & logFC_target > 0) |
                (stab_class == 'Uncertain') |
                (stab_class == 'Neutral'))

    return(drivers_mut_mavisp)
}

# Test TFinfluence function

# Run example of TFinfluence function
data(dataPRA)
data(DEGsmatrix)
data(dataTRRUST)
data(dataMAF)
data(dataMAVISp)


reference_TFresults <- data.frame(
    Target = c("CHST1", "GEN1", "TAP1", "TAP1"),
    Moonlight_gene_z_score = c(1.06066017177982, 1.30096115353815, 0.8262079533974, 0.8262079533974),
    Moonlight_Oncogenic_Mediator = c("TSG", "OCG", "OCG", "OCG"),
    logFC_target = c(1.94917640749552, 1.36435629696216, 1.36486453774995, 1.36486453774995),
    TF = c(NA, NA, "IRF1", "IRF2"),
    InteractionType = c(NA, NA, "Activation", "Activation"),
    PMID = c(NA, NA, "18694960", "15778351"),
    tf_mutation = as.character(c(NA, NA, NA, NA)),
    stab_class = as.character(c(NA, NA, NA, NA)),
    in_MAVISp = c(FALSE,FALSE,FALSE,FALSE),
    mutation_available = c(FALSE,FALSE,FALSE,FALSE),
    stringsAsFactors = FALSE
)

TFresults <- TFinfluence(dataTRRUST = dataTRRUST,
            dataMAF = dataMAF,
            dataDEGs = DEGsmatrix,
            dataPRA = dataPRA,
            dataMAVISp = dataMAVISp,
            stabClassMAVISp = 'rasp')

character_columns <- c('Target','Moonlight_Oncogenic_Mediator','TF','InteractionType','PMID','tf_mutation','stab_class')
numeric_columns <- c('Moonlight_gene_z_score','logFC_target')
logical_columns <- c('in_MAVISp','mutation_available')
all_columns <- c('Target', 'Moonlight_gene_z_score', 'Moonlight_Oncogenic_Mediator', 'logFC_target', 'TF', 'InteractionType', 'PMID', 'tf_mutation', 'stab_class','in_MAVISp','mutation_available')


# Test that output is as expected
test_that('Output of TFinfluence is a tibble', {
    expect_named(TFresults, all_columns)
    expect_true(all(sapply(TFresults[character_columns], is.character)))
    expect_true(all(sapply(TFresults[numeric_columns], is.numeric)))
    expect_true(all(sapply(TFresults[logical_columns], is.logical)))
})

# Test that the content of the output dataframe is as expected
test_that("Output of TFinfluence does not correspond to reference", {
    expect_equal(TFresults, reference_TFresults,ignore_attr = TRUE)
})

#' LPA
#'
#' This function carries out the literature phenotype analysis (LPA)
#' @param dataDEGs is output from DEA
#' @param BP is biological process
#' @param BPlist is list of genes annotated in BP
#' @import RISmed
#' @importFrom utils txtProgressBar
#' @importFrom utils setTxtProgressBar
#' @export
#' @return table with number of pubmed that affects, increase or decrase genes annotated in BP
#' @examples
#' data('DEGsmatrix')
#' data('DiseaseList')
#' BPselected <- c("apoptosis")
#' BPannotations <- DiseaseList[[match(BPselected, names(DiseaseList))]]$ID
LPA <- function(dataDEGs,
                BP,
                BPlist) {

  # Check user input

  if (.row_names_info(dataDEGs) < 0) {
    stop("Row names were generated automatically. The input DEG table needs to
         have the gene names as rownames. Double check that genes are rownames.")
  }

  if ("logFC" %in% colnames(dataDEGs) == FALSE) {
    stop("The input DEG table must contain a column called logFC.")
  }

  if (all(BP %in% names(DiseaseList)) == FALSE) {
    stop("BPname should be a character vector containing one or more BP(s)
         among possible BPs stored in the DiseaseList object.")
  }

  if (!is(BPlist, "character")) {
    stop("BPlist must be a character vector of genes")
  }

  BPgenesDEGs <- intersect(BPlist, rownames(dataDEGs))

  dataDEGsBP <- dataDEGs[BPgenesDEGs, ]

  DiseaseMN <- matrix(0, nrow(dataDEGsBP), 7)

  colnames(DiseaseMN) <- c("ID",
                           "Genes.in.dataset",
                           "Prediction..based.on.expression.direction.",
                           "Exp.Log.Ratio",
                           "Findings",
                           "PubmedDecreases",
                           "PubmedIncreases")

  DiseaseMN <- as.data.frame(DiseaseMN)
  DiseaseMN$Prediction..based.on.expression.direction. <- rep("Decreased", nrow(dataDEGsBP))
  DiseaseMN$ID <- BPgenesDEGs
  DiseaseMN$Genes.in.dataset <- BPgenesDEGs
  DiseaseMN$Exp.Log.Ratio <- dataDEGs[BPgenesDEGs, "logFC"]
  rownames(DiseaseMN) <- DiseaseMN$ID

  pb <- txtProgressBar(min = 0, max = nrow(DiseaseMN), style = 3)

  for (i in seq.int(nrow(DiseaseMN))) {

    setTxtProgressBar(pb, i)

    curG <- DiseaseMN$ID[i]

    keypubmed <- paste(curG, BP, "decreases")
    res <- EUtilsSummary(keypubmed, type = "esearch", db = "pubmed")
    fetch <- EUtilsGet(res, type = "efetch", db = "pubmed")
    RecordsDec <- length(fetch@PMID)
    DiseaseMN[curG, "PubmedDecreases"] <- RecordsDec

    keypubmed <- paste(curG, BP, "increases")
    res <- EUtilsSummary(keypubmed, type = "esearch", db = "pubmed")
    fetch <- EUtilsGet(res, type = "efetch", db = "pubmed")
    RecordsInc <- length(fetch@PMID)
    DiseaseMN[curG, "PubmedIncreases"] <- RecordsInc

    if (DiseaseMN[curG, "PubmedDecreases"] == DiseaseMN[curG, "PubmedIncreases"]) {
      DiseaseMN[curG, "Findings"] <- paste0("Affects (", RecordsInc, ")")
    }

    if (DiseaseMN[curG, "PubmedDecreases"] < DiseaseMN[curG, "PubmedIncreases"]) {
      DiseaseMN[curG, "Findings"] <- paste0("Increases (", RecordsInc, ")")
    }

    if (DiseaseMN[curG, "PubmedDecreases"] > DiseaseMN[curG, "PubmedIncreases"]) {
      DiseaseMN[curG, "Findings"] <- paste0("Decreases (", RecordsDec, ")")
    }

  }

  close(pb)

  return(DiseaseMN)

}

utils::globalVariables(c("DiseaseList"))

#' PRAtoTibble
#'
#' This function changes the PRA output to tibble format
#' @param dataPRA RDA object (list of two) from PRA
#' @import dplyr
#' @importFrom magrittr "%>%"
#' @importFrom stringr str_trim
#'
#' @return tibble with drivers
#' @export
#' @examples
#' data('dataPRA')
#' PRAtoTibble(dataPRA)
PRAtoTibble <- function(dataPRA) {

  # Check user input

  if (all(names(dataPRA) %in% c("TSG", "OCG")) == FALSE) {
    stop("The two list elements in PRA data must be named TSG and OCG")
  }

  # Wrangle data

  if (!is(dataPRA$TSG, "NULL")) {
    TSG <- as_tibble(dataPRA$TSG, rownames = NA) %>%
      rownames_to_column(var = "Hugo_Symbol") %>%
      mutate(Hugo_Symbol = str_trim(Hugo_Symbol, side = "both"),
             Moonlight_Oncogenic_Mediator = "TSG") %>%
      dplyr::rename(Moonlight_gene_z_score = value)
  } else {
    TSG <- NULL
  }

  if (!is(dataPRA$OCG, "NULL")) {
    OCG <- as_tibble(dataPRA$OCG, rownames = NA) %>%
      rownames_to_column(var = "Hugo_Symbol") %>%
      mutate(Hugo_Symbol = str_trim(Hugo_Symbol, side = "both"),
             Moonlight_Oncogenic_Mediator = "OCG") %>%
      dplyr::rename(Moonlight_gene_z_score = value)
  } else {
    OCG <- NULL
  }

  if (!is(TSG, "NULL") & !is(OCG, "NULL")) {
    drivers <- full_join(TSG, OCG, by = c("Hugo_Symbol",
                                          "Moonlight_gene_z_score",
                                          "Moonlight_Oncogenic_Mediator"))
  } else if (is(TSG, "NULL")) {
    drivers <- OCG
  } else if (is(OCG, "NULL")) {
    drivers <- TSG
  }

  return(drivers)

}

utils::globalVariables(c("Hugo_Symbol", "value"))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(dpi = 72)
knitr::opts_chunk$set(cache=FALSE)

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("Moonlight2_pipeline_upd.png")

## ----eval = FALSE-------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("Moonlight2R")

## ----eval = FALSE-------------------------------------------------------------
# install.packages("devtools")
# library(devtools)

## ----eval = FALSE-------------------------------------------------------------
# devtools::install_github(repo = "ELELAB/Moonlight2R")

## ----eval = FALSE-------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("BiocStyle")

## ----eval = FALSE-------------------------------------------------------------
# devtools::install_github(repo = "ELELAB/Moonlight2R", build_vignettes = TRUE)

## ----eval = FALSE-------------------------------------------------------------
# vignette( "Moonlight2R", package="Moonlight2R")

## ----eval = TRUE--------------------------------------------------------------
library(Moonlight2R)
library(magrittr)
library(dplyr)

## ----eval = TRUE--------------------------------------------------------------
data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(GEO_TCGAtab)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)
data(dataMethyl)
data(DEG_Methylation_Annotations)
data(Oncogenic_mediators_methylation_summary)
data(MetEvidenceDriver)
data(LUAD_sample_anno)

## ----eval = TRUE, echo = TRUE-------------------------------------------------
knitr::kable(GEO_TCGAtab, digits = 2,
       caption = "Table with GEO data set matched to one
       of the 18 given TCGA cancer types ",
       row.names = TRUE)

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
dataFilt_GEO <- getDataGEO(GEOobject = "GSE20347", platform = "GPL571")

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
dataFilt_GEO <- getDataGEO(TCGAtumor = "ESCA")

## ----eval = TRUE, echo = TRUE, results='hide'---------------------------------
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
dataFEA <- FEA(DEGsmatrix = DEGsmatrix)

## ----eval = TRUE, echo = TRUE, message=FALSE, results='hide', warning=FALSE----
plotFEA(dataFEA = dataFEA, additionalFilename = "_exampleVignette", height = 10, width = 20)

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("FEAplot.gif")

## ----eval = TRUE--------------------------------------------------------------
data(DEGsmatrix)
data(dataFilt)
dataGRN <- GRN(DEGsmatrix = DEGsmatrix, TFs = sample(rownames(DEGsmatrix), 100), normCounts = dataFilt,
	               nGenesPerm = 5, kNearest = 3, nBoot = 5, DiffGenes = TRUE)

## ----eval = TRUE, echo = TRUE, results='hide'---------------------------------
data(dataGRN)
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)

dataFEA <- FEA(DEGsmatrix = DEGsmatrix)

BPselected <- dataFEA$Diseases.or.Functions.Annotation[1:5]
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = BPselected,
               nCores=1)

## ----eval = TRUE--------------------------------------------------------------
data(dataURA)
data(tabGrowBlock)
data(knownDriverGenes)
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis","proliferation of cells"),
               thres.role = 0)

## ----eval = FALSE-------------------------------------------------------------
# data(dataPRA)
# data(dataMAF)
# data(DEGsmatrix)
# data(LOC_transcription)
# data(LOC_translation)
# data(LOC_protein)
# data(NCG)
# data(EncodePromoters)
# dataDMA <- DMA(dataMAF = dataMAF,
#                dataDEGs = DEGsmatrix,
#                dataPRA = dataPRA,
#                results_folder = "DMAresults",
#                coding_file = "css_coding.vcf.gz",
#                noncoding_file = "css_noncoding.vcf.gz")

## ----eval = TRUE--------------------------------------------------------------
data("dataMethyl")
data("dataFilt")
data("dataPRA")
data("DEGsmatrix")
data("LUAD_sample_anno")
data("NCG")
data("EncodePromoters")
data("MetEvidenceDriver")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)

## ----eval = TRUE--------------------------------------------------------------
genes_query <- "BRCA1"
dataGLS <- GLS(genes = genes_query,
	       query_string = "AND cancer AND driver")
head(dataGLS)

## ----eval = TRUE--------------------------------------------------------------

mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')

specific_protein <- loadMAVISp(mavispDB = mavisp_db_location,
                               mode = 'simple',
                               proteins_of_interest = c('RUNX1'))

all_proteins <- loadMAVISp(mavispDB = mavisp_db_location,
                           mode = 'simple')

ensemble <- loadMAVISp(mavispDB = mavisp_db_location,
                       mode = 'ensemble',
                       ensemble = 'cabsflex')

## ----eval = TRUE--------------------------------------------------------------
data(dataPRA)
data(DEGsmatrix)
data(dataTRRUST)
data(dataMAF)
data(dataMAVISp)

TFresults <- TFinfluence(dataTRRUST = dataTRRUST,
            dataMAF = dataMAF,
            dataDEGs = DEGsmatrix,
            dataPRA = dataPRA,
            dataMAVISp = dataMAVISp,
            stabClassMAVISp = 'rasp')

## -----------------------------------------------------------------------------
knitr::kable(LOC_transcription)

## -----------------------------------------------------------------------------
knitr::kable(LOC_translation)

## -----------------------------------------------------------------------------
knitr::kable(LOC_protein)

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
data(knownDriverGenes)
data(dataGRN)
plotNetworkHive(dataGRN, knownDriverGenes, 0.55)

## ----eval = TRUE, warning = FALSE, message = FALSE, include=TRUE--------------
data(dataDMA)
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
plotDMA(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        type = 'complete', additionalFilename = "")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("heatmap_complete.gif")

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
data(dataURA_plot)
plotMoonlight(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        dataURA_plot, gene_type = "drivers", n = 50)

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("moonlight_heatmap.gif")

## ----eval = TRUE, warning = FALSE, message = FALSE, include = TRUE------------
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotGMA(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
        Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
        type = "genelist", genelist = genes,
        additionalFilename = "./GMAresults/")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("heatmap_genelist_met.gif")

## ----eval = TRUE, warning = FALSE, message = FALSE, include = TRUE------------
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
data("dataURA_plot")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotMoonlightMet(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
                 Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
                 dataURA = dataURA_plot,
                 genes = genes,
                 additionalFilename = "./GMAresults/")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("moonlight_heatmap_met.gif")

## ----eval = TRUE, warning = FALSE, message = FALSE, include = TRUE------------
data("EpiMix_Results_Regular")
data("dataMethyl")
data("dataFilt")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

plotMetExp(EpiMix_results = EpiMix_Results_Regular,
           probe_name = "cg03035704",
           dataMET = dataMethyl,
           dataEXP = dataFilt,
           gene_of_interest = "ACVRL1",
           additionalFilename = "./GMAresults/")

## ----fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("plotMetExp.png")

## ----eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE, results='hide'--------
data(DEGsmatrix)
data(dataFilt)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)

dataFEA <- FEA(DEGsmatrix = DEGsmatrix)

dataGRN <- GRN(TFs = sample(rownames(DEGsmatrix), 100),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               nBoot = 5,
	       kNearest = 3)

dataURA <- URA(dataGRN = dataGRN,
              DEGsmatrix = DEGsmatrix,
              BPname = c("apoptosis",
                         "proliferation of cells"))

dataDual <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

oncogenic_mediators <- list("TSG"=names(dataDual$TSG), "OCG"=names(dataDual$OCG))


## ----eval = TRUE,message=FALSE,warning=FALSE, results='hide'------------------
data(dataURA)
plotURA(dataURA = dataURA, additionalFilename = "_exampleVignette")

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("URAplot.gif")

## ----eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE-----------------------
# data(dataFilt)
# data(DEGsmatrix)
# data(dataMAF)
# data(DiseaseList)
# data(EAGenes)
# data(tabGrowBlock)
# data(knownDriverGenes)
# data(LOC_transcription)
# data(LOC_translation)
# data(LOC_protein)
# data(NCG)
# data(EncodePromoters)
# 
# listMoonlight <- moonlight(dataDEGs = DEGsmatrix,
#                            dataFilt = dataFilt,
#                            nTF = 100,
#                            DiffGenes = TRUE,
# 			   nGenesPerm = 5,
#                            nBoot = 5,
#                            BPname = c("apoptosis","proliferation of cells"),
#                            dataMAF = dataMAF,
#                            path_cscape_coding = "css_coding.vcf.gz",
#                            path_cscape_noncoding = "css_noncoding.vcf.gz")
# save(listMoonlight, file = paste0("listMoonlight_ncancer4.Rdata"))
# 
# 

## ----eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE----
data(listMoonlight)
plotCircos(listMoonlight = listMoonlight, additionalFilename = "_ncancer5")

## ----fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE----
knitr::include_graphics("circos_ocg_tsg_ncancer5.gif")

## ----eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE-----------------------
# 
# data(DEGsmatrix)
# data(dataFilt)
# data(dataMAF)
# data(DiseaseList)
# data(EAGenes)
# data(tabGrowBlock)
# data(knownDriverGenes)
# data(LOC_transcription)
# data(LOC_translation)
# data(LOC_protein)
# data(NCG)
# data(EncodePromoters)
# 
# # Perform gene regulatory network analysis
# dataGRN <- GRN(TFs = rownames(DEGsmatrix),
# 	       DEGsmatrix = DEGsmatrix,
# 	       DiffGenes = TRUE,
#                normCounts = dataFilt,
#                nGenesPerm = 5,
#                kNearest = 3,
#                nBoot = 5)
# 
# # Perform upstream regulatory analysis
# # As example, we use apoptosis and proliferation of cells as the biological processes
# dataURA <- URA(dataGRN = dataGRN,
#                DEGsmatrix = DEGsmatrix,
#                BPname = c("apoptosis",
#                           "proliferation of cells"),
#                nCores = 1)
# 
# # Perform pattern recognition analysis
# dataPRA <- PRA(dataURA = dataURA,
#                BPname = c("apoptosis",
#                           "proliferation of cells"),
#                thres.role = 0)
# 
# # Perform driver mutation analysis
# dataDMA <- DMA(dataMAF = dataMAF,
#                dataDEGs = DEGsmatrix,
#                dataPRA = dataPRA,
#                results_folder = "DMAresults",
#                coding_file = "css_coding.vcf.gz",
#                noncoding_file = "css_noncoding.vcf.gz")
# 

## ----eval = TRUE--------------------------------------------------------------
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)

# Extract oncogenic mediators that contain at least one driver mutation
# These are the driver genes
knitr::kable(Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)))

# Extract mutation annotations of the predicted driver genes
driver_mut <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver")

# Extract driver genes with a predicted effect on the transcriptional level
transcription_mut <- Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)) %>%
  filter(Transcription_mut_sum > 0)

# Extract mutation annotations of predicted driver genes that have a driver mutation
# in its promoter region with a potential effect on the transcriptional level
promoters <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver",
         Potential_Effect_on_Transcription == 1,
         Annotation == 'Promoter')

## ----eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE,results='hide'---------

data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)
data(dataMethyl)
data(LUAD_sample_anno)
data(MetEvidenceDriver)

# Perform gene regulatory network analysis
dataGRN <- GRN(TFs = rownames(DEGsmatrix),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               kNearest = 3,
               nBoot = 5)

# Perform upstream regulatory analysis
# As example, we use apoptosis and proliferation of cells as the biological processes
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               nCores = 1)

# Perform pattern recognition analysis
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

# Perform gene methylation analysis
dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1,
               roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)


## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' plotGMA
#' This function plots results of the Gene Methylation Analysis.
#' It visualizes the number of hypo/hyper/dual methylated CpG
#' sites in oncogenic mediators or in a user-supplied gene list.
#' The results are visualized either in a single heatmap or split
#' into different ones which is specified in the function's three
#' modes: split, complete and genelist.
#' @param DEG_Methylation_Annotations A tibble which is outputted
#' from the GMA function.
#' @param Oncogenic_mediators_methylation_summary A tibble which
#' is outputted from the GMA function.
#' @param type A character string which can either be split,
#' complete or genelist. If type is set to split, the entire
#' dataset is split into groups of 40 genes and individual
#' heatmaps of groups each containing 40 genes will be created and
#' subsequently merged into one pdf where each page in the pdf
#' is an individual heatmap. The genes will be sorted alphabetically.
#' If type is set to complete, a single heatmap is created where
#' the number of differentially methylated CpG sites are shown for
#' all oncogenic mediators. If type is set to genelist, a single
#' heatmap will be created for genes supplied by the user in the
#' genelist parameter. Default is split.
#' @param genelist A character string containing HUGO symbols
#' of genes to be visualized in a single heatmap. Default is NULL.
#' @param additionalFilename A character string that can be used
#' to add a prefix or filepath to the filename of the pdf
#' visualizing the heatmap. Default is an empty string.
#' @import dplyr
#' @import tidyr
#' @import ComplexHeatmap
#' @import tidyHeatmap
#' @importFrom qpdf pdf_combine
#' @importFrom purrr map
#' @importFrom magrittr "%>%"
#' @return No value is returned. Visualizations in form of heatmaps
#' are saved.
#' @export
#' @examples
#' data("DEG_Methylation_Annotations")
#' data("Oncogenic_mediators_methylation_summary")
#' genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
#' plotGMA(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
#'         Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
#'         type = "genelist", genelist = genes,
#'         additionalFilename = "./GMAresults/")
plotGMA <- function(DEG_Methylation_Annotations,
                    Oncogenic_mediators_methylation_summary,
                    type = "split",
                    genelist = NULL,
                    additionalFilename = "") {

  # Check type input
  if (!type %in% c("split", "complete", "genelist")) {
    stop("type must be a character string of either split, complete or genelist")
  }

  # Check genelist input
  if (type == "genelist") {
    if (!is(genelist, "character")) {
      stop("genelist must be a character string")
    }
    if (length(genelist) <= 0) {
      stop("genelist must not be empty")
    }
  }

  # Get logFC of DEGs
  DEGs_logFC <- DEG_Methylation_Annotations %>%
    dplyr::select(Hugo_Symbol,
                  logFC) %>%
    distinct()

  # Reformat Oncogenic_mediators_methylation_summary data to long format
  # needed for visualization in heatmap
  summary_heatmap <- Oncogenic_mediators_methylation_summary %>%
    pivot_longer(data = .,
                 cols = contains("methyl_num"),
                 names_to = "Met_type",
                 values_to = "Met_count") %>%
    dplyr::select(-contains("NCG")) %>%
    replace_na(list(Met_count = 0)) %>%
    left_join(x = .,
              y = DEGs_logFC,
              by = "Hugo_Symbol")

  ## Genelist mode

  # If genelist mode is chosen
  if (type == "genelist") {

    # Subset data to contain only supplied genes in genelist
    summary_heatmap <- summary_heatmap %>%
      filter(Hugo_Symbol %in% genelist)

    # Get number of Moonlight_Oncogenic_Mediator groups in subsetted data
    driver_groups <- summary_heatmap$Moonlight_Oncogenic_Mediator %>%
      unique %>%
      length

    met_heatmap <- summary_heatmap

    # If genelist contain both oncogenic mediator gene types
    if (driver_groups == 2) {
       met_heatmap <- met_heatmap %>% group_by(Moonlight_Oncogenic_Mediator)
    }

   # Create heatmap of genes in genelist
   met_heatmap <- met_heatmap %>% heatmap(.data = .,
                                           .row = Met_type,
                                           .column = Hugo_Symbol,
                                           .value = Met_count,
                                           cluster_rows = FALSE,
                                           cluster_columns = TRUE,
                                           show_column_dend = FALSE,
                                           show_column_names = TRUE,
                                           scale = "none",
                                           palette_value = c("white", "blue", "darkblue"),
                                           column_title = "Heatmap of methylation state by EpiMix") %>%
      annotation_tile(logFC,
               palette = c("chartreuse4",  "firebrick3"))

    # If genelist contain both oncogenic mediator gene types
    # add gene type annotation to heatmap
    if (driver_groups == 2) {
      met_heatmap <- met_heatmap %>%
        annotation_tile(Moonlight_Oncogenic_Mediator,
                 palette = c("goldenrod2", "dodgerblue3"))
    }

    # Save heatmap
    save_pdf(met_heatmap,
             height = 15,
             width = 35,
             units = "cm",
             filename = paste(additionalFilename,
                              "heatmap_genelist_met.pdf",
                              sep = ""))

  }

  ## Complete mode
  else if (type == "complete") {

    # Heatmap of all genes grouped by oncogenic mediator types
    met_heatmap <- summary_heatmap %>%
      group_by(Moonlight_Oncogenic_Mediator) %>%
      heatmap(.data = .,
              .row = Met_type,
              .column = Hugo_Symbol,
              .value = Met_count,
              cluster_rows = FALSE,
              cluster_columns = TRUE,
              show_column_dend = FALSE,
              show_column_names = TRUE,
              scale = "none",
              palette_value = c("white", "blue", "darkblue"),
              column_title = "Heatmap of methylation state by EpiMix") %>%
      annotation_tile(Moonlight_Oncogenic_Mediator,
               palette = c("goldenrod2", "dodgerblue3")) %>%
      annotation_tile(logFC,
               palette = c("chartreuse4",  "firebrick3"))
      save_pdf(met_heatmap,
               height = 15,
               width = 35,
               units = "cm",
               filename = paste(additionalFilename,
                                "heatmap_complete_met.pdf",
                                sep = ""))

  }

  ## Split mode
  else if(type == "split") {

    if (dir.exists(paste(tempdir(), "/heatmaps", sep = ""))) {
      stop("'heatmaps' folder already exits. Please remove it.")
    }
    else {
      print("A temporary heatmaps folder is created. It will be removed before finalising.")
      dir.create(path = paste(tempdir(), "/heatmaps", sep = ""),
                 showWarnings = TRUE,
                 recursive = TRUE)
    }

    # Create vector of group assignments to be used for assigning genes
    # a group. Each group will be of length 160 corresponding to
    # 4*40 = 160 rows for 40 genes as each gene has 4 rows. Last group
    # will not necessarily be of length 160 depending on number of
    # rows in data.
    split_vector <- rep(seq(1,
                            ceiling(nrow(summary_heatmap) / 160),
                            by = 1),
                        each = 160)
    split_vector <- split_vector[1:nrow(summary_heatmap)]

    # Group data into groups of 40 genes
    # where each nested tibble will be a group of 40 genes
    grouped_data <- summary_heatmap %>%
      arrange(Hugo_Symbol) %>%
      ungroup() %>%
      mutate(gr = split_vector) %>%
      group_by(gr) %>%
      tidyr::nest(data = -gr)

    print("Start creating indivual heatmaps, this can take a while")

    # Create heatmaps for each nested tibble in grouped_data
    # where each heatmap contains 40 genes grouped by oncogenic mediator type
    # Save resulting heatmap as a pdf
    split_heatmaps <- map(1:nrow(grouped_data),
                          function(x) {

      met_heatmap <- grouped_data[[2]][[x]] %>%
        group_by(Moonlight_Oncogenic_Mediator) %>%
        heatmap(.data = .,
                .row = Met_type,
                .column = Hugo_Symbol,
                .value = Met_count,
                cluster_rows = FALSE,
                cluster_columns = FALSE,
                show_column_dend = FALSE,
                show_column_names = TRUE,
                scale = "none",
                palette_value = c("white", "blue", "darkblue"),
                column_title = "Heatmap of methylation state by EpiMix") %>%
        annotation_tile(Moonlight_Oncogenic_Mediator,
                 palette = c("goldenrod2", "dodgerblue3")) %>%
        annotation_tile(logFC,
                 palette = c("chartreuse4",  "firebrick3"))
        save_pdf(met_heatmap,
                 height = 15,
                 width = 35,
                 units = "cm",
                 filename = paste(tempdir(), "/heatmaps/heatmap_",
                                  x,
                                  ".pdf",
                                  sep = ""))

    })

    print("Indivial heatmaps finished, start combining")

    # Get names of saved pdfs containing individual heatmaps
    pdf_names <- map(1:nrow(grouped_data),
                     function(x) {
      pdf_names <- paste(tempdir(), "/heatmaps/heatmap_",
                         x,
                         ".pdf",
                         sep="")
    }) %>%
      unlist

    # Combine individual heatmaps into a single pdf
    pdf_combine(input = pdf_names,
                output = paste(additionalFilename,
                               "heatmaps_split_met.pdf",
                               sep = ""))

    # Remove temporary directory
    unlink(paste(tempdir(), "/heatmaps", sep = ""),
           recursive = TRUE)

  }
}

utils::globalVariables(c("Met_type", "Met_count", "Met_type", "Met_count"))

#' plotMetExp
#' This function visualizes results of EpiMix. 
#' @param EpiMix_results The object, a list, that was returned from
#' running the EpiMix function and is one of the outputs from the 
#' GMA function.
#' @param probe_name A character string containing the name of the
#' CpG probe that will be plotted. 
#' @param dataMET A data matrix containing the methylation data
#' where the CpG probes are in the rows and samples are in the
#' columns
#' @param dataEXP A data matrix containing the gene expression
#' data where the genes are in the rows and the samples are in 
#' the columns
#' @param gene_of_interest A character string containing the 
#' name of the gene that will be plotted. 
#' @param additionalFilename A character string that can be used
#' to add a prefix or filepath to the filename of the pdf 
#' visualizing the heatmap. Default is an empty string. 
#' @importFrom EpiMix EpiMix_PlotModel
#' @importFrom ggplot2 ggsave
#' @return No value is returned. Visualizations are saved. 
#' @export
#' @examples 
#' data("EpiMix_Results_Regular")
#' data("dataMethyl")
#' data("dataFilt")
#' pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
#' colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))
#' plotMetExp(EpiMix_results = EpiMix_Results_Regular,
#'            probe_name = "cg03035704",
#'            dataMET = dataMethyl,
#'            dataEXP = dataFilt,
#'            gene_of_interest = "ACVRL1",
#'            additionalFilename = "./GMAresults/")
plotMetExp <- function(EpiMix_results,
                       probe_name,
                       dataMET,
                       dataEXP,
                       gene_of_interest,
                       additionalFilename = "") {
  
  # Run EpiMix_PlotModel function
  EpiMix_plots <- EpiMix_PlotModel(EpiMixResults = EpiMix_results, 
                                   Probe = probe_name, 
                                   methylation.data = dataMET, 
                                   gene.expression.data = dataEXP, 
                                   GeneName = gene_of_interest)
  
  # Save mixture model plot
  ggsave(plot = EpiMix_plots$MixtureModelPlot, 
         width = 8,
         height = 8,
         filename = paste(additionalFilename,
                          "MixtureModelPlot.pdf", 
                          sep = ""),
         path = ".")
  
  # Save violin plot
  ggsave(plot = EpiMix_plots$ViolinPlot, 
         width = 8,
         height = 8,
         filename = paste(additionalFilename,
                          "ViolinPlot.pdf", 
                          sep = ""),
         path = ".")
  
  # Save correlation plot
  ggsave(plot = EpiMix_plots$CorrelationPlot, 
         width = 8,
         height = 8,
         filename = paste(additionalFilename,
                          "CorrelationPlot.pdf", 
                          sep = ""),
         path = ".")
}

---
title: "A workflow to study mechanistic indicators for driver gene prediction with Moonlight"
author: "Mona Nourbakhsh^+^ ,
        Astrid Saksager^+^,
        Nikola Tom,
        Xi Steven Chen,
        Antonio Colaprico,
        Catharina Olsen,
        Matteo Tiberti,
        Elena Papaleo"
subtitle: ^+^ These authors contributed equally to the paper as first authors.

date: "`r Sys.Date()`"

output:
    BiocStyle::html_document:
        toc: true
        number_sections: false
        toc_depth: 2
        highlight: haddock

vignette: >
  %\VignetteIndexEntry{A workflow to study mechanistic indicators for driver gene prediction with Moonlight}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}

references:
- id: ref1
  title: Orchestrating high-throughput genomic analysis with Bioconductor
  author:
  - family: Huber, Wolfgang and Carey, Vincent J and Gentleman, Robert and Anders, Simon and Carlson, Marc and Carvalho, Benilton S and Bravo, Hector Corrada and Davis, Sean and Gatto, Laurent and Girke, Thomas and others
    given:
  journal: Nature methods
  volume: 12
  number: 2
  pages: 115-121
  issued:
    year: 2015

- id: ref2
  title: GC-content normalization for RNA-Seq data
  author:
  - family: Risso, Davide and Schwartz, Katja and Sherlock, Gavin and Dudoit, Sandrine
    given:
  journal: BMC bioinformatics
  volume: 12
  number: 1
  pages: 480
  issued:
    year: 2011

- id: ref3
  title: Evaluation of statistical methods for normalization and differential expression in mRNA-Seq experiments
  author:
  - family: Bullard, James H and Purdom, Elizabeth and Hansen, Kasper D and Dudoit, Sandrine
    given:
  journal: BMC bioinformatics
  volume: 11
  number: 1
  pages: 94
  issued:
    year: 2010

- id: ref4
  title: Inferring regulatory element landscapes and transcription factor networks from cancer methylomes
  author:
  - family: Yao, L., Shen, H., Laird, P. W., Farnham, P. J., & Berman, B. P.
    given:
  journal: Genome biology
  volume: 16
  number: 1
  pages: 105
  issued:
    year: 2015

- id: ref5
  title: Evaluation of statistical methods for normalization and differential expression in mRNA-Seq experiments
  author:
  - family:  James H Bullard, Elizabeth Purdom, Kasper D Hansen and Sandrine Dudoit
    given:
  journal: BMC Bioinformatics
  volume: 11
  number: 1
  pages: 94
  issued:
    year: 2010

- id: ref6
  title: GC-content normalization for RNA-Seq data
  author:
  - family: Risso, D., Schwartz, K., Sherlock, G., & Dudoit, S.
    given:
  journal: BMC Bioinformatics
  volume: 12
  number: 1
  pages: 480
  issued:
    year: 2011

- id: ref7
  title: Identification of a CpG island methylator phenotype that defines a distinct subgroup of glioma
  author:
  - family: Noushmehr, H., Weisenberger, D.J., Diefes, K., Phillips, H.S., Pujara, K., Berman, B.P., Pan, F., Pelloski, C.E., Sulman, E.P., Bhat, K.P. et al.
    given:
  journal: Cancer cell
  volume: 17
  number: 5
  pages: 510-522
  issued:
    year: 2010

- id: ref8
  title: Molecular Profiling Reveals Biologically Discrete Subsets and Pathways of Progression in Diffuse Glioma
  author:
  - family: Ceccarelli, Michele and Barthel, Floris P and Malta, Tathiane M and Sabedot, Thais S and Salama, Sofie R and Murray, Bradley A and Morozova, Olena and Newton, Yulia and Radenbaugh, Amie and Pagnotta, Stefano M and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2015.12.028"
  DOI: "10.1016/j.cell.2015.12.028"
  volume: 164
  number: 3
  pages: 550-563
  issued:
    year: 2016


- id: ref9
  title: Comprehensive molecular profiling of lung adenocarcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature13385"
  DOI: "10.1038/nature13385"
  volume: 511
  number: 7511
  pages: 543-550
  issued:
    year: 2014


- id: ref10
  title: Comprehensive molecular characterization of gastric adenocarcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature13480"
  DOI: "10.1038/nature13480"
  issued:
    year: 2014

- id: ref11
  title: Comprehensive molecular portraits of human breast tumours
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature11412"
  DOI: "10.1038/nature11412"
  volume: 490
  number: 7418
  pages: 61-70
  issued:
    year: 2012

- id: ref12
  title: Comprehensive molecular characterization of human colon and rectal cancer
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature11252"
  DOI: "10.1038/nature11252"
  volume: 487
  number: 7407
  pages: 330-337
  issued:
    year: 2012

- id: ref13
  title: Genomic classification of cutaneous melanoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2015.05.044"
  DOI: "10.1016/j.cell.2015.05.044"
  volume: 161
  number: 7
  pages: 1681-1696
  issued:
    year: 2015

- id: ref14
  title: Comprehensive genomic characterization of head and neck squamous cell carcinomas
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature14129"
  DOI: "10.1038/nature14129"
  volume: 517
  number: 7536
  pages: 576-582
  issued:
    year: 2015

- id: ref15
  title: The somatic genomic landscape of chromophobe renal cell carcinoma
  author:
  - family: Davis, Caleb F and Ricketts, Christopher J and Wang, Min and Yang, Lixing and Cherniack, Andrew D and Shen, Hui and Buhay, Christian and Kang, Hyojin and Kim, Sang Cheol and Fahey, Catherine C and others
    given:
  journal: Cancer Cell
  URL: "http://doi.org/10.1016/j.ccr.2014.07.014"
  DOI: "10.1016/j.ccr.2014.07.014"
  volume: 26
  number: 3
  pages: 319-330
  issued:
    year: 2014


- id: ref16
  title: Comprehensive genomic characterization of squamous cell lung cancers
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature11404"
  DOI: "10.1038/nature11404"
  volume: 489
  number: 7417
  pages: 519-525
  issued:
    year: 2012

- id: ref17
  title: Integrated genomic characterization of endometrial carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature12113"
  DOI: "10.1038/nature12113"
  volume: 497
  number: 7447
  pages: 67-73
  issued:
    year: 2013

- id: ref18
  title: Integrated genomic characterization of papillary thyroid carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2014.09.050"
  DOI: "10.1016/j.cell.2014.09.050"
  volume: 159
  number: 3
  pages: 676-690
  issued:
    year: 2014

- id: ref19
  title: The molecular taxonomy of primary prostate cancer
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2015.10.025"
  DOI: "10.1016/j.cell.2015.10.025"
  volume: 163
  number: 4
  pages: 1011-1025
  issued:
    year: 2015


- id: ref20
  title: Comprehensive Molecular Characterization of Papillary Renal-Cell Carcinoma
  author:
  - family: Linehan, W Marston and Spellman, Paul T and Ricketts, Christopher J and Creighton, Chad J and Fei, Suzanne S and Davis, Caleb and Wheeler, David A and Murray, Bradley A and Schmidt, Laura and Vocke, Cathy D and others
    given:
  journal: NEW ENGLAND JOURNAL OF MEDICINE
  URL: "http://doi.org/10.1056/NEJMoa1505917"
  DOI: "10.1056/NEJMoa1505917"
  volume: 374
  number: 2
  pages: 135-145
  issued:
    year: 2016

- id: ref21
  title: Comprehensive molecular characterization of clear cell renal cell carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature12222"
  DOI: "10.1038/nature12222"
  volume: 499
  number: 7456
  pages: 43-49
  issued:
    year: 2013

- id: ref22
  title: Comprehensive Pan-Genomic Characterization of Adrenocortical Carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cancer Cell
  URL: "http://dx.doi.org/10.1016/j.ccell.2016.04.002"
  DOI: "10.1016/j.ccell.2016.04.002"
  volume: 29
  pages: 43-49
  issued:
    year: 2016

- id: ref23
  title: Complex heatmaps reveal patterns and correlations in multidimensional genomic data
  author:
  - family: Gu, Zuguang and Eils, Roland and Schlesner, Matthias
  given:
  journal: Bioinformatics
  URL: "http://dx.doi.org/10.1016/j.ccell.2016.04.002"
  DOI: "10.1016/j.ccell.2016.04.002"
  pages: "btw313"
  issued:
   year: 2016

- id: ref24
  title: "TCGA Workflow: Analyze cancer genomics and epigenomics data using Bioconductor packages"
  author:
  - family:  Silva, TC and Colaprico, A and Olsen, C and D'Angelo, F and Bontempi, G and Ceccarelli, M and Noushmehr, H
  given:
  journal: F1000Research
  URL: "http://dx.doi.org/10.12688/f1000research.8923.1"
  DOI: "10.12688/f1000research.8923.1"
  volume: 5
  number: 1542
  issued:
   year: 2016

- id: ref25
  title: "TCGAbiolinks: an R/Bioconductor package for integrative analysis of TCGA data"
  author:
  - family:  Colaprico, Antonio and Silva, Tiago C. and Olsen, Catharina and Garofano, Luciano and Cava, Claudia and Garolini, Davide and Sabedot, Thais S. and Malta, Tathiane M. and Pagnotta, Stefano M. and Castiglioni, Isabella and Ceccarelli, Michele and Bontempi, Gianluca and Noushmehr, Houtan
  given:
  journal: Nucleic Acids Research
  URL: "http://dx.doi.org/10.1093/nar/gkv1507"
  DOI: "10.1093/nar/gkv1507"
  volume: 44
  number: 8
  pages: e71
  issued:
   year: 2016

- id: ref26
  title: "Interpreting pathways to discover cancer driver genes with Moonlight"
  author:
  - family:  Colaprico, Antonio and Olsen, Catharina and Bailey, Matthew H. and Odom, Gabriel J. and Terkelsen, Thilde and Silva, Tiago C. and Olsen, André V. and Cantini, Laura and Zinovyev, Andrei and Barillot, Emmanuel and Noushmehr, Houtan and Bertoli, Gloria and Castiglioni, Isabella and Cava, Claudia and Bontempi, Gianluca and Chen, Xi Steven and Papaleo, Elena
  given:
  journal: Nature Communications
  URL: "https://doi.org/10.1038/s41467-019-13803-0"
  DOI: "10.1038/s41467-019-13803-0"
  volume: 11
  number: 69
  issued:
   year: 2020

- id: ref27
  title: "A workflow to study mechanistic indicators for driver gene prediction with Moonlight"
  author:
  - family:  Nourbakhsh, Mona and Saksager, Astrid and Tom, Nikola and Chen, Xi Steven and Colaprico, Antonio and Olsen, Catharina and Tiberti, Matteo and Papaleo, Elena
  given:
  journal: Briefings in Bioinformatics
  URL: "https://doi.org/10.1093/bib/bbad274"
  DOI: "10.1093/bib/bbad274"
  issued:
   year: 2023

- id: ref28
  title: "EpiMix is an integrative tool for epigenomic subtyping using DNA methylation"
  author:
  - family:  Zheng, Yuanning and Jun, John and Brennan, Kevin and Gevaert, Olivier
  given:
  journal: Cell Reports Methods
  URL: "https://doi.org/10.1016/j.crmeth.2023.100515"
  DOI: "10.1016/j.crmeth.2023.100515"
  issued:
   year: 2023

- id: ref29
  title: "CScape-somatic: distinguishing driver and passenger point mutations in the cancer genome"
  author:
  - family:  Rogers, Mark F and Gaunt, Tom R and Campbell, Colin
  given:
  journal: Bioinformatics
  URL: "https://doi.org/10.1093/bioinformatics/btaa242"
  DOI: "10.1093/bioinformatics/btaa242"
  issued:
   year: 2020
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(dpi = 72)
knitr::opts_chunk$set(cache=FALSE)
```


# Abstract

In order to make light of cancer development, it is crucial to understand which genes play
a role in the mechanisms linked to this disease and moreover which role that is. Commonly
biological processes such as proliferation and apoptosis have been linked to cancer progression.
We have developed the Moonlight framework that allows for prediction of cancer driver genes
through multi-omics data integration. Based on expression data we perform functional enrichment
analysis, infer gene regulatory networks and upstream regulator analysis to score the importance
of well-known biological processes with respect to the studied cancer. We then use these scores
to predict oncogenic mediators with two specific roles: genes that potentially act as tumor
suppressor genes (TSGs) and genes that potentially act as oncogenes (OCGs). This constitutes
Moonlight's primary layer. As gene expression data alone does not explain the cancer phenotypes,
a second layer of evidence is needed. We have integrated three secondary layers, one based on mutations,
one based on abnormal DNA methylation patterns, for prediction of the driver genes,
and one baed on the effect of mutations on transcription factors.
The mutational layer predicts driver mutations in the oncogenic mediators and thereby allows
for the prediction of cancer driver genes using the driver mutation prediction tool CScape-somatic.
The methylation layer investigates abnormal DNA methylation patterns and differentially methylated
CpG sites in the oncogenic mediators using the tool EpiMix and uses patterns of hypo- and
hypermethylation for prediction of the driver genes. The transcription factor layer
checks which transcription factors are important for determining the transcription
of possible driver genes, and identifies mutations on such transcription factors
from a cancer mutation dataset that are able to destabilize them and therefore likely affect transcription.
Overall, this methodology not only allows us to identify genes with dual role (TSG in one
cancer type and OCG in another) but also to elucidate the underlying biological processes.

# Introduction

Cancer development is influenced by (epi)genetic alterations in two distinctly different categories of genes,
known as  tumor suppressor genes (TSG) and oncogenes (OCG). The occurrence of these alterations in genes
of the first category leads to faster cell proliferation while alterations in genes of second
category increases or changes their function.
In 2020, we developed the Moonlight framework that allows for prediction of cancer driver genes
[@ref26]. Here, gene expression data are integrated together with
biological processes and gene regulatory networks to score the importance of well-known biological
processes with respect to the studied cancer. These scores are used to predict oncogenic mediators:
putative TSGs and putative OCGs. As gene expression data alone is not enough to explain the
deregulation of the genes, a second layer of evidence is needed. For this reason, we integrated a
secondary mutational layer which predicts driver mutations and passenger mutations
in the oncogenic mediators. The prediction of the driver mutations are carried out using the
CScape-somatic [@ref29] driver mutation prediction tool. Those oncogenic mediators with at least one
driver mutation are retained as the final set of driver genes [@ref27]. Moreover, we have integrated
a secondary methylation layer that investigates abnormal DNA methylation patterns in the oncogenic
mediators using the tool EpiMix [@ref28]. Those oncogenic mediators demonstrating hypo- and hypermethylation
patterns are predicted as OCGs and TSGs, respectively. These new functionalities are released in the
updated version of Moonlight to produce Moonlight2R.
A third transcription factor layer checks which transcription factors are important
for determining the transcription of possible driver genes, and if any mutation
available in a cancer mutation dataset is able to destabilize them and therefore
likely affect transcription. The mutations can be provided as MAF-formatted files,
while predictions on whether any mutation on a transcription factor can be destabilizing
are available as the [MAVISp database](https://osf.io/ufpzm/). MAVISp is our computational
framework for the effect of predictions of mutations using structural methods; you
can read more about it on our [preprint](https://www.biorxiv.org/content/10.1101/2022.10.22.513328v5).

# Moonlight's pipeline

Moonlight's pipeline is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("Moonlight2_pipeline_upd.png")
```

# Moonlight's proposed workflow

The proposed pipeline consists of following eight steps:

1. The input to Moonlight is a set of Differentially Expressed Genes (DEGs) between two biological conditions
such as cancer and healthy samples or two cancer subtyoes. Besides differentially expressed genes,
gene expression data is also needed. Mutation data and/or methylation data are also needed.
2. **FEA** Functional Enrichment Analysis, using Fisher's test as default method or the Fast Gene Set Enrichment Analysis (fgsea), to identify gene sets (with biological
functions linked to cancer) significantly enriched by regulated genes (RG). The analysis can be performed for both transcriptomic and proteomic dataset.
3. **GRN** Gene Regulatory Network inferred between each single DEG (sDEG) and all genes by means of
mutual information, obtaining for each DEG a list of RG.
4. **URA** Upstream Regulator Analysis for DEGs in each enriched gene set, we applied z-score being
the ratio between the sum of all predicted effects for all the gene involved in the specific function
and the square-root of the number of all genes.
5. **PRA** Pattern Recognition Analysis identifies putative TSGs (down) and OCGs (up). We either use
user defined biological processes or random forests.
6. **DMA** Driver Mutation Analysis identifies driver mutations in the putative TSGs and OCGs through
the driver mutation prediction tool, CScape-somatic.
7. **GMA** Gene Methylation Analysis identifies abnormal DNA methylation patterns in the putative
TSGs and OCGs through the tool, EpiMix.
8. We apply the above procedure to multiple cancer types to obtain cancer-specific lists of TSGs
and OCGs. We compare the lists for each cancer: if a sDEG is TSG in a cancer and OCG in another we
define it as dual-role TSG-OCG. Otherwise if we find a sDEG defined as OCG or TSG only in one tissue
we define it as tissue specific biomarker.
9. We use the COSMIC database to define a list of gold standard TSG and OCGs to assess the accuracy
of the proposed method.


# Installation

To install Moonlight2R use the code below.

## Installation from BioConductor

```{r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("Moonlight2R")
```

## Installation from GitHub

First, install `devtools` or if you already have it installed, load it.
```{r, eval = FALSE}
install.packages("devtools")
library(devtools)
```

Install Moonlight2R from GitHub:
```{r, eval = FALSE}
devtools::install_github(repo = "ELELAB/Moonlight2R")
```

## Installation from GitHub with accompanying vignette

First, install the BiocStyle Bioconductor package.

```{r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("BiocStyle")
```
Then install Moonlight2R with its accompanying vignette.

```{r, eval = FALSE}
devtools::install_github(repo = "ELELAB/Moonlight2R", build_vignettes = TRUE)
```

You can view the vignette in the following way.

```{r, eval = FALSE}
vignette( "Moonlight2R", package="Moonlight2R")
```


# Load libraries
```{r, eval = TRUE}
library(Moonlight2R)
library(magrittr)
library(dplyr)
```


# `Obtain Input`

The input to Moonlight is a set of differentially expressed genes and gene
expression data. Mutation and/or methylation data are also needed. These data
should be available for the same sample sets. Gene expression data, mutation
data, methylation data and differentially expressed genes can for example be obtained from TCGA
using the R package TCGAbiolinks. Help documents on how to use TCGAbiolinks
are available [here](https://bioconductor.org/packages/release/bioc/vignettes/TCGAbiolinks/inst/doc/index.html).
To find other examples of usage of TCGAbiolinks on TCGA cancer types see our
[GitHub repository](https://github.com/ELELAB/LUAD_LUSC_TCGA_comparison).
Example data of the input (differentially expressed genes, gene expression data,
mutation data, and methylation data) are stored in the Moonlight2R package:

```{r, eval = TRUE}
data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(GEO_TCGAtab)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)
data(dataMethyl)
data(DEG_Methylation_Annotations)
data(Oncogenic_mediators_methylation_summary)
data(MetEvidenceDriver)
data(LUAD_sample_anno)
```


# `Download`: Get GEO data

You can search GEO data using the `getDataGEO` function.

GEO_TCGAtab: a 18x12 matrix that provides the GEO data set we matched to one of the 18 given TCGA cancer types

```{r, eval = TRUE, echo = TRUE}
knitr::kable(GEO_TCGAtab, digits = 2,
       caption = "Table with GEO data set matched to one
       of the 18 given TCGA cancer types ",
       row.names = TRUE)
```



## `getDataGEO`: Search by cancer type and data type [Gene Expression]
The user can query and download the cancer types supported by GEO, using the function `getDataGEO`:

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
dataFilt_GEO <- getDataGEO(GEOobject = "GSE20347", platform = "GPL571")
```

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
dataFilt_GEO <- getDataGEO(TCGAtumor = "ESCA")
```


## `FEA`: Functional Enrichment Analysis
The user can perform a functional enrichment analysis using the function `FEA`.
To perform the analysis, the user can choose among two different methods: 
the default method is "ora" and it is based on Fisher's test. It is used to tests whether a gene set is overrepresented in a subset of Differentially Expressed Genes (DEGs) or Differentially Abundant Proteins (DAPs).
The second method is "fgsea" and it is based on Fast Gene Set Enrichment Analysis. It is used to tests whether a gene set is enriched across a fully ranked list of DEGs or DAPs.
When method = "fgsea", the function internally sets a random seed. This ensures that the stochastic permutation process used to calculate NES scores and p-values is reproducible across different runs.
For each DEG or DAP in the gene set a z-score is calculated. This score indicates how the genes act in the gene set.
```{r, eval = TRUE, echo = TRUE, results='hide'}
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
dataFEA <- FEA(DiffMatrix = DEGsmatrix, method="ora")
```

The output can be visualized with a FEA plot.

## `FEAplot`: Functional Enrichment Analysis Plot
The user can plot the result of a functional enrichment analysis using the function `plotFEA`.
A negative z-score indicates that the process' activity is decreased.  A positive z-score
indicates that the process' activity is increased.

```{r, eval = TRUE, echo = TRUE, message=FALSE, results='hide', warning=FALSE}
plotFEA(dataFEA = dataFEA, additionalFilename = "_exampleVignette", height = 10, width = 20)
```

The figure generated by the above code is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("FEAplot.gif")
```


## `GRN`: Gene Regulatory Network
The user can perform a gene regulatory network analysis using the function
`GRN` which infers the network using the parmigene package. For illustrative
purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the example below, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r, eval = TRUE}
data(DEGsmatrix)
data(dataFilt)
dataGRN <- GRN(DEGsmatrix = DEGsmatrix, TFs = sample(rownames(DEGsmatrix), 100), normCounts = dataFilt,
	               nGenesPerm = 5, kNearest = 3, nBoot = 5, DiffGenes = TRUE)
```


## `URA`: Upstream Regulator Analysis
The user can perform upstream regulator analysis using the function `URA`. This function is
applied to each DEG in the enriched gene set and its neighbors in the GRN.

```{r, eval = TRUE, echo = TRUE, results='hide'}
data(dataGRN)
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)

dataFEA <- FEA(DiffMatrix = DEGsmatrix)

BPselected <- dataFEA$Diseases.or.Functions.Annotation[1:5]
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = BPselected,
               nCores=1)
```

## `PRA`: Pattern Regognition Analysis
The user can retrieve putative TSG/OCG using either selected biological processes in
the expert-based approach or a random forest classifier trained on known COSMIC OCGs/TSGs
in the machine learning approach. The user must enter the chosen biological processes
in the `BPname` argument to use the expert-based approach or set `BPname = NULL` to
use the machine learning approach.

```{r, eval = TRUE}
data(dataURA)
data(tabGrowBlock)
data(knownDriverGenes)
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis","proliferation of cells"),
               thres.role = 0)
```

## `DMA`: Driver Mutation Analysis
The user can identify driver mutations with `DMA` in the oncogenic mediators established by `PRA`.
The passenger or driver status is estimated with CScape-somatic [@ref29].
This function will further generate three files: DEG_Mutations_Annotations.rda,
Oncogenic_mediators_mutation_summary.rda and cscape_somatic_output.rda. These will be placed
in the specified results-folder.
The user needs to download two CScape-somatic files in order to run DMA named css_coding.vcf.gz
and css_noncoding.vcf.gz, respectively. These two files can be downloaded at
http://cscape-somatic.biocompute.org.uk/#download. The corresponding .tbi files (css_coding.vcf.gz.tbi
and css_noncoding.vcf.gz.tbi) must also be downloaded and be placed in the same folder.


```{r, eval = FALSE}
data(dataPRA)
data(dataMAF)
data(DEGsmatrix)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)
dataDMA <- DMA(dataMAF = dataMAF,
               dataDEGs = DEGsmatrix,
               dataPRA = dataPRA,
               results_folder = "DMAresults",
               coding_file = "css_coding.vcf.gz",
               noncoding_file = "css_noncoding.vcf.gz")
```

## `GMA`: Gene Methylation Analysis
The user can predict driver genes with `GMA` following prediction of the
oncogenic mediators established by `PRA`. `GMA` predicts driver genes by
investigating abnormal DNA methylation patterns in the oncogenic mediators
using the tool EpiMix [@ref28]. Oncogenic mediators with hypo- and hypermethylation
patterns are predicted as OCGs and TSGs, respectively. This function will
generate these files: DEG_Methylation_Annotations.rda,
Oncogenic_mediators_methylation_summary.rda, EpiMix_Results_Enhancer.rds,
EpiMix_Results_Regular.rds, FunctionalPairs_Enhancer.csv, FunctionalPairs_Regular.csv,
and FunctionalProbes_Regular.rds. These will be placed in the specified results-folder.

```{r, eval = TRUE}
data("dataMethyl")
data("dataFilt")
data("dataPRA")
data("DEGsmatrix")
data("LUAD_sample_anno")
data("NCG")
data("EncodePromoters")
data("MetEvidenceDriver")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)
```

## `GLS`: Gene Literature Search
The user can perform a literature search on driver genes
using the `GLS` function. This function takes as input driver genes,
a query and maximum number of records to retrieve from PubMed.
Standard PubMed syntax can be used in the query. For example, Boolean
operators AND, OR, NOT can be applied and tags such as [AU],
[TITLE/ABSTRACT], [Affiliation] can be used. `GLS` fetches data of
PubMed records matching the specified query and outputs PubMed IDs
matching the query along with doi, title, abstract, year of publication,
mesh terms, and total number of PubMed publications. This is done for each
of the genes supplied in the input.

```{r, eval = TRUE}
genes_query <- "BRCA1"
dataGLS <- GLS(genes = genes_query,
	       query_string = "AND cancer AND driver")
head(dataGLS)
```

## Transcription factor-based layer of secondary evidence
We now introduce the third layer of secondary evidence. It is based on the assumption
that stability changes due to mutations on transcription factors can influence
transcription. Here, we use data on change of stability upon mutation available in the
MAVISp database, as well as data from the [TRRUST database](https://www.grnpedia.org/trrust/)
and MAF files from TCGA to i) identify transcription factors that have an effect
on the transcription of genes of interest ii) identify potentially damaging mutations on them from
MAF files and iii) use MAVISp data to assess the affect of such mutations, whether they
might destabilizing or not. We release the TRRUST-derived data within the Moonlight2R
package with a Creative Commons Attribution-ShareAlike 4.0 International license.

## `loadMAVISp`: Helper function for loading the MAVISp database
The first step to perform this is loading the data form the MAVISp database.

Notice that we have included a very small portion of the MAVISp database with Moonlight2R
for testing purposes; we recommend using the full database available at
the [MAVISp OSF repository](https://osf.io/ufpzm/). Also see the
[MAVISp preprint](https://www.biorxiv.org/content/10.1101/2022.10.22.513328v5) for specifics
on what datasets are available in MAVISp.

The `loadMAVISp` function is used to load the files available from the
MAVISp database. The user will need to decide on whether they want to import the simple or ensemble mode and
in the case of ensemble mode, choose the ensemble they want to import. Furthermore,
the user can specify specific proteins to load. The output is a list of tibbles,
each one correponding to a file in the MAVISp database. The output is used directly
in the `TFinfluence` function.

```{r, eval = TRUE}

mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')

specific_protein <- loadMAVISp(mavispDB = mavisp_db_location,
                               mode = 'simple',
                               proteins_of_interest = c('RUNX1'))

all_proteins <- loadMAVISp(mavispDB = mavisp_db_location,
                           mode = 'simple')

ensemble <- loadMAVISp(mavispDB = mavisp_db_location,
                       mode = 'ensemble',
                       ensemble = 'cabsflex')
```

## `TFinfluence`: Effect of mutations on transcription factors
As previosuly introduceed, the `TFinfluence` function can be used as after the PRA
function, to identify if there are mutations in the transcription factors of the DEGs. The function
uses data from TRRUST to couple the DEGs with their transcription factor (TF).
The mutations in the TFs come from a MAF file, and the effect of the mutation
is obtained from the MAVISp database, available at an [OSF repository](https://osf.io/ufpzm/).

```{r, eval = TRUE}
data(dataPRA)
data(DEGsmatrix)
data(dataTRRUST)
data(dataMAF)
data(dataMAVISp)

TFresults <- TFinfluence(dataTRRUST = dataTRRUST,
            dataMAF = dataMAF,
            dataDEGs = DEGsmatrix,
            dataPRA = dataPRA,
            dataMAVISp = dataMAVISp,
            stabClassMAVISp = 'rasp')
```

## `Level of consequence`: Effect of mutations on three different levels
The user can investigate the predicted effect of different mutation types on the transcriptional
level through the table LOC_transcription:

```{r}
knitr::kable(LOC_transcription)
```

The user can investigate the predicted effect of different mutation types on the translational
level through the table LOC_translation:

```{r}
knitr::kable(LOC_translation)
```

The user can investigate the predicted effect of different mutation types on the protein level
through the table LOC_protein:
```{r}
knitr::kable(LOC_protein)
```


## `plotNetworkHive`: GRN hive visualization taking into account COSMIC cancer genes

In the following plot the nodes are separated into three groups: known tumor suppressor genes (yellow),
known oncogenes (green) and the rest (gray).

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
data(knownDriverGenes)
data(dataGRN)
plotNetworkHive(dataGRN, knownDriverGenes, 0.55)
```


## `plotDMA`: Heatmap of the driver/passenger status of mutations in TSGs/OCGs

In the following plot the driver genes with driver mutations are shown.

```{r, eval = TRUE, warning = FALSE, message = FALSE, include=TRUE}
data(dataDMA)
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
plotDMA(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        type = 'complete', additionalFilename = "")
```


```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("heatmap_complete.gif")
```


## `plotMoonlight`: Heatmap of Moonlight Gene Z-scores for mutation-driven TSGs/OCGs
In the following plot the top 50 genes with the most driver mutations are visualised.
The values are the moonlight gene z-score for the two biological processes

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
data(dataURA_plot)
plotMoonlight(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        dataURA_plot, gene_type = "drivers", n = 50)
```

```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("moonlight_heatmap.gif")
```

## `plotGMA`: Heatmap of hypo/hyper/dual methylated CpG sites in TSGs/OCGs

This function plots results of the `GMA`. It visualizes the
number of hypo/hyper/dual methylated CpG sites in oncogenic
mediators or in a user-supplied gene list. The results are
visualized either in a single heatmap or split into different
ones which is specified in the function's three modes:
split, complete and genelist.

```{r, eval = TRUE, warning = FALSE, message = FALSE, include = TRUE}
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotGMA(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
        Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
        type = "genelist", genelist = genes,
        additionalFilename = "./GMAresults/")
```

```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("heatmap_genelist_met.gif")
```

## `plotMoonlightMet`: Heatmap of Moonlight Gene Z-scores for methylation-driven TSGs/OCGs

This function visualizes the effect of genes on biological processes
and total number of hypo/hyper/dual methylated CpG sites in genes.

```{r, eval = TRUE, warning = FALSE, message = FALSE, include = TRUE}
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
data("dataURA_plot")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotMoonlightMet(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
                 Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
                 dataURA = dataURA_plot,
                 genes = genes,
                 additionalFilename = "./GMAresults/")
```

```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("moonlight_heatmap_met.gif")
```

## `plotMetExp`: Visualize results from EpiMix of expression and methylation in genes

This function visualizes results of EpiMix [@ref28] in three plots: one that
visualizes the distribution of DNA methylation data (MixtureModelPlot), one
that visualizes gene expression levels (ViolinPlot) and one that visualizes
relationship between DNA methylation and gene expression (CorrelationPlot).

```{r, eval = TRUE, warning = FALSE, message = FALSE, include = TRUE}
data("EpiMix_Results_Regular")
data("dataMethyl")
data("dataFilt")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

plotMetExp(EpiMix_results = EpiMix_Results_Regular,
           probe_name = "cg03035704",
           dataMET = dataMethyl,
           dataEXP = dataFilt,
           gene_of_interest = "ACVRL1",
           additionalFilename = "./GMAresults/")
```

```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("plotMetExp.png")
```


# Moonlight Analysis: Case Studies

### Introduction
This vignette shows a complete workflow of the 'Moonlight2R' package.
The code is divided into the following case studies:

* 1. Predicting oncogenic mediators using Moonlight's primary layer
* 2. Moonlight pipeline in one function
* 3. Moonlight with driver mutation analysis
* 4. Moonlight with gene methylation analysis

## Case study n. 1: Predicting oncogenic mediators using Moonlight's primary layer

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5`
and `nBoot = 5` in the call of `GRN` in the following code block, however, we
recommend setting these parameters to `nGenesPerm = 2000` and `nBoot = 400` to
achieve optimal results, as they are set by default in the function arguments.

```{r,eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE, results='hide'}
data(DEGsmatrix)
data(dataFilt)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)

dataFEA <- FEA(DiffMatrix = DEGsmatrix)

dataGRN <- GRN(TFs = sample(rownames(DEGsmatrix), 100),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               nBoot = 5,
	       kNearest = 3)

dataURA <- URA(dataGRN = dataGRN,
              DEGsmatrix = DEGsmatrix,
              BPname = c("apoptosis",
                         "proliferation of cells"))

dataDual <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

oncogenic_mediators <- list("TSG"=names(dataDual$TSG), "OCG"=names(dataDual$OCG))

```


## `plotURA`: Upstream regulatory analysis plot
The user can plot the result of the upstream regulatory analysis using the function `plotURA`.

```{r, eval = TRUE,message=FALSE,warning=FALSE, results='hide'}
data(dataURA)
plotURA(dataURA = dataURA, additionalFilename = "_exampleVignette")
```

The figure resulted from the code above is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("URAplot.gif")
```


## Case study n. 2: Moonlight pipeline in one function

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the following code block, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r,eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE}
data(dataFilt)
data(DEGsmatrix)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)

listMoonlight <- moonlight(dataDEGs = DEGsmatrix,
                           dataFilt = dataFilt,
                           nTF = 100,
                           DiffGenes = TRUE,
			   nGenesPerm = 5,
                           nBoot = 5,
                           BPname = c("apoptosis","proliferation of cells"),
                           dataMAF = dataMAF,
                           path_cscape_coding = "css_coding.vcf.gz",
                           path_cscape_noncoding = "css_noncoding.vcf.gz")
save(listMoonlight, file = paste0("listMoonlight_ncancer4.Rdata"))


```

## `plotCircos`: Moonlight Circos Plot
An example of running Moonlight on five cancer types is visualized below in a circos plot.
Outer ring: color by cancer type, Inner ring: OCGs and TSGs,
Inner connections:  green: common OCGs yellow: common TSGs red: possible dual role

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
data(listMoonlight)
plotCircos(listMoonlight = listMoonlight, additionalFilename = "_ncancer5")
```

The figure generated by the code above is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("circos_ocg_tsg_ncancer5.gif")
```


## Case study n. 3: Moonlight with driver mutation analysis

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the call of `GRN` in the following code block, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r,eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE}

data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)

# Perform gene regulatory network analysis
dataGRN <- GRN(TFs = rownames(DEGsmatrix),
	       DEGsmatrix = DEGsmatrix,
	       DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               kNearest = 3,
               nBoot = 5)

# Perform upstream regulatory analysis
# As example, we use apoptosis and proliferation of cells as the biological processes
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               nCores = 1)

# Perform pattern recognition analysis
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

# Perform driver mutation analysis
dataDMA <- DMA(dataMAF = dataMAF,
               dataDEGs = DEGsmatrix,
               dataPRA = dataPRA,
               results_folder = "DMAresults",
               coding_file = "css_coding.vcf.gz",
               noncoding_file = "css_noncoding.vcf.gz")

```

Next, we analyze the predicted driver genes and their mutations.

```{r, eval = TRUE}
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)

# Extract oncogenic mediators that contain at least one driver mutation
# These are the driver genes
knitr::kable(Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)))

# Extract mutation annotations of the predicted driver genes
driver_mut <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver")

# Extract driver genes with a predicted effect on the transcriptional level
transcription_mut <- Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)) %>%
  filter(Transcription_mut_sum > 0)

# Extract mutation annotations of predicted driver genes that have a driver mutation
# in its promoter region with a potential effect on the transcriptional level
promoters <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver",
         Potential_Effect_on_Transcription == 1,
         Annotation == 'Promoter')
```


## Case study n. 4: Moonlight with gene methylation analysis

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the call of `GRN` in the following code block, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r,eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE,results='hide'}

data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)
data(dataMethyl)
data(LUAD_sample_anno)
data(MetEvidenceDriver)

# Perform gene regulatory network analysis
dataGRN <- GRN(TFs = rownames(DEGsmatrix),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               kNearest = 3,
               nBoot = 5)

# Perform upstream regulatory analysis
# As example, we use apoptosis and proliferation of cells as the biological processes
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               nCores = 1)

# Perform pattern recognition analysis
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

# Perform gene methylation analysis
dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1,
               roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)

```


## Citation

Please cite the MoonlightR and Moonlight2R packages:

* "Interpreting pathways to discover cancer driver genes with Moonlight." Nature Communications (2020): [10.1038/s41467-019-13803-0](https://doi.org/10.1038/s41467-019-13803-0). [@ref26]

* "A workflow to study mechanistic indicators for driver gene prediction with Moonlight." Briefings in Bioinformatics (2023): [10.1093/bib/bbad274](https://doi.org/10.1093/bib/bbad274). [@ref27]

Related publications to this vignette:

* "TCGAbiolinks: an R/Bioconductor package for integrative analysis of TCGA data." Nucleic acids research (2015): [gkv1507](http://dx.doi.org/doi:10.1093/nar/gkv1507). [@ref25]

* "TCGA Workflow: Analyze cancer genomics and epigenomics data using Bioconductor packages". F1000Research [10.12688/f1000research.8923.1](http://dx.doi.org/doi:10.12688/f1000research.8923.1) [@ref24]


******

Session Information
******
```{r sessionInfo}
sessionInfo()
```

# References

---
title: "A workflow to study mechanistic indicators for driver gene prediction with Moonlight"
author: "Mona Nourbakhsh^+^ ,
        Astrid Saksager^+^,
        Nikola Tom,
        Xi Steven Chen,
        Antonio Colaprico,
        Catharina Olsen,
        Matteo Tiberti,
        Elena Papaleo"
subtitle: ^+^ These authors contributed equally to the paper as first authors.

date: "`r Sys.Date()`"

output:
    BiocStyle::html_document:
        toc: true
        number_sections: false
        toc_depth: 2
        highlight: haddock

vignette: >
  %\VignetteIndexEntry{A workflow to study mechanistic indicators for driver gene prediction with Moonlight}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}

references:
- id: ref1
  title: Orchestrating high-throughput genomic analysis with Bioconductor
  author:
  - family: Huber, Wolfgang and Carey, Vincent J and Gentleman, Robert and Anders, Simon and Carlson, Marc and Carvalho, Benilton S and Bravo, Hector Corrada and Davis, Sean and Gatto, Laurent and Girke, Thomas and others
    given:
  journal: Nature methods
  volume: 12
  number: 2
  pages: 115-121
  issued:
    year: 2015

- id: ref2
  title: GC-content normalization for RNA-Seq data
  author:
  - family: Risso, Davide and Schwartz, Katja and Sherlock, Gavin and Dudoit, Sandrine
    given:
  journal: BMC bioinformatics
  volume: 12
  number: 1
  pages: 480
  issued:
    year: 2011

- id: ref3
  title: Evaluation of statistical methods for normalization and differential expression in mRNA-Seq experiments
  author:
  - family: Bullard, James H and Purdom, Elizabeth and Hansen, Kasper D and Dudoit, Sandrine
    given:
  journal: BMC bioinformatics
  volume: 11
  number: 1
  pages: 94
  issued:
    year: 2010

- id: ref4
  title: Inferring regulatory element landscapes and transcription factor networks from cancer methylomes
  author:
  - family: Yao, L., Shen, H., Laird, P. W., Farnham, P. J., & Berman, B. P.
    given:
  journal: Genome biology
  volume: 16
  number: 1
  pages: 105
  issued:
    year: 2015

- id: ref5
  title: Evaluation of statistical methods for normalization and differential expression in mRNA-Seq experiments
  author:
  - family:  James H Bullard, Elizabeth Purdom, Kasper D Hansen and Sandrine Dudoit
    given:
  journal: BMC Bioinformatics
  volume: 11
  number: 1
  pages: 94
  issued:
    year: 2010

- id: ref6
  title: GC-content normalization for RNA-Seq data
  author:
  - family: Risso, D., Schwartz, K., Sherlock, G., & Dudoit, S.
    given:
  journal: BMC Bioinformatics
  volume: 12
  number: 1
  pages: 480
  issued:
    year: 2011

- id: ref7
  title: Identification of a CpG island methylator phenotype that defines a distinct subgroup of glioma
  author:
  - family: Noushmehr, H., Weisenberger, D.J., Diefes, K., Phillips, H.S., Pujara, K., Berman, B.P., Pan, F., Pelloski, C.E., Sulman, E.P., Bhat, K.P. et al.
    given:
  journal: Cancer cell
  volume: 17
  number: 5
  pages: 510-522
  issued:
    year: 2010

- id: ref8
  title: Molecular Profiling Reveals Biologically Discrete Subsets and Pathways of Progression in Diffuse Glioma
  author:
  - family: Ceccarelli, Michele and Barthel, Floris P and Malta, Tathiane M and Sabedot, Thais S and Salama, Sofie R and Murray, Bradley A and Morozova, Olena and Newton, Yulia and Radenbaugh, Amie and Pagnotta, Stefano M and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2015.12.028"
  DOI: "10.1016/j.cell.2015.12.028"
  volume: 164
  number: 3
  pages: 550-563
  issued:
    year: 2016


- id: ref9
  title: Comprehensive molecular profiling of lung adenocarcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature13385"
  DOI: "10.1038/nature13385"
  volume: 511
  number: 7511
  pages: 543-550
  issued:
    year: 2014


- id: ref10
  title: Comprehensive molecular characterization of gastric adenocarcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature13480"
  DOI: "10.1038/nature13480"
  issued:
    year: 2014

- id: ref11
  title: Comprehensive molecular portraits of human breast tumours
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature11412"
  DOI: "10.1038/nature11412"
  volume: 490
  number: 7418
  pages: 61-70
  issued:
    year: 2012

- id: ref12
  title: Comprehensive molecular characterization of human colon and rectal cancer
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature11252"
  DOI: "10.1038/nature11252"
  volume: 487
  number: 7407
  pages: 330-337
  issued:
    year: 2012

- id: ref13
  title: Genomic classification of cutaneous melanoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2015.05.044"
  DOI: "10.1016/j.cell.2015.05.044"
  volume: 161
  number: 7
  pages: 1681-1696
  issued:
    year: 2015

- id: ref14
  title: Comprehensive genomic characterization of head and neck squamous cell carcinomas
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature14129"
  DOI: "10.1038/nature14129"
  volume: 517
  number: 7536
  pages: 576-582
  issued:
    year: 2015

- id: ref15
  title: The somatic genomic landscape of chromophobe renal cell carcinoma
  author:
  - family: Davis, Caleb F and Ricketts, Christopher J and Wang, Min and Yang, Lixing and Cherniack, Andrew D and Shen, Hui and Buhay, Christian and Kang, Hyojin and Kim, Sang Cheol and Fahey, Catherine C and others
    given:
  journal: Cancer Cell
  URL: "http://doi.org/10.1016/j.ccr.2014.07.014"
  DOI: "10.1016/j.ccr.2014.07.014"
  volume: 26
  number: 3
  pages: 319-330
  issued:
    year: 2014


- id: ref16
  title: Comprehensive genomic characterization of squamous cell lung cancers
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature11404"
  DOI: "10.1038/nature11404"
  volume: 489
  number: 7417
  pages: 519-525
  issued:
    year: 2012

- id: ref17
  title: Integrated genomic characterization of endometrial carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature12113"
  DOI: "10.1038/nature12113"
  volume: 497
  number: 7447
  pages: 67-73
  issued:
    year: 2013

- id: ref18
  title: Integrated genomic characterization of papillary thyroid carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2014.09.050"
  DOI: "10.1016/j.cell.2014.09.050"
  volume: 159
  number: 3
  pages: 676-690
  issued:
    year: 2014

- id: ref19
  title: The molecular taxonomy of primary prostate cancer
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cell
  URL: "http://doi.org/10.1016/j.cell.2015.10.025"
  DOI: "10.1016/j.cell.2015.10.025"
  volume: 163
  number: 4
  pages: 1011-1025
  issued:
    year: 2015


- id: ref20
  title: Comprehensive Molecular Characterization of Papillary Renal-Cell Carcinoma
  author:
  - family: Linehan, W Marston and Spellman, Paul T and Ricketts, Christopher J and Creighton, Chad J and Fei, Suzanne S and Davis, Caleb and Wheeler, David A and Murray, Bradley A and Schmidt, Laura and Vocke, Cathy D and others
    given:
  journal: NEW ENGLAND JOURNAL OF MEDICINE
  URL: "http://doi.org/10.1056/NEJMoa1505917"
  DOI: "10.1056/NEJMoa1505917"
  volume: 374
  number: 2
  pages: 135-145
  issued:
    year: 2016

- id: ref21
  title: Comprehensive molecular characterization of clear cell renal cell carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Nature
  URL: "http://doi.org/10.1038/nature12222"
  DOI: "10.1038/nature12222"
  volume: 499
  number: 7456
  pages: 43-49
  issued:
    year: 2013

- id: ref22
  title: Comprehensive Pan-Genomic Characterization of Adrenocortical Carcinoma
  author:
  - family: Cancer Genome Atlas Research Network and others
    given:
  journal: Cancer Cell
  URL: "http://dx.doi.org/10.1016/j.ccell.2016.04.002"
  DOI: "10.1016/j.ccell.2016.04.002"
  volume: 29
  pages: 43-49
  issued:
    year: 2016

- id: ref23
  title: Complex heatmaps reveal patterns and correlations in multidimensional genomic data
  author:
  - family: Gu, Zuguang and Eils, Roland and Schlesner, Matthias
  given:
  journal: Bioinformatics
  URL: "http://dx.doi.org/10.1016/j.ccell.2016.04.002"
  DOI: "10.1016/j.ccell.2016.04.002"
  pages: "btw313"
  issued:
   year: 2016

- id: ref24
  title: "TCGA Workflow: Analyze cancer genomics and epigenomics data using Bioconductor packages"
  author:
  - family:  Silva, TC and Colaprico, A and Olsen, C and D'Angelo, F and Bontempi, G and Ceccarelli, M and Noushmehr, H
  given:
  journal: F1000Research
  URL: "http://dx.doi.org/10.12688/f1000research.8923.1"
  DOI: "10.12688/f1000research.8923.1"
  volume: 5
  number: 1542
  issued:
   year: 2016

- id: ref25
  title: "TCGAbiolinks: an R/Bioconductor package for integrative analysis of TCGA data"
  author:
  - family:  Colaprico, Antonio and Silva, Tiago C. and Olsen, Catharina and Garofano, Luciano and Cava, Claudia and Garolini, Davide and Sabedot, Thais S. and Malta, Tathiane M. and Pagnotta, Stefano M. and Castiglioni, Isabella and Ceccarelli, Michele and Bontempi, Gianluca and Noushmehr, Houtan
  given:
  journal: Nucleic Acids Research
  URL: "http://dx.doi.org/10.1093/nar/gkv1507"
  DOI: "10.1093/nar/gkv1507"
  volume: 44
  number: 8
  pages: e71
  issued:
   year: 2016

- id: ref26
  title: "Interpreting pathways to discover cancer driver genes with Moonlight"
  author:
  - family:  Colaprico, Antonio and Olsen, Catharina and Bailey, Matthew H. and Odom, Gabriel J. and Terkelsen, Thilde and Silva, Tiago C. and Olsen, André V. and Cantini, Laura and Zinovyev, Andrei and Barillot, Emmanuel and Noushmehr, Houtan and Bertoli, Gloria and Castiglioni, Isabella and Cava, Claudia and Bontempi, Gianluca and Chen, Xi Steven and Papaleo, Elena
  given:
  journal: Nature Communications
  URL: "https://doi.org/10.1038/s41467-019-13803-0"
  DOI: "10.1038/s41467-019-13803-0"
  volume: 11
  number: 69
  issued:
   year: 2020

- id: ref27
  title: "A workflow to study mechanistic indicators for driver gene prediction with Moonlight"
  author:
  - family:  Nourbakhsh, Mona and Saksager, Astrid and Tom, Nikola and Chen, Xi Steven and Colaprico, Antonio and Olsen, Catharina and Tiberti, Matteo and Papaleo, Elena
  given:
  journal: Briefings in Bioinformatics
  URL: "https://doi.org/10.1093/bib/bbad274"
  DOI: "10.1093/bib/bbad274"
  issued:
   year: 2023

- id: ref28
  title: "EpiMix is an integrative tool for epigenomic subtyping using DNA methylation"
  author:
  - family:  Zheng, Yuanning and Jun, John and Brennan, Kevin and Gevaert, Olivier
  given:
  journal: Cell Reports Methods
  URL: "https://doi.org/10.1016/j.crmeth.2023.100515"
  DOI: "10.1016/j.crmeth.2023.100515"
  issued:
   year: 2023

- id: ref29
  title: "CScape-somatic: distinguishing driver and passenger point mutations in the cancer genome"
  author:
  - family:  Rogers, Mark F and Gaunt, Tom R and Campbell, Colin
  given:
  journal: Bioinformatics
  URL: "https://doi.org/10.1093/bioinformatics/btaa242"
  DOI: "10.1093/bioinformatics/btaa242"
  issued:
   year: 2020
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(dpi = 72)
knitr::opts_chunk$set(cache=FALSE)
```


# Abstract

In order to make light of cancer development, it is crucial to understand which genes play
a role in the mechanisms linked to this disease and moreover which role that is. Commonly
biological processes such as proliferation and apoptosis have been linked to cancer progression.
We have developed the Moonlight framework that allows for prediction of cancer driver genes
through multi-omics data integration. Based on expression data we perform functional enrichment
analysis, infer gene regulatory networks and upstream regulator analysis to score the importance
of well-known biological processes with respect to the studied cancer. We then use these scores
to predict oncogenic mediators with two specific roles: genes that potentially act as tumor
suppressor genes (TSGs) and genes that potentially act as oncogenes (OCGs). This constitutes
Moonlight's primary layer. As gene expression data alone does not explain the cancer phenotypes,
a second layer of evidence is needed. We have integrated three secondary layers, one based on mutations,
one based on abnormal DNA methylation patterns, for prediction of the driver genes,
and one baed on the effect of mutations on transcription factors.
The mutational layer predicts driver mutations in the oncogenic mediators and thereby allows
for the prediction of cancer driver genes using the driver mutation prediction tool CScape-somatic.
The methylation layer investigates abnormal DNA methylation patterns and differentially methylated
CpG sites in the oncogenic mediators using the tool EpiMix and uses patterns of hypo- and
hypermethylation for prediction of the driver genes. The transcription factor layer
checks which transcription factors are important for determining the transcription
of possible driver genes, and identifies mutations on such transcription factors
from a cancer mutation dataset that are able to destabilize them and therefore likely affect transcription.
Overall, this methodology not only allows us to identify genes with dual role (TSG in one
cancer type and OCG in another) but also to elucidate the underlying biological processes.

# Introduction

Cancer development is influenced by (epi)genetic alterations in two distinctly different categories of genes,
known as  tumor suppressor genes (TSG) and oncogenes (OCG). The occurrence of these alterations in genes
of the first category leads to faster cell proliferation while alterations in genes of second
category increases or changes their function.
In 2020, we developed the Moonlight framework that allows for prediction of cancer driver genes
[@ref26]. Here, gene expression data are integrated together with
biological processes and gene regulatory networks to score the importance of well-known biological
processes with respect to the studied cancer. These scores are used to predict oncogenic mediators:
putative TSGs and putative OCGs. As gene expression data alone is not enough to explain the
deregulation of the genes, a second layer of evidence is needed. For this reason, we integrated a
secondary mutational layer which predicts driver mutations and passenger mutations
in the oncogenic mediators. The prediction of the driver mutations are carried out using the
CScape-somatic [@ref29] driver mutation prediction tool. Those oncogenic mediators with at least one
driver mutation are retained as the final set of driver genes [@ref27]. Moreover, we have integrated
a secondary methylation layer that investigates abnormal DNA methylation patterns in the oncogenic
mediators using the tool EpiMix [@ref28]. Those oncogenic mediators demonstrating hypo- and hypermethylation
patterns are predicted as OCGs and TSGs, respectively. These new functionalities are released in the
updated version of Moonlight to produce Moonlight2R.
A third transcription factor layer checks which transcription factors are important
for determining the transcription of possible driver genes, and if any mutation
available in a cancer mutation dataset is able to destabilize them and therefore
likely affect transcription. The mutations can be provided as MAF-formatted files,
while predictions on whether any mutation on a transcription factor can be destabilizing
are available as the [MAVISp database](https://osf.io/ufpzm/). MAVISp is our computational
framework for the effect of predictions of mutations using structural methods; you
can read more about it on our [preprint](https://www.biorxiv.org/content/10.1101/2022.10.22.513328v5).

# Moonlight's pipeline

Moonlight's pipeline is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("Moonlight2_pipeline_upd.png")
```

# Moonlight's proposed workflow

The proposed pipeline consists of following eight steps:

1. The input to Moonlight is a set of differentially expressed genes between two biological conditions
such as cancer and healthy samples or two cancer subtyoes. Besides differentially expressed genes,
gene expression data is also needed. Mutation data and/or methylation data are also needed.
2. **FEA** Functional Enrichment Analysis, using Fisher's test, to identify gene sets (with biological
functions linked to cancer) significantly enriched by regulated genes (RG).
3. **GRN** Gene Regulatory Network inferred between each single DEG (sDEG) and all genes by means of
mutual information, obtaining for each DEG a list of RG.
4. **URA** Upstream Regulator Analysis for DEGs in each enriched gene set, we applied z-score being
the ratio between the sum of all predicted effects for all the gene involved in the specific function
and the square-root of the number of all genes.
5. **PRA** Pattern Recognition Analysis identifies putative TSGs (down) and OCGs (up). We either use
user defined biological processes or random forests.
6. **DMA** Driver Mutation Analysis identifies driver mutations in the putative TSGs and OCGs through
the driver mutation prediction tool, CScape-somatic.
7. **GMA** Gene Methylation Analysis identifies abnormal DNA methylation patterns in the putative
TSGs and OCGs through the tool, EpiMix.
8. We apply the above procedure to multiple cancer types to obtain cancer-specific lists of TSGs
and OCGs. We compare the lists for each cancer: if a sDEG is TSG in a cancer and OCG in another we
define it as dual-role TSG-OCG. Otherwise if we find a sDEG defined as OCG or TSG only in one tissue
we define it as tissue specific biomarker.
9. We use the COSMIC database to define a list of gold standard TSG and OCGs to assess the accuracy
of the proposed method.


# Installation

To install Moonlight2R use the code below.

## Installation from BioConductor

```{r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("Moonlight2R")
```

## Installation from GitHub

First, install `devtools` or if you already have it installed, load it.
```{r, eval = FALSE}
install.packages("devtools")
library(devtools)
```

Install Moonlight2R from GitHub:
```{r, eval = FALSE}
devtools::install_github(repo = "ELELAB/Moonlight2R")
```

## Installation from GitHub with accompanying vignette

First, install the BiocStyle Bioconductor package.

```{r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("BiocStyle")
```
Then install Moonlight2R with its accompanying vignette.

```{r, eval = FALSE}
devtools::install_github(repo = "ELELAB/Moonlight2R", build_vignettes = TRUE)
```

You can view the vignette in the following way.

```{r, eval = FALSE}
vignette( "Moonlight2R", package="Moonlight2R")
```


# Load libraries
```{r, eval = TRUE}
library(Moonlight2R)
library(magrittr)
library(dplyr)
```


# `Obtain Input`

The input to Moonlight is a set of differentially expressed genes and gene
expression data. Mutation and/or methylation data are also needed. These data
should be available for the same sample sets. Gene expression data, mutation
data, methylation data and differentially expressed genes can for example be obtained from TCGA
using the R package TCGAbiolinks. Help documents on how to use TCGAbiolinks
are available [here](https://bioconductor.org/packages/release/bioc/vignettes/TCGAbiolinks/inst/doc/index.html).
To find other examples of usage of TCGAbiolinks on TCGA cancer types see our
[GitHub repository](https://github.com/ELELAB/LUAD_LUSC_TCGA_comparison).
Example data of the input (differentially expressed genes, gene expression data,
mutation data, and methylation data) are stored in the Moonlight2R package:

```{r, eval = TRUE}
data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(GEO_TCGAtab)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)
data(dataMethyl)
data(DEG_Methylation_Annotations)
data(Oncogenic_mediators_methylation_summary)
data(MetEvidenceDriver)
data(LUAD_sample_anno)
```


# `Download`: Get GEO data

You can search GEO data using the `getDataGEO` function.

GEO_TCGAtab: a 18x12 matrix that provides the GEO data set we matched to one of the 18 given TCGA cancer types

```{r, eval = TRUE, echo = TRUE}
knitr::kable(GEO_TCGAtab, digits = 2,
       caption = "Table with GEO data set matched to one
       of the 18 given TCGA cancer types ",
       row.names = TRUE)
```



## `getDataGEO`: Search by cancer type and data type [Gene Expression]
The user can query and download the cancer types supported by GEO, using the function `getDataGEO`:

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
dataFilt_GEO <- getDataGEO(GEOobject = "GSE20347", platform = "GPL571")
```

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
dataFilt_GEO <- getDataGEO(TCGAtumor = "ESCA")
```


## `FEA`: Functional Enrichment Analysis
The user can perform a functional enrichment analysis using the function `FEA`.
For each DEG in the gene set a z-score is calculated. This score indicates how the genes act in the gene set.

```{r, eval = TRUE, echo = TRUE, results='hide'}
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
dataFEA <- FEA(DEGsmatrix = DEGsmatrix)
```

The output can be visualized with a FEA plot.

## `FEAplot`: Functional Enrichment Analysis Plot
The user can plot the result of a functional enrichment analysis using the function `plotFEA`.
A negative z-score indicates that the process' activity is decreased.  A positive z-score
indicates that the process' activity is increased.

```{r, eval = TRUE, echo = TRUE, message=FALSE, results='hide', warning=FALSE}
plotFEA(dataFEA = dataFEA, additionalFilename = "_exampleVignette", height = 10, width = 20)
```

The figure generated by the above code is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("FEAplot.gif")
```


## `GRN`: Gene Regulatory Network
The user can perform a gene regulatory network analysis using the function
`GRN` which infers the network using the parmigene package. For illustrative
purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the example below, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r, eval = TRUE}
data(DEGsmatrix)
data(dataFilt)
dataGRN <- GRN(DEGsmatrix = DEGsmatrix, TFs = sample(rownames(DEGsmatrix), 100), normCounts = dataFilt,
	               nGenesPerm = 5, kNearest = 3, nBoot = 5, DiffGenes = TRUE)
```


## `URA`: Upstream Regulator Analysis
The user can perform upstream regulator analysis using the function `URA`. This function is
applied to each DEG in the enriched gene set and its neighbors in the GRN.

```{r, eval = TRUE, echo = TRUE, results='hide'}
data(dataGRN)
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)

dataFEA <- FEA(DEGsmatrix = DEGsmatrix)

BPselected <- dataFEA$Diseases.or.Functions.Annotation[1:5]
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = BPselected,
               nCores=1)
```

## `PRA`: Pattern Regognition Analysis
The user can retrieve putative TSG/OCG using either selected biological processes in
the expert-based approach or a random forest classifier trained on known COSMIC OCGs/TSGs
in the machine learning approach. The user must enter the chosen biological processes
in the `BPname` argument to use the expert-based approach or set `BPname = NULL` to
use the machine learning approach.

```{r, eval = TRUE}
data(dataURA)
data(tabGrowBlock)
data(knownDriverGenes)
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis","proliferation of cells"),
               thres.role = 0)
```

## `DMA`: Driver Mutation Analysis
The user can identify driver mutations with `DMA` in the oncogenic mediators established by `PRA`.
The passenger or driver status is estimated with CScape-somatic [@ref29].
This function will further generate three files: DEG_Mutations_Annotations.rda,
Oncogenic_mediators_mutation_summary.rda and cscape_somatic_output.rda. These will be placed
in the specified results-folder.
The user needs to download two CScape-somatic files in order to run DMA named css_coding.vcf.gz
and css_noncoding.vcf.gz, respectively. These two files can be downloaded at
http://cscape-somatic.biocompute.org.uk/#download. The corresponding .tbi files (css_coding.vcf.gz.tbi
and css_noncoding.vcf.gz.tbi) must also be downloaded and be placed in the same folder.


```{r, eval = FALSE}
data(dataPRA)
data(dataMAF)
data(DEGsmatrix)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)
dataDMA <- DMA(dataMAF = dataMAF,
               dataDEGs = DEGsmatrix,
               dataPRA = dataPRA,
               results_folder = "DMAresults",
               coding_file = "css_coding.vcf.gz",
               noncoding_file = "css_noncoding.vcf.gz")
```

## `GMA`: Gene Methylation Analysis
The user can predict driver genes with `GMA` following prediction of the
oncogenic mediators established by `PRA`. `GMA` predicts driver genes by
investigating abnormal DNA methylation patterns in the oncogenic mediators
using the tool EpiMix [@ref28]. Oncogenic mediators with hypo- and hypermethylation
patterns are predicted as OCGs and TSGs, respectively. This function will
generate these files: DEG_Methylation_Annotations.rda,
Oncogenic_mediators_methylation_summary.rda, EpiMix_Results_Enhancer.rds,
EpiMix_Results_Regular.rds, FunctionalPairs_Enhancer.csv, FunctionalPairs_Regular.csv,
and FunctionalProbes_Regular.rds. These will be placed in the specified results-folder.

```{r, eval = TRUE}
data("dataMethyl")
data("dataFilt")
data("dataPRA")
data("DEGsmatrix")
data("LUAD_sample_anno")
data("NCG")
data("EncodePromoters")
data("MetEvidenceDriver")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)
```

## `GLS`: Gene Literature Search
The user can perform a literature search on driver genes
using the `GLS` function. This function takes as input driver genes,
a query and maximum number of records to retrieve from PubMed.
Standard PubMed syntax can be used in the query. For example, Boolean
operators AND, OR, NOT can be applied and tags such as [AU],
[TITLE/ABSTRACT], [Affiliation] can be used. `GLS` fetches data of
PubMed records matching the specified query and outputs PubMed IDs
matching the query along with doi, title, abstract, year of publication,
mesh terms, and total number of PubMed publications. This is done for each
of the genes supplied in the input.

```{r, eval = TRUE}
genes_query <- "BRCA1"
dataGLS <- GLS(genes = genes_query,
	       query_string = "AND cancer AND driver")
head(dataGLS)
```

## Transcription factor-based layer of secondary evidence
We now introduce the third layer of secondary evidence. It is based on the assumption
that stability changes due to mutations on transcription factors can influence
transcription. Here, we use data on change of stability upon mutation available in the
MAVISp database, as well as data from the [TRRUST database](https://www.grnpedia.org/trrust/)
and MAF files from TCGA to i) identify transcription factors that have an effect
on the transcription of genes of interest ii) identify potentially damaging mutations on them from
MAF files and iii) use MAVISp data to assess the affect of such mutations, whether they
might destabilizing or not. We release the TRRUST-derived data within the Moonlight2R
package with a Creative Commons Attribution-ShareAlike 4.0 International license.

## `loadMAVISp`: Helper function for loading the MAVISp database
The first step to perform this is loading the data form the MAVISp database.

Notice that we have included a very small portion of the MAVISp database with Moonlight2R
for testing purposes; we recommend using the full database available at
the [MAVISp OSF repository](https://osf.io/ufpzm/). Also see the
[MAVISp preprint](https://www.biorxiv.org/content/10.1101/2022.10.22.513328v5) for specifics
on what datasets are available in MAVISp.

The `loadMAVISp` function is used to load the files available from the
MAVISp database. The user will need to decide on whether they want to import the simple or ensemble mode and
in the case of ensemble mode, choose the ensemble they want to import. Furthermore,
the user can specify specific proteins to load. The output is a list of tibbles,
each one correponding to a file in the MAVISp database. The output is used directly
in the `TFinfluence` function.

```{r, eval = TRUE}

mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')

specific_protein <- loadMAVISp(mavispDB = mavisp_db_location,
                               mode = 'simple',
                               proteins_of_interest = c('RUNX1'))

all_proteins <- loadMAVISp(mavispDB = mavisp_db_location,
                           mode = 'simple')

ensemble <- loadMAVISp(mavispDB = mavisp_db_location,
                       mode = 'ensemble',
                       ensemble = 'cabsflex')
```

## `TFinfluence`: Effect of mutations on transcription factors
As previosuly introduceed, the `TFinfluence` function can be used as after the PRA
function, to identify if there are mutations in the transcription factors of the DEGs. The function
uses data from TRRUST to couple the DEGs with their transcription factor (TF).
The mutations in the TFs come from a MAF file, and the effect of the mutation
is obtained from the MAVISp database, available at an [OSF repository](https://osf.io/ufpzm/).

```{r, eval = TRUE}
data(dataPRA)
data(DEGsmatrix)
data(dataTRRUST)
data(dataMAF)
data(dataMAVISp)

TFresults <- TFinfluence(dataTRRUST = dataTRRUST,
            dataMAF = dataMAF,
            dataDEGs = DEGsmatrix,
            dataPRA = dataPRA,
            dataMAVISp = dataMAVISp,
            stabClassMAVISp = 'rasp')
```

## `Level of consequence`: Effect of mutations on three different levels
The user can investigate the predicted effect of different mutation types on the transcriptional
level through the table LOC_transcription:

```{r}
knitr::kable(LOC_transcription)
```

The user can investigate the predicted effect of different mutation types on the translational
level through the table LOC_translation:

```{r}
knitr::kable(LOC_translation)
```

The user can investigate the predicted effect of different mutation types on the protein level
through the table LOC_protein:
```{r}
knitr::kable(LOC_protein)
```


## `plotNetworkHive`: GRN hive visualization taking into account COSMIC cancer genes

In the following plot the nodes are separated into three groups: known tumor suppressor genes (yellow),
known oncogenes (green) and the rest (gray).

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
data(knownDriverGenes)
data(dataGRN)
plotNetworkHive(dataGRN, knownDriverGenes, 0.55)
```


## `plotDMA`: Heatmap of the driver/passenger status of mutations in TSGs/OCGs

In the following plot the driver genes with driver mutations are shown.

```{r, eval = TRUE, warning = FALSE, message = FALSE, include=TRUE}
data(dataDMA)
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
plotDMA(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        type = 'complete', additionalFilename = "")
```


```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("heatmap_complete.gif")
```


## `plotMoonlight`: Heatmap of Moonlight Gene Z-scores for mutation-driven TSGs/OCGs
In the following plot the top 50 genes with the most driver mutations are visualised.
The values are the moonlight gene z-score for the two biological processes

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
data(dataURA_plot)
plotMoonlight(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        dataURA_plot, gene_type = "drivers", n = 50)
```

```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("moonlight_heatmap.gif")
```

## `plotGMA`: Heatmap of hypo/hyper/dual methylated CpG sites in TSGs/OCGs

This function plots results of the `GMA`. It visualizes the
number of hypo/hyper/dual methylated CpG sites in oncogenic
mediators or in a user-supplied gene list. The results are
visualized either in a single heatmap or split into different
ones which is specified in the function's three modes:
split, complete and genelist.

```{r, eval = TRUE, warning = FALSE, message = FALSE, include = TRUE}
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotGMA(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
        Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
        type = "genelist", genelist = genes,
        additionalFilename = "./GMAresults/")
```

```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("heatmap_genelist_met.gif")
```

## `plotMoonlightMet`: Heatmap of Moonlight Gene Z-scores for methylation-driven TSGs/OCGs

This function visualizes the effect of genes on biological processes
and total number of hypo/hyper/dual methylated CpG sites in genes.

```{r, eval = TRUE, warning = FALSE, message = FALSE, include = TRUE}
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
data("dataURA_plot")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotMoonlightMet(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
                 Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
                 dataURA = dataURA_plot,
                 genes = genes,
                 additionalFilename = "./GMAresults/")
```

```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("moonlight_heatmap_met.gif")
```

## `plotMetExp`: Visualize results from EpiMix of expression and methylation in genes

This function visualizes results of EpiMix [@ref28] in three plots: one that
visualizes the distribution of DNA methylation data (MixtureModelPlot), one
that visualizes gene expression levels (ViolinPlot) and one that visualizes
relationship between DNA methylation and gene expression (CorrelationPlot).

```{r, eval = TRUE, warning = FALSE, message = FALSE, include = TRUE}
data("EpiMix_Results_Regular")
data("dataMethyl")
data("dataFilt")

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

plotMetExp(EpiMix_results = EpiMix_Results_Regular,
           probe_name = "cg03035704",
           dataMET = dataMethyl,
           dataEXP = dataFilt,
           gene_of_interest = "ACVRL1",
           additionalFilename = "./GMAresults/")
```

```{r, fig.width=3, fig.height=4, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("plotMetExp.png")
```


# Moonlight Analysis: Case Studies

### Introduction
This vignette shows a complete workflow of the 'Moonlight2R' package.
The code is divided into the following case studies:

* 1. Predicting oncogenic mediators using Moonlight's primary layer
* 2. Moonlight pipeline in one function
* 3. Moonlight with driver mutation analysis
* 4. Moonlight with gene methylation analysis

## Case study n. 1: Predicting oncogenic mediators using Moonlight's primary layer

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5`
and `nBoot = 5` in the call of `GRN` in the following code block, however, we
recommend setting these parameters to `nGenesPerm = 2000` and `nBoot = 400` to
achieve optimal results, as they are set by default in the function arguments.

```{r,eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE, results='hide'}
data(DEGsmatrix)
data(dataFilt)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)

dataFEA <- FEA(DEGsmatrix = DEGsmatrix)

dataGRN <- GRN(TFs = sample(rownames(DEGsmatrix), 100),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               nBoot = 5,
	       kNearest = 3)

dataURA <- URA(dataGRN = dataGRN,
              DEGsmatrix = DEGsmatrix,
              BPname = c("apoptosis",
                         "proliferation of cells"))

dataDual <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

oncogenic_mediators <- list("TSG"=names(dataDual$TSG), "OCG"=names(dataDual$OCG))

```


## `plotURA`: Upstream regulatory analysis plot
The user can plot the result of the upstream regulatory analysis using the function `plotURA`.

```{r, eval = TRUE,message=FALSE,warning=FALSE, results='hide'}
data(dataURA)
plotURA(dataURA = dataURA, additionalFilename = "_exampleVignette")
```

The figure resulted from the code above is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("URAplot.gif")
```


## Case study n. 2: Moonlight pipeline in one function

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the following code block, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r,eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE}
data(dataFilt)
data(DEGsmatrix)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)

listMoonlight <- moonlight(dataDEGs = DEGsmatrix,
                           dataFilt = dataFilt,
                           nTF = 100,
                           DiffGenes = TRUE,
			   nGenesPerm = 5,
                           nBoot = 5,
                           BPname = c("apoptosis","proliferation of cells"),
                           dataMAF = dataMAF,
                           path_cscape_coding = "css_coding.vcf.gz",
                           path_cscape_noncoding = "css_noncoding.vcf.gz")
save(listMoonlight, file = paste0("listMoonlight_ncancer4.Rdata"))


```

## `plotCircos`: Moonlight Circos Plot
An example of running Moonlight on five cancer types is visualized below in a circos plot.
Outer ring: color by cancer type, Inner ring: OCGs and TSGs,
Inner connections:  green: common OCGs yellow: common TSGs red: possible dual role

```{r, eval = TRUE, echo = TRUE, results='hide', warning = FALSE, message = FALSE}
data(listMoonlight)
plotCircos(listMoonlight = listMoonlight, additionalFilename = "_ncancer5")
```

The figure generated by the code above is shown below:
```{r, fig.width=3, echo = FALSE, fig.align="center",hide=TRUE, message=FALSE,warning=FALSE}
knitr::include_graphics("circos_ocg_tsg_ncancer5.gif")
```


## Case study n. 3: Moonlight with driver mutation analysis

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the call of `GRN` in the following code block, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r,eval = FALSE,echo=TRUE,message=FALSE,warning=FALSE}

data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)

# Perform gene regulatory network analysis
dataGRN <- GRN(TFs = rownames(DEGsmatrix),
	       DEGsmatrix = DEGsmatrix,
	       DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               kNearest = 3,
               nBoot = 5)

# Perform upstream regulatory analysis
# As example, we use apoptosis and proliferation of cells as the biological processes
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               nCores = 1)

# Perform pattern recognition analysis
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

# Perform driver mutation analysis
dataDMA <- DMA(dataMAF = dataMAF,
               dataDEGs = DEGsmatrix,
               dataPRA = dataPRA,
               results_folder = "DMAresults",
               coding_file = "css_coding.vcf.gz",
               noncoding_file = "css_noncoding.vcf.gz")

```

Next, we analyze the predicted driver genes and their mutations.

```{r, eval = TRUE}
data(Oncogenic_mediators_mutation_summary)
data(DEG_Mutations_Annotations)

# Extract oncogenic mediators that contain at least one driver mutation
# These are the driver genes
knitr::kable(Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)))

# Extract mutation annotations of the predicted driver genes
driver_mut <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver")

# Extract driver genes with a predicted effect on the transcriptional level
transcription_mut <- Oncogenic_mediators_mutation_summary %>%
  filter(!is.na(CScape_Driver)) %>%
  filter(Transcription_mut_sum > 0)

# Extract mutation annotations of predicted driver genes that have a driver mutation
# in its promoter region with a potential effect on the transcriptional level
promoters <- DEG_Mutations_Annotations %>%
  filter(!is.na(Moonlight_Oncogenic_Mediator),
         CScape_Mut_Class == "Driver",
         Potential_Effect_on_Transcription == 1,
         Annotation == 'Promoter')
```


## Case study n. 4: Moonlight with gene methylation analysis

For illustrative purposes and to decrease runtime, we have set `nGenesPerm = 5` and `nBoot = 5`
in the call of `GRN` in the following code block, however, we recommend setting these parameters to
`nGenesPerm = 2000` and `nBoot = 400` to achieve optimal results, as they are
set by default in the function arguments.

```{r,eval = TRUE,echo=TRUE,message=FALSE,warning=FALSE,results='hide'}

data(DEGsmatrix)
data(dataFilt)
data(dataMAF)
data(DiseaseList)
data(EAGenes)
data(tabGrowBlock)
data(knownDriverGenes)
data(LOC_transcription)
data(LOC_translation)
data(LOC_protein)
data(NCG)
data(EncodePromoters)
data(dataMethyl)
data(LUAD_sample_anno)
data(MetEvidenceDriver)

# Perform gene regulatory network analysis
dataGRN <- GRN(TFs = rownames(DEGsmatrix),
               DEGsmatrix = DEGsmatrix,
               DiffGenes = TRUE,
               normCounts = dataFilt,
               nGenesPerm = 5,
               kNearest = 3,
               nBoot = 5)

# Perform upstream regulatory analysis
# As example, we use apoptosis and proliferation of cells as the biological processes
dataURA <- URA(dataGRN = dataGRN,
               DEGsmatrix = DEGsmatrix,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               nCores = 1)

# Perform pattern recognition analysis
dataPRA <- PRA(dataURA = dataURA,
               BPname = c("apoptosis",
                          "proliferation of cells"),
               thres.role = 0)

# Subset column names (sample names) in expression data to patient level
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\1", colnames(dataFilt))

# Perform gene methylation analysis
dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt,
               dataPRA = dataPRA, dataDEGs = DEGsmatrix,
               sample_info = LUAD_sample_anno, met_platform = "HM450",
               prevalence_filter = NULL,
               output_dir = "./GMAresults", cores = 1,
               roadmap.epigenome.ids = "E096",
               roadmap.epigenome.groups = NULL)

```


## Citation

Please cite the MoonlightR and Moonlight2R packages:

* "Interpreting pathways to discover cancer driver genes with Moonlight." Nature Communications (2020): [10.1038/s41467-019-13803-0](https://doi.org/10.1038/s41467-019-13803-0). [@ref26]

* "A workflow to study mechanistic indicators for driver gene prediction with Moonlight." Briefings in Bioinformatics (2023): [10.1093/bib/bbad274](https://doi.org/10.1093/bib/bbad274). [@ref27]

Related publications to this vignette:

* "TCGAbiolinks: an R/Bioconductor package for integrative analysis of TCGA data." Nucleic acids research (2015): [gkv1507](http://dx.doi.org/doi:10.1093/nar/gkv1507). [@ref25]

* "TCGA Workflow: Analyze cancer genomics and epigenomics data using Bioconductor packages". F1000Research [10.12688/f1000research.8923.1](http://dx.doi.org/doi:10.12688/f1000research.8923.1) [@ref24]


******

Session Information
******
```{r sessionInfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{tabGrowBlock}
\alias{tabGrowBlock}
\title{Information of growing/blocking characteristics of 101 biological
processes}
\format{
A 101x3 matrix
}
\usage{
data(tabGrowBlock)
}
\value{
A 101x3 matrix
}
\description{
A matrix with biological processes in rows and the cancer #' growing or blocking effect of the process in columns
}
\details{
For each biological processes the cancer growing/blocking 
effect is indicated
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataFEA}
\alias{dataFEA}
\title{Functional enrichment analysis}
\format{
A dataframe of dimension 101x7
}
\usage{
data(dataFEA)
}
\value{
A dataframe of dimension 101x7
}
\description{
The output of the FEA function which does enrichment analysis
}
\details{
The input to the FEA is the differentially expressed genes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataMethyl}
\alias{dataMethyl}
\title{Methylation data matrix from TCGA-LUAD project}
\format{
A 73x27 matrix.
}
\usage{
data(dataMethyl)
}
\value{
A 73x27 matrix.
}
\description{
A data matrix containing methylation data from TCGA-LUAD
where CpG probes are in rows and samples are in columns.
}
\details{
The CpG probes are in rows and samples are in columns.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataMAF}
\alias{dataMAF}
\title{Mutation data from TCGA LUAD}
\format{
A 500x141 matrix.
}
\usage{
data(dataMAF)
}
\value{
A 500x141 matrix.
}
\description{
An examplary MAF file from TCGA on lung cancer LUAD. It contains 500 randomly selected mutations.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotURA.R
\name{plotURA}
\alias{plotURA}
\title{plotURA: Upstream regulatory analysis heatmap plot}
\usage{
plotURA(dataURA, additionalFilename = "URAplot")
}
\arguments{
\item{dataURA}{output URA function}

\item{additionalFilename}{figure name}
}
\value{
heatmap
}
\description{
This function visualizes the URA in a heatmap
}
\examples{
data(dataURA)
data(DiseaseList)
data(tabGrowBlock)
data(knownDriverGenes)
dataDual <- PRA(dataURA = dataURA,
BPname = c("apoptosis","proliferation of cells"),
thres.role = 0)
TSGs_genes <- names(dataDual$TSG)
OCGs_genes <- names(dataDual$OCG)
plotURA(dataURA = dataURA[c(TSGs_genes, OCGs_genes),],additionalFilename = "_example")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RunCscape_somatic.R
\name{RunCscape_somatic}
\alias{RunCscape_somatic}
\title{RunCscape_somatic}
\usage{
RunCscape_somatic(input, coding_file, noncoding_file)
}
\arguments{
\item{input}{Input matching cscape input}

\item{coding_file}{cscape_table with coding scores}

\item{noncoding_file}{cscape_table with noncoding scores}
}
\value{
returns a tibble with a score and remark for each SNP
}
\description{
This function retrive cscape-scores to SNPs
}
\examples{
\donttest{
cscape_out <- RunCscape_somatic(input, coding_file, noncoding_file)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{DEG_Mutations_Annotations}
\alias{DEG_Mutations_Annotations}
\title{Differentially expressed genes's Mutations}
\format{
A 3561x173 matrix.
}
\usage{
data(DEG_Mutations_Annotations)
}
\value{
A 3561x173 matrix.
}
\description{
Output from DMA. 
This contains the differentially expressed genes's mutations and all annotations
generated in DMA() on the TCGA-LUAD project.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{EAGenes}
\alias{EAGenes}
\title{Information about genes}
\format{
A 20038x5 matrix
}
\usage{
data(EAGenes)
}
\value{
A 20038x5 matrix
}
\description{
A matrix containing information about 20038 genes 
including their gene description, location and family
}
\details{
The matrix contains the genes in rows and description, 
location and family in columns.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRN.R
\name{GRN}
\alias{GRN}
\title{Generate network}
\usage{
GRN(
  TFs,
  DEGsmatrix,
  DiffGenes = FALSE,
  normCounts,
  kNearest = 3,
  nGenesPerm = 2000,
  nBoot = 400,
  noise_mi = 1e-12
)
}
\arguments{
\item{TFs}{a vector of genes.}

\item{DEGsmatrix}{DEGsmatrix output from DEA such as dataDEGs}

\item{DiffGenes}{if TRUE consider only diff.expr genes in GRN}

\item{normCounts}{is a matrix of gene expression with genes in rows and samples in columns.}

\item{kNearest}{the number of nearest neighbors to consider to estimate the mutual information.
Must be less than the number of columns of normCounts.}

\item{nGenesPerm}{nGenesPerm}

\item{nBoot}{nBoot}

\item{noise_mi}{noise in knnmi.cross function. Default is 1e-12.}
}
\value{
an adjacent matrix
}
\description{
This function carries out the gene regulatory network inference using parmigene
}
\examples{
data('DEGsmatrix')
data('dataFilt')
dataGRN <- GRN(TFs = sample(rownames(DEGsmatrix), 30),
DEGsmatrix = DEGsmatrix,
DiffGenes = TRUE,
normCounts = dataFilt,
nGenesPerm = 2,
nBoot = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{DEGsmatrix}
\alias{DEGsmatrix}
\title{Differentially expressed genes}
\format{
A 3390x5 matrix
}
\usage{
data(DEGsmatrix)
}
\value{
A 3390x5 matrix
}
\description{
A matrix containing differentially expressed genes between lung cancer and normal
samples found using TCGA-LUAD data and TCGAbiolinks.
}
\details{
The matrix contains the differentially expressed genes in rows and log2 fold change
and FDR values in columns.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{NCG}
\alias{NCG}
\title{Network of Cancer Genes 7.0}
\format{
The format have been rearranged from the original.
<symbold>|<NCG_driver>|<NCG_cgc_annotation>|<NCG_vogelstein_annotation>|
<NCG_saito_annotation>|<NCG_pubmed_id>
}
\source{
\url{http://ncg.kcl.ac.uk/}
}
\usage{
data(NCG)
}
\value{
A 3347x7 table
}
\description{
A dataset retrived from Network of Cancer Genes 7.0
}
\details{
The NCG_driver is reported as a OCG or TSG when at least one of three
three databases have documented it. These are cosmic gene census (cgc),
vogelstein et al. 2013 or saito et al. 2020. The NCG_driver is reported as a
candidate, when literature support the gene as a cancer driver.
}
\references{
Comparative assessment of genes driving cancer and somatic
evolution in non-cancer tissues: an update of the Network of Cancer Genes (NCG)
resource.
Dressler L., Bortolomeazzi M., Keddar M.R., Misetic H., Sartini G.,
Acha-Sagredo A., Montorsi L., Wijewardhane N., Repana D., Nulsen J.,
Goldman J., Pollit M., Davis P., Strange A., Ambrose K. and Ciccarelli F.D.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataMAVISp}
\alias{dataMAVISp}
\title{MAVISp dataset example}
\format{
A list of data frames (183 x 43 and 1180 x 46).
}
\usage{
data(dataMAVISp)
}
\value{
A list of tibbles (183 x 43 and 1180 x 46).
}
\description{
MAVISp results for proteins TP53 and RUNX1, simple mode. They are used
for testing and vignettes. They are stored as a list of tibbles.
}
\details{
The dataframes contain the full available MAVISp data for simple mode at
present time.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LPA.R
\name{LPA}
\alias{LPA}
\title{LPA}
\usage{
LPA(dataDEGs, BP, BPlist)
}
\arguments{
\item{dataDEGs}{is output from DEA}

\item{BP}{is biological process}

\item{BPlist}{is list of genes annotated in BP}
}
\value{
table with number of pubmed that affects, increase or decrase genes annotated in BP
}
\description{
This function carries out the literature phenotype analysis (LPA)
}
\examples{
data('DEGsmatrix')
data('DiseaseList')
BPselected <- c("apoptosis")
BPannotations <- DiseaseList[[match(BPselected, names(DiseaseList))]]$ID
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataGRN_no_noise}
\alias{dataGRN_no_noise}
\title{Gene regulatory network}
\format{
A list of 2 elements where the first element is a 23x613 matrix 
and the second element is a vector of length 23
}
\usage{
data(dataGRN_no_noise)
}
\value{
A list of 2 elements where the first element is a 23x613 matrix and the second element is a vector of length 23
}
\description{
The output of the GRN function which finds connections 
between genes where the noise is set to 0 for testing reproducibility purposes.
}
\details{
The input to the GRN is the differentially expressed genes and the gene expression data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{LUAD_sample_anno}
\alias{LUAD_sample_anno}
\title{Sample annotations of TCGA-LUAD project}
\format{
A 23x2 matrix
}
\usage{
data(LUAD_sample_anno)
}
\value{
A 23x2 matrix
}
\description{
A matrix that annotates LUAD samples as either
cancer or normal
}
\details{
The matrix contains two columns: "primary" which
contains patient barcodes of TCGA-LUAD and "sample.type"
which denotes if the sample is either a "Cancer" or 
"Normal" sample
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCircos.R
\name{plotCircos}
\alias{plotCircos}
\title{plotCircos}
\usage{
plotCircos(
  listMoonlight,
  listMutation = NULL,
  additionalFilename = NULL,
  intensityColOCG = 0.5,
  intensityColTSG = 0.5,
  intensityColDual = 0.5,
  fontSize = 1
)
}
\arguments{
\item{listMoonlight}{output Moonlight function}

\item{listMutation}{listMutation}

\item{additionalFilename}{additionalFilename}

\item{intensityColOCG}{intensityColOCG}

\item{intensityColTSG}{intensityColTSG}

\item{intensityColDual}{intensityColDual}

\item{fontSize}{fontSize}
}
\value{
no return value, plot is saved
}
\description{
This function visualize the plotCircos
}
\examples{
data('listMoonlight')
plotCircos(listMoonlight = listMoonlight, additionalFilename = "_ncancer5")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataGMA}
\alias{dataGMA}
\title{Output example from GMA function}
\format{
A list of length two
}
\usage{
data(dataGMA)
}
\value{
A list of length two
}
\description{
The predicted driver genes based on methylation
evidence
}
\details{
The data is a list of two elements where each 
element represents predicted oncogenes and tumor suppressors
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moonlight.R
\name{moonlight}
\alias{moonlight}
\title{moonlight pipeline}
\usage{
moonlight(
  dataDEGs,
  dataFilt,
  BPname = NULL,
  Genelist = NULL,
  kNearest = 3,
  nGenesPerm = 2000,
  DiffGenes = FALSE,
  nBoot = 400,
  nTF = NULL,
  thres.role = 0,
  dataMAF,
  path_cscape_coding,
  path_cscape_noncoding
)
}
\arguments{
\item{dataDEGs}{table of differentially expressed genes}

\item{dataFilt}{matrix of gene expression data with genes in rows and samples in columns}

\item{BPname}{biological processes to use, if NULL: all processes will be used in analysis, RF for candidate; if not NULL the candidates for these processes will be determined (no learning)}

\item{Genelist}{Genelist}

\item{kNearest}{kNearest}

\item{nGenesPerm}{nGenesPerm}

\item{DiffGenes}{DiffGenes}

\item{nBoot}{nBoot}

\item{nTF}{nTF}

\item{thres.role}{thres.role}

\item{dataMAF}{A MAF file rda object for DMA}

\item{path_cscape_coding}{A character string to path of CScape-somatic coding file}

\item{path_cscape_noncoding}{A character string to path of CScape-somatic non-coding file}
}
\value{
table with cancer driver genes TSG and OCG.
}
\description{
moonlight is a tool for identification of cancer driver genes.
This function wraps the different steps of the complete analysis workflow.
}
\examples{
\donttest{
drivers <- moonlight(dataDEGs = DEGsmatrix,
dataFilt = dataFilt,
BPname = c("apoptosis", "proliferation of cells"),
dataMAF = dataMAF,
path_cscape_coding = "css_coding.vcf.gz",
path_cscape_noncoding = "css_noncoding.vcf.gz")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MAFtoCscape.R
\name{MAFtoCscape}
\alias{MAFtoCscape}
\title{MAFtoCscape}
\usage{
MAFtoCscape(MAF)
}
\arguments{
\item{MAF}{tibble of MAF}
}
\value{
tibble of cscape-somatic input
}
\description{
This function extracts columns from a MAF tibble to fit CScape input format
}
\examples{
data(dataMAF)
MAFtoCscape(dataMAF[seq.int(2),])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{LOC_translation}
\alias{LOC_translation}
\title{Level of Consequence: Translation}
\format{
A 18x7 table
}
\usage{
data(LOC_translation)
}
\value{
A 18x7 table
}
\description{
A dataset describing if a mutation of a certain class and
type possibly have an effect on peptide level.
}
\details{
The values are binary: 0 no effect is possible, 1 an effect is possible.

See supplementary material for details.
}
\references{
paper
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TFinfluence.R
\name{TFinfluence}
\alias{TFinfluence}
\title{TFinfluence}
\usage{
TFinfluence(
  dataPRA,
  dataDEGs,
  dataTRRUST,
  dataMAF,
  dataMAVISp,
  stabClassMAVISp = "rasp"
)
}
\arguments{
\item{dataPRA}{Output PRA function. List of TSG and OCG
Must contain the following elements
\itemize{
\item TSG (The TSGs identified by moonlight along with the moonlight score)
\item OCG (The OCGs identified by moonlight along with the moonlight score)
}}

\item{dataDEGs}{Output DEA function. Tibble containing differentially expressed genes.
Must contain the following columns
\itemize{
\item GENE (HUGO symbol of DEG)
\item logFC (The log fold change of DEG)
}}

\item{dataTRRUST}{Tibble containing gene-TF pairs and type of interaction
Must contain the following columns:
\itemize{
\item TF (HUGO symbol of TF)
\item Target (HUGO symbol of target gene of TF)
\item InteractionType (The effect the TF has on the target gene (Activation, Repression))
}}

\item{dataMAF}{Tibble containing mutation info from MAF
Must at least contain the following columns:
\itemize{
\item Hugo_Symbol eg. BRCA1
\item HGVSp_Short eg. p.V83F
}}

\item{dataMAVISp}{Output loadMAVISp function. List of tibbles, one for each protein.
The tibbles must contain at least
\itemize{
\item (First column must contain the mutation (e.g. A54W). It is assumed that the column name is empty)
}
Then the tibbles must contain columns mathing either of the following names(or they will be excluded) 
\itemize{
\item (Stability classification, [A-Za-z0-9]+, \\(Rosetta, FoldX\\)) (Values: Stabilizing, Neutral, Destabilizing, Uncertain)
\item (Stability classification, [A-Za-z0-9]+, \\(RaSP, FoldX\\)) (Values: Stabilizing, Neutral, Destabilizing, Uncertain)
}}

\item{stabClassMAVISp}{The protocol to use for mutation stability classification. Default is rasp.
Accepts one of the following strings
\itemize{
\item rosetta (uses the FoldX/Rosetta protocol)
\item rasp (uses the FoldX/RaSP protocol)
}}
}
\value{
returns a tibble containing:
\itemize{
\item GENE
\item TF 
\item InteractionType 
\item PMID
\item tf_mutation
\item stab_class (the effect on stability as classified by MAVISp)
}
}
\description{
This function finds mutations in the transcription factors (TF) of the DEGs that have a TF in the database.
}
\examples{

data('dataPRA')
data('DEGsmatrix')
data('dataTRRUST')
data('dataMAF')
data('dataMAVISp')

TFinfluence(dataTRRUST = dataTRRUST,
           dataMAF = dataMAF,
           dataDEGs = DEGsmatrix,
           dataPRA = dataPRA,
           dataMAVISp = dataMAVISp,
           stabClassMAVISp = 'rasp')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotHeatmap.R
\name{plotHeatmap}
\alias{plotHeatmap}
\title{plotHeatmap}
\usage{
plotHeatmap(df)
}
\arguments{
\item{df}{a tibble}
}
\value{
The name of the alphabeatically first gene in the tibble
}
\description{
This function creates a unclustered heatmap from the inputted data tibble
and saves it
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{EncodePromoters}
\alias{EncodePromoters}
\title{Promoters}
\format{
A tibble with no columnnames or rownames.
\enumerate{
\item The first column is chromosome eg. chr1
\item The second column is start position eg. 10451
\item The third column is end position eg. 10563
}
}
\source{
\url{https://www.encodeproject.org/}
}
\usage{
data(EncodePromoters)
}
\value{
A 84738x6 table
}
\description{
Experimentially verified promoter sites by J. Michael Cherry, Stanford.
Downloaded from the ENCODE identifier ENCSR294YNI.
It contains chromosome, start and end sites of promoters.
}
\references{
ENCODE identifier: ENCSR294YNI

Luo Y, Hitz BC, Gabdank I, Hilton JA, Kagda MS, Lam B, Myers Z, Sud P, Jou J,
Lin K, Baymuradov UK, Graham K, Litton C, Miyasato SR, Strattan JS, Jolanki O,
Lee JW, Tanaka FY, Adenekan P, O'Neill E, Cherry JM.
New developments on the Encyclopedia of DNA Elements (ENCODE) data portal.
Nucleic Acids Res. 2020 Jan 8;48(D1):D882-D889. doi: 10.1093/nar/gkz1062.
PMID: 31713622; PMCID: PMC7061942.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GMA.R
\name{GMA}
\alias{GMA}
\title{GMA
This function carries out Gene Methylation Analysis}
\usage{
GMA(
  dataMET,
  dataEXP,
  dataPRA,
  dataDEGs,
  sample_info,
  met_platform = "HM450",
  prevalence_filter = NULL,
  output_dir = "./GMAresults",
  cores = 1,
  roadmap.epigenome.ids = NULL,
  roadmap.epigenome.groups = NULL
)
}
\arguments{
\item{dataMET}{A data matrix containing the methylation data
where the CpG probes are in the rows and samples are in the
columns}

\item{dataEXP}{A data matrix containing the gene expression
data where the genes are in the rows and the samples are in
the columns}

\item{dataPRA}{A table containing the output of the PRA
function}

\item{dataDEGs}{A table containing the output of a DEA
where gene names are rownames}

\item{sample_info}{A table containing information on the
samples. This table needs to contain two columns called
primary and sample.type. The primary column contains
sample names which should be the same as the column names
in dataMET. The sample.type column indicates for each
sample if it is a Cancer or Normal sample.}

\item{met_platform}{A character string representing the
microarray type that was used to collect the methylation
data. This can either be HM27, HM450 or EPIC. Default
is HM450.}

\item{prevalence_filter}{A float or NULL representing if
a prevalence filter should be applied or not. Default is NULL,
meaning a prevalence filter will not be applied. If a float
is specified, a prevalence filter will be applied where
methylation states of probes will be altered depending on the
threshold of prevalence supplied as prevalence_filter. For
example, if prevalence_filter = 20, it means that if the prevalence
of the hyper- or hypomethylated CpG probe exceeds 20, the methylation
state will be unchanged but if the prevalence is lower than 20 the
methylation state will be changed to NA, meaning no methylation state
was detected. In case of dual methylated probes, the methylation
state will stay dual if both the prevalence of hyper- and
hypomethylations exceed 20, but if only one of the prevalences exceed
20 the dual state will be changed to the state exceeding 20. If none
of the prevalences exceed 20, the dual state will be changed to NA.}

\item{output_dir}{Path to where the results will be stored.
If this directory does not exist, it will be created by the
function. Default is ./GMAresults.}

\item{cores}{Number of cores to be used. Default is 1.}

\item{roadmap.epigenome.ids}{A character string representing
the epigenome ID that will be used to select enhancers.
Since enhancers are tissue-specific, the tissue type needs
to be specified in EpiMix. The enhancers are found from the
RoadmapEpigenome project and the IDs can be found from
Figure 2 in the publication with doi: 10.1038/nature14248.
Default is NULL.}

\item{roadmap.epigenome.groups}{A character string representing
the epigenome group that will be used to select enhancers.
Details are provided above with the roadmap.epigenome.ids
parameter. Default is NULL.}
}
\value{
List of two elements, containing predicted oncogenes
and tumor suppressors. Additionally, various output files are
saved in the specified output directory:
DEG_Methylation_Annotations.rda, Oncogenic_mediators_methylation_summary.rda,
EpiMix_Results_Enhancer.rds, EpiMix_Results_Regular.rds,
FunctionalPairs_Enhancer.csv, FunctionalPairs_Regular.csv,
FunctionalProbes_Regular.rds
}
\description{
GMA
This function carries out Gene Methylation Analysis
}
\examples{
data("dataMethyl")
data("dataFilt")
data("dataPRA")
data("DEGsmatrix")
data("LUAD_sample_anno")
data("NCG")
data("EncodePromoters")
data("MetEvidenceDriver")
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
dataFilt_subset <- dataFilt[,1:10]
colnames(dataFilt_subset) <- sub(pattern, "\\\\1", colnames(dataFilt_subset))
dataGMA <- GMA(dataMET = dataMethyl, dataEXP = dataFilt_subset,
dataPRA = dataPRA, dataDEGs = DEGsmatrix,
sample_info = LUAD_sample_anno, met_platform = "HM450",
prevalence_filter = NULL,
output_dir = "./GMAresults", cores = 1, roadmap.epigenome.ids = "E096",
roadmap.epigenome.groups = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{DEG_Methylation_Annotations}
\alias{DEG_Methylation_Annotations}
\title{Output example from GMA function}
\format{
A 3435x35 tibble
}
\usage{
data(DEG_Methylation_Annotations)
}
\value{
A 3435x35 tibble
}
\description{
Output file from running GMA function which
is a summary of DEGs and associated CpG probes
}
\details{
The data is a table where each row is a CpG probe
in a DEG. Various annotations such as start/end site of
CpG probe, promoter/enhancer annotations, NCG annotations
are included in the table.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRAtoTibble.R
\name{PRAtoTibble}
\alias{PRAtoTibble}
\title{PRAtoTibble}
\usage{
PRAtoTibble(dataPRA)
}
\arguments{
\item{dataPRA}{RDA object (list of two) from PRA}
}
\value{
tibble with drivers
}
\description{
This function changes the PRA output to tibble format
}
\examples{
data('dataPRA')
PRAtoTibble(dataPRA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataFilt}
\alias{dataFilt}
\title{Gene expression data from TCGA-LUAD}
\format{
A 3000x20 matrix
}
\usage{
data(dataFilt)
}
\value{
A 3000x20 matrix
}
\description{
A matrix that provides processed gene expression 
data (obtained from RNA seq) from the TCGA-LUAD project
}
\details{
The matrix contains the genes in rows and samples in 
columns. The data has been downloaded and processed using 
TCGAbiolinks.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/URA.R
\name{URA}
\alias{URA}
\title{URA Upstream Regulator Analysis}
\usage{
URA(dataGRN, DEGsmatrix, BPname, nCores = 1)
}
\arguments{
\item{dataGRN}{output GNR function}

\item{DEGsmatrix}{output DPA function}

\item{BPname}{biological processes}

\item{nCores}{number of cores to use}
}
\value{
an adjacent matrix
}
\description{
This function carries out the upstream regulator analysis
}
\examples{
data(DEGsmatrix)
dataDEGs <- DEGsmatrix
data(dataGRN)
data(DiseaseList)
data(EAGenes)
dataURA <- URA(dataGRN = dataGRN,
DEGsmatrix = dataDEGs,
BPname = c("apoptosis",
"proliferation of cells"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataDMA}
\alias{dataDMA}
\title{Output example from the function Driver Mutation Analysis}
\format{
A list of two.
}
\usage{
data(dataDMA)
}
\value{
A list of two, containing 0 tumor-suppressor and 1 oncogene.
}
\description{
The predicted driver genes, which have at least one driver mutation.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{EpiMix_Results_Regular}
\alias{EpiMix_Results_Regular}
\title{Output example from GMA function}
\format{
A list of length nine
}
\usage{
data(EpiMix_Results_Regular)
}
\value{
A list of length nine
}
\description{
The object, a list, that was returned from
running the EpiMix function and is one of the outputs from the 
GMA function.
}
\details{
The data is a list of nine elements which is outputted
from the EpiMix function
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{listMoonlight}
\alias{listMoonlight}
\title{List of oncogenic mediators of 5 TCGA cancer types}
\format{
A list containing 5 elements where each element contains 
differentially expressed genes and output from the URA and PRA 
functions of 5 TCGA cancer types
}
\usage{
data(listMoonlight)
}
\value{
A list containing 5 elements where each element contains 
differentially expressed genes and output from the URA and PRA 
functions of 5 TCGA cancer types
}
\description{
A list of oncogenic mediators of 5 TCGA cancer types: 
BLCA, BRCA, LUAD, READ and STAD
}
\details{
Each element in the list contains differentially expressed 
genes and output from the URA and PRA functions
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataURA}
\alias{dataURA}
\title{Upstream regulator analysis}
\format{
A 23x2 matrix
}
\usage{
data(dataURA)
}
\value{
A 23x2 matrix
}
\description{
The output of the URA function which carries out the 
upstream regulator analysis
}
\details{
The input to URA is the output of GRN and a list of biological 
processes and the differentially expressed genes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FEA.R
\name{FEA}
\alias{FEA}
\title{FEA}
\usage{
FEA(BPname = NULL, DiffMatrix, method = "ora", seed = NULL)
}
\arguments{
\item{BPname}{BPname biological process such as "proliferation of cells", "ALL" (default) if FEA should be carried out for all 101 biological processes}

\item{DiffMatrix}{output from DEA such as dataDEGs or from DAA such as differential abundace matrix}

\item{method}{method to perform Functional Enirchment analysis. Must be either "ora" (Over-Representation Analysis) or "fgsea" (Fast Gene Set Enrichment Analysis)}

\item{seed}{internal seed before the 'fgsea' function is called to ensure reproducibility of the permutation-based p-values. Default is NULL}
}
\value{
matrix from FEA
}
\description{
This function carries out the functional enrichment analysis (FEA).
The analysis can be performed using as input a data matrix from Differential Expression Analysis of genes or Differential Abundance Analysis of proteins.
The analysis can be performed using the ORA (Over Representation Analysis)-based method which implements Fisher's Exact Test or the pre-ranked FCS(Functional Class Scoring)-based method
which implements fgsea (Fast Gene Set Enrichment Analysis).
}
\examples{
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
DEGsmatrix <- DEGsmatrix[seq.int(2), ]
dataFEA <- FEA(DiffMatrix = DEGsmatrix, BPname = "apoptosis")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataGLS}
\alias{dataGLS}
\title{Literature search of driver genes}
\format{
A 13x8 tibble.
}
\usage{
data(dataGLS)
}
\value{
A 13x8 tibble.
}
\description{
A tibble containing results of literature search where predicted
driver genes stored in dataDMA were queried for their role as drivers in
PubMed
}
\details{
The tibble contains PubMed IDs, doi, title, abstract, year of publication,
keywords, and total number of publications for the genes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{DiseaseList}
\alias{DiseaseList}
\title{Cancer-related biological processes}
\format{
A list of 101 elements
}
\usage{
data(DiseaseList)
}
\value{
A list of 101 elements
}
\description{
A dataset containing information about 101 cancer-related biological processes.
}
\details{
The dataset contains a list of the 101 biological processes which includes genes
playing a role in each biological processes including literature findings of the genes' function
in the biological processes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{cscape_somatic_output}
\alias{cscape_somatic_output}
\title{Cscape-somatic annotations of TCGA-LUAD}
\format{
A 645x7 matrix.
}
\usage{
data(cscape_somatic_output)
}
\value{
A 645x7 matrix.
}
\description{
Output from DMA. 
This contains the cscape-somatic annotations for all differentially expressed genes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabix_func.R
\name{tabix_func}
\alias{tabix_func}
\title{tabix_func}
\usage{
tabix_func(Ranges, Reference_Allele, Mutant, file_coding, file_noncoding)
}
\arguments{
\item{Ranges}{The position}

\item{Reference_Allele}{The reference nucleotide}

\item{Mutant}{The mutant nucleotide}

\item{file_coding}{cscape_table with coding scores}

\item{file_noncoding}{cscape_table with noncoding scores}
}
\value{
returns the score
}
\description{
This function retrives the individial score for a SNP
}
\examples{
\donttest{
data <- tabix_func(Ranges, Reference_Allele, Mutant, file_coding, file_noncoding)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EpiMix_getInfiniumAnnotation.R
\name{EpiMix_getInfiniumAnnotation}
\alias{EpiMix_getInfiniumAnnotation}
\title{EpiMix_getInfiniumAnnotation
This function gets Infinium probe annotation from the
sesameData library. This function is from the EpiMix package
https://bioconductor.org/packages/release/bioc/html/EpiMix.html.
Zheng Y, Jun J, Gevaert O (2023). EpiMix: 
EpiMix: an integrative tool for the population-level analysis of 
DNA methylation. R package version 1.1.2.}
\usage{
EpiMix_getInfiniumAnnotation(plat = "EPIC", genome = "hg38")
}
\arguments{
\item{plat}{A character string representing the methylation platform 
which can either be HM27, HM450 or EPIC}

\item{genome}{A character string representing the genome build version
which can either be hg19 or hg38}
}
\value{
Probe annotations
}
\description{
EpiMix_getInfiniumAnnotation
This function gets Infinium probe annotation from the
sesameData library. This function is from the EpiMix package
https://bioconductor.org/packages/release/bioc/html/EpiMix.html.
Zheng Y, Jun J, Gevaert O (2023). EpiMix: 
EpiMix: an integrative tool for the population-level analysis of 
DNA methylation. R package version 1.1.2.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{Oncogenic_mediators_methylation_summary}
\alias{Oncogenic_mediators_methylation_summary}
\title{Output example from GMA function}
\format{
A 25x7 tibble
}
\usage{
data(Oncogenic_mediators_methylation_summary)
}
\value{
A 25x7 tibble
}
\description{
Output file from running the GMA function which
is a summary of the oncogenic mediators and their sum of
methylated CpG probes together with the evidence level of
their role as driver gene.
}
\details{
The data is a table where each row is an oncogenic
mediator and the columns represent the predicted driver role
and the sum of hypo-, hyper-, and dualmethylated CpG sites.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDMA.R
\name{plotDMA}
\alias{plotDMA}
\title{plotDMA}
\usage{
plotDMA(
  DEG_Mutations_Annotations,
  Oncogenic_mediators_mutation_summary,
  type = "split",
  genelist = c(),
  additionalFilename = ""
)
}
\arguments{
\item{DEG_Mutations_Annotations}{A tibble, output file from DMA.}

\item{Oncogenic_mediators_mutation_summary}{A tibble, output file from DMA.}

\item{type}{A character string. It can take the values \code{"split"} or \code{"complete"}.
If both type and genelist are \code{NULL}, the function will default to \code{"split"}.
\itemize{
\item \code{"split"} will split the entire dataset into sections of 40 genes and create individual plots.
These plots will be merged into one pdf. The genes will be sorted alphabeatically.
\item \code{"complete"} will create one plot, though it will not be possible to see the individual gene names.
The heatmap will be clustered hierarchically.
}}

\item{genelist}{A character vector containing HUGO symbols.
A single heatmap will be created with only these genes.
The heatmap will be hierarchically clustered. This will overwrite \code{type}.}

\item{additionalFilename}{A character string. Adds prefix or filepath to the filename of the pdf.}
}
\value{
No return value. DMA results are plotted.
}
\description{
This function creates one or more heatmaps on the output from DMA.
It visualises the CScape-Somatic annotations per oncogenic mediator either
in a single heatmap or split into several different ones.
It is also possible to provide a personalised genelist to visualise.
}
\examples{
data('DEG_Mutations_Annotations')
data('Oncogenic_mediators_mutation_summary')
plotDMA(DEG_Mutations_Annotations,
        Oncogenic_mediators_mutation_summary,
        genelist = c("ACSS2", "AFAP1L1"), additionalFilename = "myplots_")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMetExp.R
\name{plotMetExp}
\alias{plotMetExp}
\title{plotMetExp
This function visualizes results of EpiMix.}
\usage{
plotMetExp(
  EpiMix_results,
  probe_name,
  dataMET,
  dataEXP,
  gene_of_interest,
  additionalFilename = ""
)
}
\arguments{
\item{EpiMix_results}{The object, a list, that was returned from
running the EpiMix function and is one of the outputs from the 
GMA function.}

\item{probe_name}{A character string containing the name of the
CpG probe that will be plotted.}

\item{dataMET}{A data matrix containing the methylation data
where the CpG probes are in the rows and samples are in the
columns}

\item{dataEXP}{A data matrix containing the gene expression
data where the genes are in the rows and the samples are in 
the columns}

\item{gene_of_interest}{A character string containing the 
name of the gene that will be plotted.}

\item{additionalFilename}{A character string that can be used
to add a prefix or filepath to the filename of the pdf 
visualizing the heatmap. Default is an empty string.}
}
\value{
No value is returned. Visualizations are saved.
}
\description{
plotMetExp
This function visualizes results of EpiMix.
}
\examples{
data("EpiMix_Results_Regular")
data("dataMethyl")
data("dataFilt")
pattern <- "^(.{4}-.{2}-.{4}-.{2}).*"
colnames(dataFilt) <- sub(pattern, "\\\\1", colnames(dataFilt))
plotMetExp(EpiMix_results = EpiMix_Results_Regular,
           probe_name = "cg03035704",
           dataMET = dataMethyl,
           dataEXP = dataFilt,
           gene_of_interest = "ACVRL1",
           additionalFilename = "./GMAresults/")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSEA.R
\name{GSEA}
\alias{GSEA}
\title{GSEA}
\usage{
GSEA(DEGsmatrix, top, plot = FALSE)
}
\arguments{
\item{DEGsmatrix}{DEGsmatrix output from DEA such as dataDEGs}

\item{top}{is the number of top BP to plot}

\item{plot}{if TRUE return a GSEA's plot}
}
\value{
return GSEA result
}
\description{
This function carries out the GSEA enrichment analysis.
}
\examples{
data("DEGsmatrix")
DEGsmatrix_example <- DEGsmatrix[1:2,]
dataFEA <- GSEA(DEGsmatrix = DEGsmatrix_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFEA.R
\name{plotFEA}
\alias{plotFEA}
\title{plotFEA}
\usage{
plotFEA(
  dataFEA,
  topBP = 10,
  additionalFilename = NULL,
  height,
  width,
  offsetValue = 5,
  angle = 90,
  xleg = 35,
  yleg = 5,
  titleMain = "",
  minY = -5,
  maxY = 10,
  mycols = c("#8DD3C7", "#FFFFB3", "#BEBADA")
)
}
\arguments{
\item{dataFEA}{dataFEA}

\item{topBP}{topBP}

\item{additionalFilename}{additionalFilename}

\item{height}{Figure height}

\item{width}{Figure width}

\item{offsetValue}{offsetValue}

\item{angle}{angle}

\item{xleg}{xleg}

\item{yleg}{yleg}

\item{titleMain}{title of the plot}

\item{minY}{minY}

\item{maxY}{maxY}

\item{mycols}{colors to use for the plot}
}
\value{
no return value, FEA result is plotted
}
\description{
This function visualize the functional enrichment analysis (FEA)'s barplot
}
\examples{
data(DEGsmatrix)
data(DiseaseList)
data(EAGenes)
data(dataFEA)
plotFEA(dataFEA = dataFEA[1:10,], additionalFilename = "_example",height = 20,width = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{Oncogenic_mediators_mutation_summary}
\alias{Oncogenic_mediators_mutation_summary}
\title{Oncogenic Mediators Mutation Summary}
\format{
A 12x15 matrix.
}
\usage{
data(Oncogenic_mediators_mutation_summary)
}
\value{
A 12x15 matrix.
}
\description{
Output from DMA. 
This contains the oncogenic mediator from the TCGA-LUAD project, and their mutation
assesments summarised based on CSCape-somatic and Level of Consequence.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DMA.R
\name{DMA}
\alias{DMA}
\title{DMA}
\usage{
DMA(
  dataMAF,
  dataDEGs,
  dataPRA,
  runCscape = TRUE,
  coding_file,
  noncoding_file,
  results_folder = "./DMAresults"
)
}
\arguments{
\item{dataMAF}{A MAF file rda object.
The MAF file must at least contain the following columns:
\itemize{
\item Hugo_Symbol eg. BRCA1
\item Chromosome eg. chr1
\item Start_Position eg. 54402
\item End_Position e.g. 54443
\item Strand eg. +
\item Variant_Classification
\item Variant_Type
\item Reference_Allele
\item Tumor_Seq_Allele1
\item Tumor_Seq_Allele2
}}

\item{dataDEGs}{Output DEA function.}

\item{dataPRA}{Output PRA function.}

\item{runCscape}{Bolean. If \code{FALSE} will load CScape output file from results-folder
Default = \code{TRUE}.}

\item{coding_file}{A character string. Path to and name of CScape-somatic coding file.
Can be downloaded at \url{http://cscape-somatic.biocompute.org.uk/#download}. The .tbi file must be placed in the same folder.}

\item{noncoding_file}{A charcter string. Path to and name of CScape-somatic noncoding file.
Can be downloaded at \url{http://cscape-somatic.biocompute.org.uk/#download}. The .tbi file must be placed in the same folder.}

\item{results_folder}{A character string. Path to the results generated by this function.}
}
\value{
List of two, containing TSGs and OCGs with at least one driver mutation. Additionally files are saved in \code{results_folder}.
All output files are in compressed .rda format.
\describe{
\item{DEG_mutations_annotations.rda}{All differentially expressed genes’ mutations and their annotations. These annotations include e.g. Cscape-somatic assessment, Level of Consequence, overlab with promoter sites and information from Network of Cancer Genes (NCG 7.0). All information from MAF and DEA is contained.}
\item{Oncogenic_mediators_annotation_summary.rda}{All oncogenic mediators and an summarisation of their mutation based on CScape-somatic assessment, Level of Consequences and total number of mutations. If a gene as previously been assessed as a driver in Network of Cancer Genes (7.0), it is annotated in a separate column. }
\item{Cscape_somatic_output.rda}{The file contain the cscape-somatic assessment for every mutation found in the differentially expressed genes. It is formatted exactly as the output of cscape-somatic, as if it was run in the terminal, except it is saved as .rda instead of csv. }
}
}
\description{
This function carries out the driver mutation analysis.
}
\details{
For more information about the different annotations added to the mutations please see the documentation as followes:
\code{data(NCG)}, \code{data(EncodePromoters)}, \code{data(LOC_protein)}
\code{data(LOC_transcription)} and \code{data(LOC_translation)}.
}
\examples{
\donttest{
DMA(dataMAF = dataMAF,
    dataDEGs = DEGsmatrix,
    dataPRA = dataPRA,
    coding_file = "path/css_coding.vcf.gz",
    noncoding_file = "path/css_noncoding.vcf.gz",
    results_folder = "path/results")

#If the cscape-somatic file have already been created
cscape_somatic_output <- read.csv("./results/Cscape_somatic_output.csv")
save(cscape_somatic_output, file = "./results/Cscape_somatic_output.rda")

DMA(dataMAF = dataMAF,
    dataDEGs = DEGsmatrix,
    dataPRA = dataPRA,
    runCscape = FALSE,
    results_folder = "./results")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confidence.R
\name{confidence}
\alias{confidence}
\title{confidence}
\usage{
confidence(s, type)
}
\arguments{
\item{s}{the score}

\item{type}{coding or noncoding}
}
\value{
returns a confidence level or remark/error message
}
\description{
This function annotated a confidence level to the score
}
\examples{

remark <- confidence(0.8, type='Coding')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMoonlight.R
\name{plotMoonlight}
\alias{plotMoonlight}
\title{plotMoonlight}
\usage{
plotMoonlight(
  DEG_Mutations_Annotations,
  Oncogenic_mediators_mutation_summary,
  dataURA,
  gene_type = "drivers",
  n = 50,
  genelist = c(),
  BPlist = c(),
  additionalFilename = ""
)
}
\arguments{
\item{DEG_Mutations_Annotations}{A tibble, output file from DMA.}

\item{Oncogenic_mediators_mutation_summary}{A tibble, output file from DMA.}

\item{dataURA}{Output URA function.}

\item{gene_type}{A character string either \code{"mediators"} or \code{"drivers"}.
\itemize{
\item If \code{NULL} defaults to \code{"drivers"}.
\item \code{"mediators"} will show the oncogenic mediators with the highst number of mutations regardless of driver/passenger classification.
\item \code{"drivers"} will show the driver genes with the highest number of driver mutations.
}}

\item{n}{Numeric. The top number of genes to be plotted. If \code{NULL} defaults to 50.}

\item{genelist}{A vector of strings containing Hugo Symbols of genes.
Overwrites \code{gene_type} argument.}

\item{BPlist}{A vector of strings. Selection of the biological processes to visualise.
If left empty defaults to every BP provided in the URA file.}

\item{additionalFilename}{A character string. Adds prefix or filepath to the filename of the pdf.}
}
\value{
No return value. Moonlight scores are plotted for selected genes.
}
\description{
This function creates a heatmap of Moonlight gene z-scores for selected genes.
}
\examples{
data(DEG_Mutations_Annotations)
data(Oncogenic_mediators_mutation_summary)
data(dataURA_plot)
plotMoonlight(DEG_Mutations_Annotations,
              Oncogenic_mediators_mutation_summary,
              dataURA_plot, genelist = c("AFAP1L1", "ABCG2"),
              additionalFilename = "myplot_")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRA.R
\name{PRA}
\alias{PRA}
\title{Pattern Recognition Analysis (PRA)}
\usage{
PRA(dataURA, BPname, thres.role = 0)
}
\arguments{
\item{dataURA}{output URA function}

\item{BPname}{BPname}

\item{thres.role}{thres.role}
}
\value{
returns list of TSGs and OCGs when biological processes are provided, otherwise a randomForest based classifier that can be used on new data
}
\description{
This function carries out the pattern recognition analysis
}
\examples{
data(dataURA)
data(DiseaseList)
data(tabGrowBlock)
data(knownDriverGenes)
dataPRA <- PRA(dataURA = dataURA[seq.int(2),],
BPname = c("apoptosis","proliferation of cells"),
thres.role = 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{GEO_TCGAtab}
\alias{GEO_TCGAtab}
\title{Information on GEO and TCGA data}
\format{
A 18x12 matrix
}
\usage{
data(GEO_TCGAtab)
}
\value{
A 18x12 matrix
}
\description{
A matrix that provides the GEO dataset
  matched to one of 18 TCGA cancer types
}
\details{
The matrix contains the cancer types in rows and 
information about sample type from both TCGA and GEO in columns.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{DAPsmatrix}
\alias{DAPsmatrix}
\title{Differentially abundant proteins}
\format{
A 4873x4 matrix
}
\usage{
data(DAPsmatrix)
}
\value{
A 4873x4 matrix
}
\description{
A matrix containing differentially abundant proteins between colon adenocarinoma cancer and normal
samples found using CPTAC data portal.
}
\details{
The matrix contains the differentially abundant protein with gene names in rows and log2 fold change
PValues and  Direction of abundance change values in columns.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{knownDriverGenes}
\alias{knownDriverGenes}
\title{Information of known cancer driver genes from COSMIC}
\format{
A list containing two elements where the first element is a 
character vector of 55 and the second element is a character vector of #' 84
}
\usage{
data(knownDriverGenes)
}
\value{
A list containing two elements where the first element is a 
character vector of 55 and the second element is a character vector of #' 84
}
\description{
A list of known cancer driver genes from COSMIC
}
\details{
The list contains two elements: a vector of known tumor #' suppressors and a vector of known oncogenes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGMA.R
\name{plotGMA}
\alias{plotGMA}
\title{plotGMA
This function plots results of the Gene Methylation Analysis.
It visualizes the number of hypo/hyper/dual methylated CpG
sites in oncogenic mediators or in a user-supplied gene list.
The results are visualized either in a single heatmap or split
into different ones which is specified in the function's three
modes: split, complete and genelist.}
\usage{
plotGMA(
  DEG_Methylation_Annotations,
  Oncogenic_mediators_methylation_summary,
  type = "split",
  genelist = NULL,
  additionalFilename = ""
)
}
\arguments{
\item{DEG_Methylation_Annotations}{A tibble which is outputted
from the GMA function.}

\item{Oncogenic_mediators_methylation_summary}{A tibble which
is outputted from the GMA function.}

\item{type}{A character string which can either be split,
complete or genelist. If type is set to split, the entire
dataset is split into groups of 40 genes and individual
heatmaps of groups each containing 40 genes will be created and
subsequently merged into one pdf where each page in the pdf
is an individual heatmap. The genes will be sorted alphabetically.
If type is set to complete, a single heatmap is created where
the number of differentially methylated CpG sites are shown for
all oncogenic mediators. If type is set to genelist, a single
heatmap will be created for genes supplied by the user in the
genelist parameter. Default is split.}

\item{genelist}{A character string containing HUGO symbols
of genes to be visualized in a single heatmap. Default is NULL.}

\item{additionalFilename}{A character string that can be used
to add a prefix or filepath to the filename of the pdf
visualizing the heatmap. Default is an empty string.}
}
\value{
No value is returned. Visualizations in form of heatmaps
are saved.
}
\description{
plotGMA
This function plots results of the Gene Methylation Analysis.
It visualizes the number of hypo/hyper/dual methylated CpG
sites in oncogenic mediators or in a user-supplied gene list.
The results are visualized either in a single heatmap or split
into different ones which is specified in the function's three
modes: split, complete and genelist.
}
\examples{
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotGMA(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
        Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
        type = "genelist", genelist = genes,
        additionalFilename = "./GMAresults/")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataFEA_fgsea}
\alias{dataFEA_fgsea}
\title{Functional enrichment analysis with fgsea method}
\format{
A dataframe of dimension 101x8
}
\usage{
data(dataFEA_fgsea)
}
\value{
A dataframe of dimension 101x8
}
\description{
The output of the FEA function which does enrichment analysis using fgsea method
}
\details{
The input to the FEA is the differentially expressed genes.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GLS.R
\name{GLS}
\alias{GLS}
\title{GLS
This function carries out gene literature search.}
\usage{
GLS(genes, query_string = "AND cancer AND driver")
}
\arguments{
\item{genes}{A character string containing the genes
to search in PubMed database}

\item{query_string}{A character string containing words
in query to follow the gene of interest. Default is
"AND cancer AND driver" resulting in a final query of
"Gene AND cancer AND driver". Standard PubMed syntax
can be used in the query. For example Boolean operators
AND, OR, NOT can be applied and tags such as [AU],
[TITLE/ABSTRACT], [Affiliation] can be used.}
}
\value{
A tibble containing results of literature search
where PubMed was queried for information of input genes.
Each row in the tibble contains a PubMed ID matching the
query, doi, title, abstract, year of publication, mesh_terms,
and total number of PubMed publications, resulting in a
total of eight columns.
}
\description{
GLS
This function carries out gene literature search.
}
\examples{
genes_query <- "ABCG2"
dataGLS <- GLS(genes = genes_query,
              query_string = "AND cancer AND driver AND
              '2020/01/01'[Date - Publication] : '2025/01/01'[Date - Publication]")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataURA_plot}
\alias{dataURA_plot}
\title{Upstream regulator analysis}
\format{
A 12x2 matrix
}
\usage{
data(dataURA_plot)
}
\value{
A 12x2 matrix
}
\description{
The output of the URA function which carries out the 
upstream regulator analysis
}
\details{
This URA data is used to showcase some of the visualization
functions
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataPRA}
\alias{dataPRA}
\title{Output example from function Pattern Recognition Analysis}
\format{
A list of two.
}
\usage{
data(dataPRA)
}
\value{
A list of two.
}
\description{
The predicted TSGs and OCGs and their moonlight gene z-score based on the small sample TCGA-LUAD data.
The PRA() were run with expert-based approach with apoptosis and proliferation of cells.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMoonlightMet.R
\name{plotMoonlightMet}
\alias{plotMoonlightMet}
\title{plotMoonlightMet
This function visualizes the effect of genes on biological processes
and total number of hypo/hyper/dual methylated CpG sites in genes.}
\usage{
plotMoonlightMet(
  DEG_Methylation_Annotations,
  Oncogenic_mediators_methylation_summary,
  dataURA,
  genes,
  additionalFilename = ""
)
}
\arguments{
\item{DEG_Methylation_Annotations}{A tibble which is outputted
from the GMA function.}

\item{Oncogenic_mediators_methylation_summary}{A tibble which
is outputted from the GMA function.}

\item{dataURA}{Output of the URA function: a table containing
the effect of oncogenic mediators on biological processes. This
effect is quantified through Moonlight Gene Z-scores.}

\item{genes}{A character string containing the genes to be
visualized.}

\item{additionalFilename}{A character string that can be used
to add a prefix or filepath to the filename of the pdf
visualizing the heatmap. Default is an empty string.}
}
\value{
No value is returned. Visualization in form of a heatmap
is saved.
}
\description{
plotMoonlightMet
This function visualizes the effect of genes on biological processes
and total number of hypo/hyper/dual methylated CpG sites in genes.
}
\examples{
data("DEG_Methylation_Annotations")
data("Oncogenic_mediators_methylation_summary")
data("dataURA_plot")
genes <- c("ACAN", "ACE2", "ADAM19", "AFAP1L1")
plotMoonlightMet(DEG_Methylation_Annotations = DEG_Methylation_Annotations,
                 Oncogenic_mediators_methylation_summary = Oncogenic_mediators_methylation_summary,
                 dataURA = dataURA_plot,
                 genes = genes,
                 additionalFilename = "./GMAresults/")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadMAVISp.R
\name{loadMAVISp}
\alias{loadMAVISp}
\title{loadMAVISp}
\usage{
loadMAVISp(
  mavispDB = NULL,
  proteins_of_interest = NULL,
  mode = "simple",
  ensemble = "md"
)
}
\arguments{
\item{mavispDB}{path to the MAVISp database as a string. The database can be downloaded from an OSF repository (https://osf.io/ufpzm/) and has the following structure.}

\item{proteins_of_interest}{vector containing specific proteins of interest in HUGO format}

\item{mode}{string determining whether to use simple or ensemble mode of mavisp. Default is simple mode.
Takes values: 
\itemize{
\item simple
\item ensemble
}}

\item{ensemble}{string specifying the ensemble to use. The available ensembles  can be found in the MAVISp database. This is ignored for simple mode.}
}
\value{
returns a list of tibbles each containing the MAVISp entry of one protein. Each entry will contain an extra column specifying what data
the stability classification is based on.
}
\description{
This function loads the MAVISp database from the directory specified by the user.
}
\examples{

mavisp_db_location <- system.file('extdata', 'mavisp_db', package='Moonlight2R')

mavisp_data <- loadMAVISp(mavispDB = mavisp_db_location,
                          proteins_of_interest = c('TP53'),
                          mode = 'ensemble')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotNetworkHive.R
\name{plotNetworkHive}
\alias{plotNetworkHive}
\title{plotNetworkHive: Hive network plot}
\usage{
plotNetworkHive(dataGRN, namesGenes, thres, additionalFilename = NULL)
}
\arguments{
\item{dataGRN}{output GRN function}

\item{namesGenes}{list TSG and OCG to define axes}

\item{thres}{threshold of edges to be included}

\item{additionalFilename}{additionalFilename}
}
\value{
no results Hive plot is executed
}
\description{
This function visualizes the GRN as a hive plot
}
\examples{
data(knownDriverGenes)
data(dataGRN)
plotNetworkHive(dataGRN = dataGRN, namesGenes = knownDriverGenes, thres = 0.55)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{MetEvidenceDriver}
\alias{MetEvidenceDriver}
\title{Methylation evidence table to define driver genes}
\format{
A 30x6 tibble.
}
\usage{
data(MetEvidenceDriver)
}
\value{
A 30x6 tibble.
}
\description{
A tibble containing combinations of methylation states
of probes used to define driver genes
}
\details{
The tibble contains a value of 1 if a probe is found that
is either hypo-, hyper-, dualmethylated or not methylated. This is
compared with Moonlight's predictions of role of oncogenic mediators
to define driver genes based on methylation evidence.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LiftMAF.R
\name{LiftMAF}
\alias{LiftMAF}
\title{LiftMAF}
\usage{
LiftMAF(Infile, Current_Build)
}
\arguments{
\item{Infile}{A tibble of MAF.}

\item{Current_Build}{A charcter string, either \code{GRCh38} or \code{GRCh37}}
}
\value{
MAF tibble with positions lifted to another build
}
\description{
This function lifts a MAF file to a different genomic build.
}
\examples{
data(dataMAF)
dataMAF_example <- dataMAF[1,]
LiftMAF(dataMAF_example, Current_Build = 'GRCh38')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDataGEO.R
\name{getDataGEO}
\alias{getDataGEO}
\title{getDataGEO}
\usage{
getDataGEO(GEOobject = "GSE39004", platform = "GPL6244", TCGAtumor = NULL)
}
\arguments{
\item{GEOobject}{GEOobject}

\item{platform}{platform}

\item{TCGAtumor}{tumor name}
}
\value{
return GEO gset
}
\description{
This function retrieves and prepares GEO data
}
\examples{
data(GEO_TCGAtab)
dataGEO <-  getDataGEO(GEOobject = "GSE15641", platform = "GPL96")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rdata.R
\docType{data}
\name{dataGRN}
\alias{dataGRN}
\title{Gene regulatory network}
\format{
A list of 2 elements where the first element is a 23x613 matrix 
and the second element is a vector of length 23
}
\usage{
data(dataGRN)
}
\value{
A list of 2 elements where the first element is a 23x613 matrix and the second element is a vector of length 23
}
\description{
The output of the GRN function which finds connections 
between genes.
}
\details{
The input to the GRN is the differentially expressed genes and the gene expression data.
}
\keyword{datasets}
